@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -143,7 +143,7 @@ static void about_create(void)
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
 	label = gtk_label_new
-		("Copyright (C) 1999,2000 Hiroyuki Yamamoto <hiro-y@kcn.ne.jp>");
+		("Copyright (C) 1999-2001 Hiroyuki Yamamoto <hiro-y@kcn.ne.jp>");
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
 	hbox = gtk_hbox_new(FALSE, 0);
@@ -180,22 +180,23 @@ static void about_create(void)
 			    (GTK_SCROLLED_WINDOW(scrolledwin)),
 			    gtk_scrolled_window_get_vadjustment
 			    (GTK_SCROLLED_WINDOW(scrolledwin)));
+	gtk_text_set_word_wrap(GTK_TEXT(text), TRUE);
 	gtk_container_add(GTK_CONTAINER(scrolledwin), text);
 
 	gtk_text_freeze(GTK_TEXT(text));
 
 	gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL,
-		_("The portions applied from fetchmail is Copyright 1997 by Eric S.\n"
-		  "Raymond.  Portions of those are also copyrighted by Carl Harris,\n"
-		  "1993 and 1995.  Copyright retained for the purpose of protecting free\n"
+		_("The portions applied from fetchmail is Copyright 1997 by Eric S. "
+		  "Raymond.  Portions of those are also copyrighted by Carl Harris, "
+		  "1993 and 1995.  Copyright retained for the purpose of protecting free "
 		  "redistribution of source.\n\n"), -1);
 
 	gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL,
-		_("The MD5 support is copyright by RSA Data Security, Inc.  See the\n"
+		_("The MD5 support is copyright by RSA Data Security, Inc.  See the "
 		  "header comment of the md5.c module for license terms.\n\n"), -1);
 
 	gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL,
-		_("Kcc is copyright by Yasuhiro Tonooka <tonooka@msi.co.jp>,\n"
+		_("Kcc is copyright by Yasuhiro Tonooka <tonooka@msi.co.jp>, "
 		  "and libkcc is copyright by takeshi@SoftAgency.co.jp.\n\n"), -1);
 
 #if USE_GPGME
@@ -204,21 +205,21 @@ static void about_create(void)
 #endif /* USE_GPGME */
 
 	gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL,
-		_("This program is free software; you can redistribute it and/or modify\n"
-		  "it under the terms of the GNU General Public License as published by\n"
-		  "the Free Software Foundation; either version 2, or (at your option)\n"
+		_("This program is free software; you can redistribute it and/or modify "
+		  "it under the terms of the GNU General Public License as published by "
+		  "the Free Software Foundation; either version 2, or (at your option) "
 		  "any later version.\n\n"), -1);
 
 	gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL,
-		_("This program is distributed in the hope that it will be useful,\n"
-		  "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-		  "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
+		_("This program is distributed in the hope that it will be useful, "
+		  "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+		  "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. "
 		  "See the GNU General Public License for more details.\n\n"), -1);
 
 	gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL,
-		_("You should have received a copy of the GNU General Public License\n"
-		  "along with this program; if not, write to the Free Software\n"
-		  "Foundation, Inc., 59 Temple Place - Suite 330, Boston,\n"
+		_("You should have received a copy of the GNU General Public License "
+		  "along with this program; if not, write to the Free Software "
+		  "Foundation, Inc., 59 Temple Place - Suite 330, Boston, "
 		  "MA 02111-1307, USA."), -1);
 
 	gtk_text_thaw(GTK_TEXT(text));
