@@ -1531,8 +1531,8 @@ static void addressbook_read_file(void)
 
 	xml_get_dtd(file);
 
-	xml_parse_next_tag(file);
-	if (!xml_compare_tag(file, "addressbook")) {
+	if (xml_parse_next_tag(file) < 0 ||
+	    xml_compare_tag(file, "addressbook") == FALSE) {
 		g_warning("Invalid addressbook data\n");
 		xml_close_file(file);
 		return;
@@ -1560,8 +1560,8 @@ static void addressbook_get_tree(XMLFile *file, GtkCTreeNode *node,
 	gtk_ctree_node_set_row_data(GTK_CTREE(addrbook.ctree), node, folder);
 
 	if (file) {
-		xml_parse_next_tag(file);
-		if (!xml_compare_tag(file, folder_tag)) {
+		if (xml_parse_next_tag(file) < 0 ||
+		    xml_compare_tag(file, folder_tag) == FALSE) {
 			g_warning("Invalid addressbook data\n");
 			return;
 		}
@@ -1578,7 +1578,7 @@ static void addressbook_add_objs(XMLFile *file, GtkCTreeNode *node)
 
 	for (;;) {
 		prev_level = file->level;
-		xml_parse_next_tag(file);
+		if (xml_parse_next_tag(file) < 0) return;
 		if (file->level < prev_level) return;
 
 		if (xml_compare_tag(file, "group")) {
@@ -1786,9 +1786,9 @@ static AddressItem *addressbook_parse_item(XMLFile *file)
 
 	level = file->level;
 
-	for (;;) {
-		xml_parse_next_tag(file);
+	while (xml_parse_next_tag(file) == 0) {
 		if (file->level < level) return item;
+		if (file->level == level) break;
 
 		element = xml_get_element(file);
 
@@ -1800,8 +1800,8 @@ static AddressItem *addressbook_parse_item(XMLFile *file)
 			item->remarks = element;
 		}
 
-		xml_parse_next_tag(file);
-		if (file->level != level) PARSE_ITEM_ERROR();
+		if (xml_parse_next_tag(file) < 0) break;
+		if (file->level != level) break;
 	}
 
 	PARSE_ITEM_ERROR();
@@ -1876,13 +1876,17 @@ static void addressbook_node_write_begin(GtkCTreeNode *node, FILE *fp)
 			fprintf(fp, "<%s>\n", folder->name);
 		} else {
 			tab_indent_out(fp, GTK_CTREE_ROW(node)->level - 1);
-			fprintf(fp, "<folder name=\"%s\">\n", folder->name);
+			fputs("<folder name=\"", fp);
+			xml_file_put_escape_str(fp, folder->name);
+			fputs("\">\n", fp);
 		}
 	} else if (obj->type == ADDR_GROUP) {
 		AddressGroup *group = ADDRESS_GROUP(obj);
 
 		tab_indent_out(fp, GTK_CTREE_ROW(node)->level - 1);
-		fprintf(fp, "<group name=\"%s\">\n", group->name);
+		fputs("<group name=\"", fp);
+		xml_file_put_escape_str(fp, group->name);
+		fputs("\">\n", fp);
 	}
 }
 
@@ -1926,15 +1930,22 @@ static void addressbook_write_items(FILE *fp, GList *items, guint level)
 
 			tab_indent_out(fp, level);
 			fputs("<item>\n", fp);
+
 			tab_indent_out(fp, level + 1);
-			fprintf(fp, "<name>%s</name>\n",
-				item->name ? item->name : "");
+			fputs("<name>", fp);
+			xml_file_put_escape_str(fp, item->name);
+			fputs("</name>\n", fp);
+
 			tab_indent_out(fp, level + 1);
-			fprintf(fp, "<address>%s</address>\n",
-				item->address ? item->address : "");
+			fputs("<address>", fp);
+			xml_file_put_escape_str(fp, item->address);
+			fputs("</address>\n", fp);
+
 			tab_indent_out(fp, level + 1);
-			fprintf(fp, "<remarks>%s</remarks>\n",
-				item->remarks ? item->remarks : "");
+			fputs("<remarks>", fp);
+			xml_file_put_escape_str(fp, item->remarks);
+			fputs("</remarks>\n", fp);
+
 			tab_indent_out(fp, level);
 			fputs("</item>\n", fp);
 		}
@@ -1946,7 +1957,7 @@ static void tab_indent_out(FILE *fp, guint level)
 	gint i;
 
 	for (i = 0; i < level; i++)
-		fputc('\t', fp);
+		fputs("    ", fp);
 }
 
 static void key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data)
