@@ -1,4 +1,21 @@
-/*  alertpanel.c  -  show alert panel  */
+/*
+ * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
+ * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -14,9 +31,9 @@
 #include "gtkutils.h"
 
 #define TITLE_FONT	"-*-helvetica-medium-r-normal--17-*-*-*-*-*-*-*," \
-			"-*-gothic-medium-r-normal--16-*-*-*-*-*-*-*,*"
+			"-*-*-medium-r-normal--16-*-*-*-*-*-*-*,*"
 #define MESSAGE_FONT	"-*-helvetica-medium-r-normal--14-*-*-*-*-*-*-*," \
-			"-*-gothic-medium-r-normal--14-*-*-*-*-*-*-*,*"
+			"-*-*-medium-r-normal--14-*-*-*-*-*-*-*,*"
 #define ALERT_PANEL_WIDTH	380
 #define TITLE_HEIGHT		72
 #define MESSAGE_HEIGHT		62
@@ -26,14 +43,17 @@ static AlertValue value;
 
 static GtkWidget *dialog;
 
-static void alertpanel_create(const gchar *title,
-			      const gchar *message,
-			      const gchar *button1_label,
-			      const gchar *button2_label,
-			      const gchar *button3_label);
-static void alertpanel_button_clicked(GtkWidget *widget, gpointer data);
-static void alertpanel_close(GtkWidget *widget, GdkEventAny *event,
-			     gpointer data);
+static void alertpanel_show		(void);
+static void alertpanel_create		(const gchar	*title,
+					 const gchar	*message,
+					 const gchar	*button1_label,
+					 const gchar	*button2_label,
+					 const gchar	*button3_label);
+static void alertpanel_button_clicked	(GtkWidget	*widget,
+					 gpointer	 data);
+static void alertpanel_close		(GtkWidget	*widget,
+					 GdkEventAny	*event,
+					 gpointer	 data);
 
 AlertValue alertpanel(const gchar *title,
 		      const gchar *message,
@@ -48,7 +68,64 @@ AlertValue alertpanel(const gchar *title,
 
 	alertpanel_create(title, message, button1_label, button2_label,
 			  button3_label);
+	alertpanel_show();
 
+	debug_print("return value = %d\n", value);
+	return value;
+}
+
+void alertpanel_message(const gchar *title, const gchar *message)
+{
+	if (alertpanel_is_open)
+		return;
+	else
+		alertpanel_is_open = TRUE;
+
+	alertpanel_create(title, message, NULL, NULL, NULL);
+	alertpanel_show();
+}
+
+void alertpanel_notice(const gchar *format, ...)
+{
+	va_list args;
+	gchar buf[256];
+
+	va_start(args, format);
+	g_vsnprintf(buf, sizeof(buf), format, args);
+	va_end(args);
+	strretchomp(buf);
+
+	alertpanel_message(_("Notice"), buf);
+}
+
+void alertpanel_warning(const gchar *format, ...)
+{
+	va_list args;
+	gchar buf[256];
+
+	va_start(args, format);
+	g_vsnprintf(buf, sizeof(buf), format, args);
+	va_end(args);
+	strretchomp(buf);
+
+	alertpanel_message(_("Warning"), buf);
+}
+
+void alertpanel_error(const gchar *format, ...)
+{
+	va_list args;
+	gchar buf[256];
+
+	va_start(args, format);
+	g_vsnprintf(buf, sizeof(buf), format, args);
+	va_end(args);
+	strretchomp(buf);
+
+	alertpanel_message(_("Error"), buf);
+}
+
+static void alertpanel_show(void)
+{
 	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
 	manage_window_set_transient(GTK_WINDOW(dialog));
 	value = G_ALERTWAIT;
@@ -57,11 +134,9 @@ AlertValue alertpanel(const gchar *title,
 		gtk_main_iteration();
 
 	gtk_widget_destroy(dialog);
+	GTK_EVENTS_FLUSH();
 
 	alertpanel_is_open = FALSE;
-
-	debug_print("return value = %d\n", value);
-	return value;
 }
 
 static void alertpanel_create(const gchar *title,
@@ -71,7 +146,6 @@ static void alertpanel_create(const gchar *title,
 			      const gchar *button3_label)
 {
 	static GdkFont *titlefont;
-	static GdkFont *messagefont;
 	GtkStyle *style;
 	GtkWidget *label;
 	GtkWidget *hbox;
@@ -80,11 +154,14 @@ static void alertpanel_create(const gchar *title,
 	GtkWidget *button1;
 	GtkWidget *button2;
 	GtkWidget *button3;
+	const gchar *label2;
+	const gchar *label3;
+
+	debug_print(_("Creating alert panel dialog...\n"));
 
 	dialog = gtk_dialog_new();
 	gtk_window_set_title(GTK_WINDOW(dialog), title);
 	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, FALSE);
-	gtk_widget_set_usize(dialog, ALERT_PANEL_WIDTH, -1);
 	gtk_container_set_border_width
 		(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), 5);
 	gtk_window_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
@@ -99,8 +176,7 @@ static void alertpanel_create(const gchar *title,
 	/* for title label */
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
-			   hbox, TRUE, TRUE, 0);
-	gtk_widget_set_usize(hbox, -1, TITLE_HEIGHT);
+			   hbox, TRUE, TRUE, 16);
 
 	/* title label */
 	//pixmapwid = create_pixmapwid(dialog, GNUstep_xpm);
@@ -121,31 +197,37 @@ static void alertpanel_create(const gchar *title,
 
 	/* for message label */
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-	gtk_widget_set_usize(hbox, -1, MESSAGE_HEIGHT);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 20);
 
 	/* message label */
 	label = gtk_label_new(message);
-	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
-	style = gtk_style_copy(gtk_widget_get_style(label));
-	if (!messagefont)
-		messagefont = gdk_fontset_load(MESSAGE_FONT);
-	if (messagefont)
-		style->font = messagefont;
-	gtk_widget_set_style(label, style);
+	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 32);
+	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
 
 	/* for button(s) */
 	if (!button1_label)
 		button1_label = _("OK");
+	label2 = button2_label;
+	label3 = button3_label;
+	if (label2 && *label2 == '+') label2++;
+	if (label3 && *label3 == '+') label3++;
 
 	gtkut_button_set_create(&confirm_area,
 				&button1, button1_label,
-				button2_label ? &button2 : NULL, button2_label,
-				button3_label ? &button3 : NULL, button3_label);
+				button2_label ? &button2 : NULL, label2,
+				button3_label ? &button3 : NULL, label3);
 
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(button1);
 	gtk_widget_grab_focus(button1);
+	if (button2_label && *button2_label == '+') {
+		gtk_widget_grab_default(button2);
+		gtk_widget_grab_focus(button2);
+	}
+	if (button3_label && *button3_label == '+') {
+		gtk_widget_grab_default(button3);
+		gtk_widget_grab_focus(button3);
+	}
 
 	gtk_signal_connect(GTK_OBJECT(button1), "clicked",
 			   GTK_SIGNAL_FUNC(alertpanel_button_clicked),
