@@ -430,34 +430,6 @@ CodeConvFunc conv_get_code_conv_func(const gchar *charset)
 	return code_conv;
 }
 
-void conv_unmime_header(gchar *outbuf, gint outlen, const gchar *str,
-			const gchar *charset)
-{
-	gchar *buf;
-	CharSet cur_charset;
-
-	Xstrdup_a(buf, str, {return;});
-	cur_charset = conv_get_current_charset();
-
-#if HAVE_LIBJCONV
-	UnMimeHeaderConv(buf, outbuf, outlen);
-	if (cur_charset == C_EUC_JP)
-		conv_unreadable_eucjp(outbuf);
-#else
-	UnMimeHeader(buf);
-	if (cur_charset == C_EUC_JP) {
-		gchar *tmp;
-		gint len;
-
-		len = strlen(buf) * 2 + 1;
-		Xalloca(tmp, len, {strncpy2(outbuf, buf, outlen); return;});
-		conv_jistodisp(tmp, len, buf);
-		strncpy2(outbuf, tmp, outlen);
-	} else
-		strncpy2(outbuf, buf, outlen);
-#endif
-}
-
 static const struct {
 	CharSet charset;
 	gchar *const name;
@@ -723,6 +695,65 @@ const gchar *conv_get_outgoing_charset_str(void)
 	return str ? str : "US-ASCII";
 }
 
+void conv_unmime_header_overwrite(gchar *str)
+{
+	gchar *buf;
+	gint outlen;
+	CharSet cur_charset;
+
+	cur_charset = conv_get_current_charset();
+
+#if HAVE_LIBJCONV
+	Xstrdup_a(buf, str, {return;});
+	outlen = strlen(str) + 1;
+	UnMimeHeaderConv(buf, str, outlen);
+	if (cur_charset == C_EUC_JP)
+		conv_unreadable_eucjp(str);
+#else
+	if (cur_charset == C_EUC_JP) {
+		gchar *tmp;
+		gint len;
+
+		Xstrdup_a(buf, str, {return;});
+		outlen = strlen(str) + 1;
+		UnMimeHeader(buf);
+		len = strlen(buf) * 2 + 1;
+		Xalloca(tmp, len, {strncpy2(str, buf, outlen); return;});
+		conv_jistodisp(tmp, len, buf);
+		strncpy2(str, tmp, outlen);
+	} else
+		UnMimeHeader(str);
+#endif
+}
+
+void conv_unmime_header(gchar *outbuf, gint outlen, const gchar *str,
+			const gchar *charset)
+{
+	gchar *buf;
+	CharSet cur_charset;
+
+	Xstrdup_a(buf, str, {return;});
+	cur_charset = conv_get_current_charset();
+
+#if HAVE_LIBJCONV
+	UnMimeHeaderConv(buf, outbuf, outlen);
+	if (cur_charset == C_EUC_JP)
+		conv_unreadable_eucjp(outbuf);
+#else
+	UnMimeHeader(buf);
+	if (cur_charset == C_EUC_JP) {
+		gchar *tmp;
+		gint len;
+
+		len = strlen(buf) * 2 + 1;
+		Xalloca(tmp, len, {strncpy2(outbuf, buf, outlen); return;});
+		conv_jistodisp(tmp, len, buf);
+		strncpy2(outbuf, tmp, outlen);
+	} else
+		strncpy2(outbuf, buf, outlen);
+#endif
+}
+
 #define MAX_ENCLEN	75
 #define MAX_LINELEN	76
 
@@ -766,11 +797,11 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 		if ((len - (destp - dest)) < (MAX_LINELEN + 1) * 2) break;
 
 		/* encode string including space
-		   if multi-byte string follows */
-		if (is_next_mbs(wsrcp)) {
+		   if non-ASCII string follows */
+		if (is_next_nonascii(wsrcp)) {
 			wp = wsrcp;
 			while ((wp = find_wspace(wp)) != NULL)
-				if (!is_next_mbs(wp)) break;
+				if (!is_next_nonascii(wp)) break;
 		} else
 			wp = find_wspace(wsrcp);
 
@@ -977,11 +1008,11 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 		if ((len - (destp - dest)) < (MAX_LINELEN + 1) * 2) break;
 
 		/* encode string including space
-		   if multi-byte string follows */
-		if (is_next_mbs(wsrcp)) {
+		   if non-ASCII string follows */
+		if (is_next_nonascii(wsrcp)) {
 			wp = wsrcp;
 			while ((wp = find_wspace(wp)) != NULL)
-				if (!is_next_mbs(wp)) break;
+				if (!is_next_nonascii(wp)) break;
 		} else
 			wp = find_wspace(wsrcp);
 
