@@ -134,9 +134,6 @@ gchar *conv_codeset_strdup(const gchar *inbuf,
 
 CodeConvFunc conv_get_code_conv_func(const gchar *charset);
 
-void conv_unmime_header(gchar *outbuf, gint outlen, const gchar *str,
-			const gchar *charset);
-
 const gchar *conv_get_charset_str		(CharSet	 charset);
 CharSet conv_get_charset_from_str		(const gchar	*charset);
 CharSet conv_get_current_charset		(void);
@@ -144,10 +141,15 @@ const gchar *conv_get_current_charset_str	(void);
 CharSet conv_get_outgoing_charset		(void);
 const gchar *conv_get_outgoing_charset_str	(void);
 
-void conv_encode_header(gchar		*dest,
-			gint		 len,
-			const gchar	*src,
-			gint		 header_len);
+void conv_unmime_header_overwrite	(gchar		*str);
+void conv_unmime_header			(gchar		*outbuf,
+					 gint		 outlen,
+					 const gchar	*str,
+					 const gchar	*charset);
+void conv_encode_header			(gchar		*dest,
+					 gint		 len,
+					 const gchar	*src,
+					 gint		 header_len);
 
 
 #endif /* __CODECONV_H__ */
