@@ -494,6 +494,7 @@ gint folder_item_add_msg(FolderItem *dest, const gchar *file)
 
 	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(file != NULL, -1);
+	g_return_val_if_fail(dest->folder->add_msg != NULL, -1);
 
 	folder = dest->folder;
 	if (dest->last_num < 0) folder->scan(folder, dest);
@@ -538,6 +539,40 @@ gint folder_item_move_msgs_with_dest(FolderItem *dest, GSList *msglist)
 	return num;
 }
 
+gint folder_item_copy_msg(FolderItem *dest, MsgInfo *msginfo)
+{
+	Folder *folder;
+	gint num;
+
+	g_return_val_if_fail(dest != NULL, -1);
+	g_return_val_if_fail(msginfo != NULL, -1);
+
+	folder = dest->folder;
+	if (dest->last_num < 0) folder->scan(folder, dest);
+
+	num = folder->copy_msg(folder, dest, msginfo);
+	if (num > 0) dest->last_num = num;
+
+	return num;
+}
+
+gint folder_item_copy_msgs_with_dest(FolderItem *dest, GSList *msglist)
+{
+	Folder *folder;
+	gint num;
+
+	g_return_val_if_fail(dest != NULL, -1);
+	g_return_val_if_fail(msglist != NULL, -1);
+
+	folder = dest->folder;
+	if (dest->last_num < 0) folder->scan(folder, dest);
+
+	num = folder->copy_msgs_with_dest(folder, dest, msglist);
+	if (num > 0) dest->last_num = num;
+
+	return num;
+}
+
 gint folder_item_remove_msg(FolderItem *item, gint num)
 {
 	Folder *folder;
@@ -637,6 +672,8 @@ static void folder_init(Folder *folder, FolderType type, const gchar *name)
 		folder->add_msg             = mh_add_msg;
 		folder->move_msg            = mh_move_msg;
 		folder->move_msgs_with_dest = mh_move_msgs_with_dest;
+		folder->copy_msg            = mh_copy_msg;
+		folder->copy_msgs_with_dest = mh_copy_msgs_with_dest;
 		folder->remove_msg          = mh_remove_msg;
 		folder->remove_all_msg      = mh_remove_all_msg;
 		folder->is_msg_changed      = mh_is_msg_changed;
@@ -882,22 +919,38 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 		Folder *folder = item->folder;
 
 		fprintf(fp, "<folder type=\"%s\"", folder_type_str[folder->type]);
-		if (folder->name)
-			fprintf(fp, " name=\"%s\"", folder->name);
-		if (folder->type == F_MH)
-			fprintf(fp, " path=\"%s\"", LOCAL_FOLDER(folder)->rootpath);
+		if (folder->name) {
+			fputs(" name=\"", fp);
+			xml_file_put_escape_str(fp, folder->name);
+			fputs("\"", fp);
+		}
+		if (folder->type == F_MH) {
+			fputs(" path=\"", fp);
+			xml_file_put_escape_str
+				(fp, LOCAL_FOLDER(folder)->rootpath);
+			fputs("\"", fp);
+		}
 		if (folder->account)
-			fprintf(fp, " account_id=\"%d\"", folder->account->account_id);
+			fprintf(fp, " account_id=\"%d\"",
+				folder->account->account_id);
 	} else {
 		fprintf(fp, "<folderitem type=\"%s\"",
 			folder_item_stype_str[item->stype]);
-		if (item->name)
-			fprintf(fp, " name=\"%s\"", item->name);
-		if (item->path)
-			fprintf(fp, " path=\"%s\"", item->path);
+		if (item->name) {
+			fputs(" name=\"", fp);
+			xml_file_put_escape_str(fp, item->name);
+			fputs("\"", fp);
+		}
+		if (item->path) {
+			fputs(" path=\"", fp);
+			xml_file_put_escape_str(fp, item->path);
+			fputs("\"", fp);
+		}
 		if (item->account)
-			fprintf(fp, " account_id = \"%d\"", item->account->account_id);
-		fprintf(fp, " mtime=\"%ld\" new=\"%d\" unread=\"%d\" total=\"%d\"",
+			fprintf(fp, " account_id = \"%d\"",
+				item->account->account_id);
+		fprintf(fp,
+			" mtime=\"%ld\" new=\"%d\" unread=\"%d\" total=\"%d\"",
 			item->mtime, item->new, item->unread, item->total);
 	}
 
