@@ -194,8 +194,8 @@ static void folderview_drag_received_cb  (GtkWidget        *widget,
 
 static GtkItemFactoryEntry folderview_mail_popup_entries[] =
 {
-	{N_("/Create _new folder"),	NULL, folderview_new_folder_cb,    0, NULL},
-	{N_("/_Rename folder"),		NULL, folderview_rename_folder_cb, 0, NULL},
+	{N_("/Create _new folder..."),	NULL, folderview_new_folder_cb,    0, NULL},
+	{N_("/_Rename folder..."),	NULL, folderview_rename_folder_cb, 0, NULL},
 	{N_("/_Delete folder"),		NULL, folderview_delete_folder_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Remove _mailbox"),	NULL, folderview_remove_mailbox_cb, 0, NULL},
@@ -205,8 +205,8 @@ static GtkItemFactoryEntry folderview_mail_popup_entries[] =
 
 static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 {
-	{N_("/Create _new folder"),	NULL, folderview_new_imap_folder_cb, 0, NULL},
-	{N_("/_Rename folder"),		NULL, NULL, 0, NULL},
+	{N_("/Create _new folder..."),	NULL, folderview_new_imap_folder_cb, 0, NULL},
+	{N_("/_Rename folder..."),	NULL, NULL, 0, NULL},
 	{N_("/_Delete folder"),		NULL, folderview_rm_imap_folder_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Remove _IMAP4 server"),	NULL, folderview_rm_imap_server_cb, 0, NULL},
@@ -216,7 +216,8 @@ static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 
 static GtkItemFactoryEntry folderview_news_popup_entries[] =
 {
-	{N_("/_Subscribe to newsgroup"), NULL, folderview_new_news_group_cb, 0, NULL},
+	{N_("/_Subscribe to newsgroup..."),
+					 NULL, folderview_new_news_group_cb, 0, NULL},
 	{N_("/_Remove newsgroup"),	 NULL, folderview_rm_news_group_cb, 0, NULL},
 	{N_("/---"),			 NULL, NULL, 0, "<Separator>"},
 	{N_("/Remove _news server"),	 NULL, folderview_rm_news_server_cb, 0, NULL},
@@ -800,6 +801,85 @@ static void folderview_append_folder(FolderView *folderview, Folder *folder)
 	folderview_sort_folders(folderview, root, folder);
 }
 
+void folderview_new_folder(FolderView *folderview)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	FolderItem *item;
+
+	if (!folderview->selected) return;
+
+	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+
+	switch (item->folder->type) {
+	case F_MH:
+	case F_MBOX:
+	case F_MAILDIR:
+		folderview_new_folder_cb(folderview, 0, NULL);
+		break;
+	case F_IMAP:
+		folderview_new_imap_folder_cb(folderview, 0, NULL);
+		break;
+	case F_NEWS:
+	default:
+		break;
+	}
+}
+
+void folderview_rename_folder(FolderView *folderview)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	FolderItem *item;
+
+	if (!folderview->selected) return;
+
+	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+	if (!item->path) return;
+	if (item->stype != F_NORMAL) return;
+
+	switch (item->folder->type) {
+	case F_MH:
+	case F_MBOX:
+	case F_MAILDIR:
+		folderview_rename_folder_cb(folderview, 0, NULL);
+		break;
+	case F_IMAP:
+	case F_NEWS:
+	default:
+		break;
+	}
+}
+
+void folderview_delete_folder(FolderView *folderview)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	FolderItem *item;
+
+	if (!folderview->selected) return;
+
+	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+	if (!item->path) return;
+	if (item->stype != F_NORMAL) return;
+
+	switch (item->folder->type) {
+	case F_MH:
+	case F_MBOX:
+	case F_MAILDIR:
+		folderview_delete_folder_cb(folderview, 0, NULL);
+		break;
+	case F_IMAP:
+		folderview_rm_imap_folder_cb(folderview, 0, NULL);
+	case F_NEWS:
+	default:
+		break;
+	}
+}
+
 
 /* callback functions */
 
@@ -857,37 +937,37 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 
 	if (folder->type == F_MH && item->parent == NULL) {
 		menu_set_sensitive(folderview->mail_factory,
-				   "/Create new folder", TRUE);
+				   "/Create new folder...", TRUE);
 		menu_set_sensitive(folderview->mail_factory,
 				   "/Remove mailbox", TRUE);
 	} else if (folder->type == F_MH && item->stype != F_NORMAL) {
 		menu_set_sensitive(folderview->mail_factory,
-				   "/Create new folder", TRUE);
+				   "/Create new folder...", TRUE);
 	} else if (folder->type == F_MH) {
 		menu_set_sensitive(folderview->mail_factory,
-				   "/Create new folder", TRUE);
+				   "/Create new folder...", TRUE);
 		menu_set_sensitive(folderview->mail_factory,
-				   "/Rename folder", TRUE);
+				   "/Rename folder...", TRUE);
 		menu_set_sensitive(folderview->mail_factory,
 				   "/Delete folder", TRUE);
 	} else if (folder->type == F_IMAP && item->parent == NULL) {
 		menu_set_sensitive(folderview->imap_factory,
-				   "/Create new folder", TRUE);
+				   "/Create new folder...", TRUE);
 		menu_set_sensitive(folderview->imap_factory,
 				   "/Remove IMAP4 server", TRUE);
 	} else if (folder->type == F_IMAP && item->parent != NULL) {
 		menu_set_sensitive(folderview->imap_factory,
-				   "/Create new folder", TRUE);
+				   "/Create new folder...", TRUE);
 		menu_set_sensitive(folderview->imap_factory,
 				   "/Delete folder", TRUE);
 	} else if (folder->type == F_NEWS && item->parent == NULL) {
 		menu_set_sensitive(folderview->news_factory,
-				   "/Subscribe to newsgroup", TRUE);
+				   "/Subscribe to newsgroup...", TRUE);
 		menu_set_sensitive(folderview->news_factory,
 				   "/Remove news server", TRUE);
 	} else if (folder->type == F_NEWS && item->parent != NULL) {
 		menu_set_sensitive(folderview->news_factory,
-				   "/Subscribe to newsgroup", TRUE);
+				   "/Subscribe to newsgroup...", TRUE);
 		menu_set_sensitive(folderview->news_factory,
 				   "/Remove newsgroup", TRUE);
 	}
