@@ -342,7 +342,7 @@ gint imap_move_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
 	}
 
 	Xstrdup_a(destdir, dest->path, return -1);
-	imap_path_subst_slash_to_dot(destdir);
+	/* imap_path_subst_slash_to_dot(destdir); */
 
 	imap_copy_message(SESSION(session)->sock, msginfo->msgnum, destdir);
 	imap_set_article_flags(session, msginfo->msgnum, msginfo->msgnum,
@@ -371,9 +371,8 @@ gint imap_move_msgs_with_dest(Folder *folder, FolderItem *dest,
 	session = imap_session_connect_if_not(folder);
 	if (!session) return -1;
 
-	Xalloca(destdir, strlen(dest->path) + 1, return -1);
-	strcpy(destdir, dest->path);
-	imap_path_subst_slash_to_dot(destdir);
+	Xstrdup_a(destdir, dest->path, return -1);
+	/* imap_path_subst_slash_to_dot(destdir); */
 
 	for (cur = msglist; cur != NULL; cur = cur->next) {
 		msginfo = (MsgInfo *)cur->data;
@@ -681,9 +680,9 @@ static gint imap_select(gint sock, const gchar *folder,
 	*exists = *recent = *unseen = *uid = 0;
 	argbuf = g_ptr_array_new();
 
-	Xalloca(imappath, strlen(folder) + 1, return -1);
-	strcpy(imappath, folder);
-	imap_path_subst_slash_to_dot(imappath);
+	Xstrdup_a(imappath, folder, return -1);
+	/* imap_path_subst_slash_to_dot(imappath); */
+
 	imap_gen_send(sock, "SELECT \"%s\"", imappath);
 	if ((ok = imap_ok(sock, argbuf)) != IMAP_SUCCESS)
 		goto bail;
