@@ -48,6 +48,20 @@ struct _IMAPSession
 
 #define IMAPBUFSIZE	8192
 
+typedef enum
+{
+	IMAP_FLAG_SEEN		= 1 << 0,
+	IMAP_FLAG_ANSWERED	= 1 << 1,
+	IMAP_FLAG_FLAGGED	= 1 << 2,
+	IMAP_FLAG_DELETED	= 1 << 3,
+	IMAP_FLAG_DRAFT		= 1 << 4
+} IMAPFlags;
+
+#define IMAP_IS_SEEN(flags)	((flags & IMAP_FLAG_SEEN) != 0)
+#define IMAP_IS_ANSWERED(flags)	((flags & IMAP_FLAG_ANSWERED) != 0)
+#define IMAP_IS_FLAGGED(flags)	((flags & IMAP_FLAG_FLAGGED) != 0)
+#define IMAP_IS_DELETED(flags)	((flags & IMAP_FLAG_DELETED) != 0)
+#define IMAP_IS_DRAFT(flags)	((flags & IMAP_FLAG_DRAFT) != 0)
 
 Session *imap_session_new		(const gchar	*server,
 					 gushort	 port,
@@ -63,6 +77,19 @@ gchar *imap_fetch_msg			(Folder		*folder,
 					 FolderItem	*item,
 					 gint		 num);
 
+gint imap_move_msg			(Folder		*folder,
+					 FolderItem     *dest,
+					 MsgInfo        *msginfo);
+
+gint imap_move_msgs_with_dest		(Folder         *folder,
+					 FolderItem     *dest,
+					 GSList         *msglist);
+gint imap_remove_msg			(Folder		*folder,
+					 FolderItem	*item,
+					 gint		 num);
+gint imap_remove_all_msg		(Folder		*folder,
+					 FolderItem	*item);
+
 void imap_scan_folder			(Folder		*folder,
 					 FolderItem	*item);
 
