@@ -41,6 +41,7 @@
 #include "inc.h"
 #include "mainwindow.h"
 #include "folderview.h"
+#include "summaryview.h"
 #include "prefs_common.h"
 #include "prefs_account.h"
 #include "account.h"
@@ -62,6 +63,7 @@
 
 #define MSGBUFSIZE	8192
 
+static void inc_finished		(MainWindow		*mainwin);
 static void inc_account_mail		(PrefsAccount		*account,
 					 MainWindow		*mainwin);
 
@@ -93,8 +95,27 @@ static gint inc_spool			(void);
 static gint get_spool			(FolderItem	*dest,
 					 const gchar	*mbox);
 
+static void inc_finished(MainWindow *mainwin)
+{
+	FolderItem *item;
+
+	if (prefs_common.open_inbox_on_inc) {
+		item = cur_account && cur_account->inbox
+			? folder_find_item_from_path(cur_account->inbox)
+			: folder_get_default_inbox();
+		folderview_unselect(mainwin->folderview);
+		folderview_select(mainwin->folderview, item);
+	} else {
+		item = mainwin->summaryview->folder_item;
+		folderview_unselect(mainwin->folderview);
+		folderview_select(mainwin->folderview, item);
+	}
+}
+
 void inc_mail(MainWindow *mainwin)
 {
+	summary_write_cache(mainwin->summaryview);
+
 	if (prefs_common.use_extinc && prefs_common.extinc_path) {
 		gint pid;
 
@@ -124,11 +145,7 @@ void inc_mail(MainWindow *mainwin)
 		inc_account_mail(cur_account, mainwin);
 	}
 
-	folderview_unselect(mainwin->folderview);
-	folderview_select(mainwin->folderview,
-			  cur_account && cur_account->inbox
-			  ? folder_find_item_from_path(cur_account->inbox)
-			  : folder_get_default_inbox());
+	inc_finished(mainwin);
 }
 
 static void inc_account_mail(PrefsAccount *account, MainWindow *mainwin)
@@ -152,6 +169,8 @@ void inc_all_account_mail(MainWindow *mainwin)
 	GList *list, *queue_list = NULL;
 	IncProgressDialog *inc_dialog;
 
+	summary_write_cache(mainwin->summaryview);
+
 	if (prefs_common.inc_local) inc_spool();
 
 	list = account_get_list();
@@ -180,11 +199,7 @@ void inc_all_account_mail(MainWindow *mainwin)
 
 	inc_start(inc_dialog);
 
-	folderview_unselect(mainwin->folderview);
-	folderview_select(mainwin->folderview,
-			  cur_account && cur_account->inbox
-			  ? folder_find_item_from_path(cur_account->inbox)
-			  : folder_get_default_inbox());
+	inc_finished(mainwin);
 }
 
 static IncProgressDialog *inc_progress_dialog_create(void)
@@ -313,8 +328,10 @@ static void inc_start(IncProgressDialog *inc_dialog)
 			gchar *pass;
 			gchar *message;
 
-			message = g_strdup_printf(_("Input password for %s:"),
-						  pop3_state->user);
+			message = g_strdup_printf
+				(_("Input password for %s on %s:"),
+				 pop3_state->user,
+				 pop3_state->ac_prefs->recv_server);
 
 			pass = input_dialog_with_invisible(_("Input password"),
 							   message, NULL);
@@ -337,8 +354,10 @@ static void inc_start(IncProgressDialog *inc_dialog)
 
 		if (pop3_state->error_val == PS_AUTHFAIL) {
 			manage_window_focus_in(inc_dialog->dialog->window, NULL, NULL);
-			alertpanel_error(_("Authorization for %s failed"),
-					 pop3_state->user);
+			alertpanel_error
+				(_("Authorization for %s on %s failed"),
+				 pop3_state->user,
+				 pop3_state->ac_prefs->recv_server);
 		}
 
 		statusbar_pop_all();
@@ -445,6 +464,8 @@ static IncState inc_pop3_session_do(IncSession *session)
 				 server, port);
 		manage_window_focus_out(inc_dialog->dialog->window, NULL, NULL);
 		pop3_automaton_terminate(-1, atm);
+		automaton_destroy(atm);
+
 		return pop3_state->inc_state;
 	}
 
@@ -465,20 +486,27 @@ static IncState inc_pop3_session_do(IncSession *session)
 	//pthread_join(sockinfo->connect_thr, NULL);
 #endif
 	sock_sockinfo_free(sockinfo);
+	automaton_destroy(atm);
 
 	return pop3_state->inc_state;
 }
 
 static gint pop3_automaton_terminate(gint source, Automaton *atm)
 {
-	if (atm->tag > 0) gdk_input_remove(atm->tag);
-	if (atm->timeout_tag > 0) gtk_timeout_remove(atm->timeout_tag);
+	if (atm->tag > 0) {
+		gdk_input_remove(atm->tag);
+		atm->tag = 0;
+	}
+	if (atm->timeout_tag > 0) {
+		gtk_timeout_remove(atm->timeout_tag);
+		atm->timeout_tag = 0;
+	}
 	if (source > 0) {
 		sock_close(source);
 		gtk_main_quit();
 	}
 
-	automaton_destroy(atm);
+	atm->terminated = TRUE;
 
 	return 0;
 }
