@@ -143,12 +143,22 @@ static void add_mailbox_cb	 (MainWindow	*mainwin,
 static void update_folderview_cb (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
+static void new_folder_cb	 (MainWindow	*mainwin,
+				  guint		 action,
+				  GtkWidget	*widget);
+static void rename_folder_cb	 (MainWindow	*mainwin,
+				  guint		 action,
+				  GtkWidget	*widget);
+static void delete_folder_cb	 (MainWindow	*mainwin,
+				  guint		 action,
+				  GtkWidget	*widget);
 static void import_mbox_cb	 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
-static void clean_trash_cb	 (MainWindow	*mainwin,
+static void empty_trash_cb	 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
+
 static void save_as_cb		 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
@@ -215,6 +225,9 @@ static void reedit_cb			(MainWindow	*mainwin,
 static void move_to_cb			(MainWindow	*mainwin,
 					 guint		 action,
 					 GtkWidget	*widget);
+static void copy_to_cb			(MainWindow	*mainwin,
+					 guint		 action,
+					 GtkWidget	*widget);
 static void delete_cb			(MainWindow	*mainwin,
 					 guint		 action,
 					 GtkWidget	*widget);
@@ -263,10 +276,6 @@ static void update_summary_cb	 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
 
-static void select_all_cb	 (MainWindow	*mainwin,
-				  guint		 action,
-				  GtkWidget	*widget);
-
 static void prev_cb		 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
@@ -313,10 +322,15 @@ static void scan_tree_func	 (Folder	*folder,
 static GtkItemFactoryEntry mainwin_entries[] =
 {
 	{N_("/_File"),				NULL, NULL, 0, "<Branch>"},
-	{N_("/_File/_Add mailbox"),		NULL, add_mailbox_cb, 0, NULL},
+	{N_("/_File/_Add mailbox..."),		NULL, add_mailbox_cb, 0, NULL},
 	{N_("/_File/_Update folder tree"),	NULL, update_folderview_cb, 0, NULL},
-	{N_("/_File/_Import mbox file"),	NULL, import_mbox_cb, 0, NULL},
-	{N_("/_File/Clean _trash"),		NULL, clean_trash_cb, 0, NULL},
+	{N_("/_File/_Folder"),			NULL, NULL, 0, "<Branch>"},
+	{N_("/_File/_Folder/Create _new folder..."),
+						NULL, new_folder_cb, 0, NULL},
+	{N_("/_File/_Folder/_Rename folder..."),NULL, rename_folder_cb, 0, NULL},
+	{N_("/_File/_Folder/_Delete folder"),	NULL, delete_folder_cb, 0, NULL},
+	{N_("/_File/_Import mbox file..."),	NULL, import_mbox_cb, 0, NULL},
+	{N_("/_File/Empty _trash"),		NULL, empty_trash_cb, 0, NULL},
 	{N_("/_File/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_File/_Save as..."),		NULL, save_as_cb, 0, NULL},
 	{N_("/_File/_Print..."),		"<alt>P", print_cb, 0, NULL},
@@ -330,6 +344,90 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Edit/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Edit/_Search"),			"<control>S", summary_search_cb, 0, NULL},
 
+	{N_("/_View"),				NULL, NULL, 0, "<Branch>"},
+	{N_("/_View/_Folder tree"),		NULL, toggle_folder_cb, 0, "<ToggleItem>"},
+	{N_("/_View/_Message view"),		NULL, toggle_message_cb, 0, "<ToggleItem>"},
+	{N_("/_View/_Toolbar"),			NULL, NULL, 0, "<Branch>"},
+	{N_("/_View/_Toolbar/Icon _and text"),	NULL, toggle_toolbar_cb, TOOLBAR_BOTH, "<RadioItem>"},
+	{N_("/_View/_Toolbar/_Icon"),		NULL, toggle_toolbar_cb, TOOLBAR_ICON, "/View/Toolbar/Icon and text"},
+	{N_("/_View/_Toolbar/_Text"),		NULL, toggle_toolbar_cb, TOOLBAR_TEXT, "/View/Toolbar/Icon and text"},
+	{N_("/_View/_Toolbar/_Non-display"),	NULL, toggle_toolbar_cb, TOOLBAR_NONE, "/View/Toolbar/Icon and text"},
+	{N_("/_View/_Status bar"),		NULL, toggle_statusbar_cb, 0, "<ToggleItem>"},
+	{N_("/_View/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_View/Separate f_older tree"),	NULL, separate_widget_cb, SEPARATE_FOLDER, "<ToggleItem>"},
+	{N_("/_View/Separate m_essage view"),	NULL, separate_widget_cb, SEPARATE_MESSAGE, "<ToggleItem>"},
+	{N_("/_View/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_View/_Code set"),		NULL, NULL, 0, "<Branch>"},
+	{N_("/_View/_Code set/_Auto detect"),
+	 NULL, set_charset_cb, C_AUTO, "<RadioItem>"},
+
+#define CODESET_SEPARATOR \
+	{N_("/_View/_Code set/---"),		NULL, NULL, 0, "<Separator>"}
+#define CODESET_ACTION(action) \
+	 NULL, set_charset_cb, action, "/View/Code set/Auto detect"
+
+	{N_("/_View/_Code set/---"),		NULL, NULL, 0, "<Separator>"},
+	{N_("/_View/_Code set/7bit ascii (US-ASC_II)"),
+	 CODESET_ACTION(C_US_ASCII)},
+
+#if HAVE_LIBJCONV
+	{N_("/_View/_Code set/Unicode (_UTF-8)"),
+	 CODESET_ACTION(C_UTF_8)},
+	CODESET_SEPARATOR,
+#endif
+	{N_("/_View/_Code set/Western European (ISO-8859-_1)"),
+	 CODESET_ACTION(C_ISO_8859_1)},
+	CODESET_SEPARATOR,
+#if HAVE_LIBJCONV
+	{N_("/_View/_Code set/Central European (ISO-8859-_2)"),
+	 CODESET_ACTION(C_ISO_8859_2)},
+	CODESET_SEPARATOR,
+	{N_("/_View/_Code set/_Baltic (ISO-8859-13)"),
+	 CODESET_ACTION(C_ISO_8859_13)},
+	{N_("/_View/_Code set/Baltic (ISO-8859-_4)"),
+	 CODESET_ACTION(C_ISO_8859_4)},
+	CODESET_SEPARATOR,
+	{N_("/_View/_Code set/Greek (ISO-8859-_7)"),
+	 CODESET_ACTION(C_ISO_8859_7)},
+	CODESET_SEPARATOR,
+	{N_("/_View/_Code set/Turkish (ISO-8859-_9)"),
+	 CODESET_ACTION(C_ISO_8859_9)},
+	CODESET_SEPARATOR,
+	{N_("/_View/_Code set/Cyrillic (ISO-8859-_5)"),
+	 CODESET_ACTION(C_ISO_8859_5)},
+	{N_("/_View/_Code set/Cyrillic (KOI8-_R)"),
+	 CODESET_ACTION(C_KOI8_R)},
+	{N_("/_View/_Code set/Cyrillic (Windows-1251)"),
+	 CODESET_ACTION(C_CP1251)},
+	CODESET_SEPARATOR,
+#endif
+	{N_("/_View/_Code set/Japanese (ISO-2022-_JP)"),
+	 CODESET_ACTION(C_ISO_2022_JP)},
+#if HAVE_LIBJCONV
+	{N_("/_View/_Code set/Japanese (ISO-2022-JP-2)"),
+	 CODESET_ACTION(C_ISO_2022_JP_2)},
+#endif
+	{N_("/_View/_Code set/Japanese (_EUC-JP)"),
+	 CODESET_ACTION(C_EUC_JP)},
+	{N_("/_View/_Code set/Japanese (_Shift__JIS)"),
+	 CODESET_ACTION(C_SHIFT_JIS)},
+#if HAVE_LIBJCONV
+	CODESET_SEPARATOR,
+	{N_("/_View/_Code set/Simplified Chinese (_GB2312)"),
+	 CODESET_ACTION(C_GB2312)},
+	{N_("/_View/_Code set/Traditional Chinese (_Big5)"),
+	 CODESET_ACTION(C_BIG5)},
+	{N_("/_View/_Code set/Traditional Chinese (EUC-_TW)"),
+	 CODESET_ACTION(C_EUC_TW)},
+	{N_("/_View/_Code set/Chinese (ISO-2022-_CN)"),
+	 CODESET_ACTION(C_ISO_2022_CN)},
+	CODESET_SEPARATOR,
+	{N_("/_View/_Code set/Korean (EUC-_KR)"),
+	 CODESET_ACTION(C_EUC_KR)},
+	{N_("/_View/_Code set/Korean (ISO-2022-KR)"),
+	 CODESET_ACTION(C_ISO_2022_KR)},
+#endif
+
 	{N_("/_Message"),			NULL, NULL, 0, "<Branch>"},
 	{N_("/_Message/Rece_ive new mail"),	"<alt>I",	inc_mail_cb, 0, NULL},
 	{N_("/_Message/Receive from _all accounts"),
@@ -346,6 +444,7 @@ static GtkItemFactoryEntry mainwin_entries[] =
 						"<shift><control>F", reply_cb, COMPOSE_FORWARD_AS_ATTACH, NULL},
 	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Message/M_ove..."),		"<alt>O", move_to_cb, 0, NULL},
+	{N_("/_Message/_Copy..."),		NULL, copy_to_cb, 0, NULL},
 	{N_("/_Message/_Delete"),		"<alt>D", delete_cb,  0, NULL},
 	{N_("/_Message/_Mark"),			NULL, NULL, 0, "<Branch>"},
 	{N_("/_Message/_Mark/_Mark"),		NULL, mark_cb,   0, NULL},
@@ -359,77 +458,6 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Message/View _source"),		"<control>U", view_source_cb, 0, NULL},
 	{N_("/_Message/Show all _header"),	"<control>H",	header_window_show_cb,	0, NULL},
 	{N_("/_Message/Re_edit"),		NULL, reedit_cb, 0, NULL},
-	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Message/_Code set"),		NULL, NULL, 0, "<Branch>"},
-	{N_("/_Message/_Code set/_Auto detect"),
-	 NULL, set_charset_cb, C_AUTO, "<RadioItem>"},
-
-#define CODESET_SEPARATOR \
-	{N_("/_Message/_Code set/---"),		NULL, NULL, 0, "<Separator>"}
-#define CODESET_ACTION(action) \
-	 NULL, set_charset_cb, action, "/Message/Code set/Auto detect"
-
-	{N_("/_Message/_Code set/---"),		NULL, NULL, 0, "<Separator>"},
-	{N_("/_Message/_Code set/7bit ascii (US-ASC_II)"),
-	 CODESET_ACTION(C_US_ASCII)},
-
-#if HAVE_LIBJCONV
-	{N_("/_Message/_Code set/Unicode (_UTF-8)"),
-	 CODESET_ACTION(C_UTF_8)},
-	CODESET_SEPARATOR,
-#endif
-	{N_("/_Message/_Code set/Western European (ISO-8859-_1)"),
-	 CODESET_ACTION(C_ISO_8859_1)},
-	CODESET_SEPARATOR,
-#if HAVE_LIBJCONV
-	{N_("/_Message/_Code set/Central European (ISO-8859-_2)"),
-	 CODESET_ACTION(C_ISO_8859_2)},
-	CODESET_SEPARATOR,
-	{N_("/_Message/_Code set/_Baltic (ISO-8859-13)"),
-	 CODESET_ACTION(C_ISO_8859_13)},
-	{N_("/_Message/_Code set/Baltic (ISO-8859-_4)"),
-	 CODESET_ACTION(C_ISO_8859_4)},
-	CODESET_SEPARATOR,
-	{N_("/_Message/_Code set/Greek (ISO-8859-_7)"),
-	 CODESET_ACTION(C_ISO_8859_7)},
-	CODESET_SEPARATOR,
-	{N_("/_Message/_Code set/Turkish (ISO-8859-_9)"),
-	 CODESET_ACTION(C_ISO_8859_9)},
-	CODESET_SEPARATOR,
-	{N_("/_Message/_Code set/Cyrillic (ISO-8859-_5)"),
-	 CODESET_ACTION(C_ISO_8859_5)},
-	{N_("/_Message/_Code set/Cyrillic (KOI8-_R)"),
-	 CODESET_ACTION(C_KOI8_R)},
-	{N_("/_Message/_Code set/Cyrillic (Windows-1251)"),
-	 CODESET_ACTION(C_CP1251)},
-	CODESET_SEPARATOR,
-#endif
-	{N_("/_Message/_Code set/Japanese (ISO-2022-_JP)"),
-	 CODESET_ACTION(C_ISO_2022_JP)},
-#if HAVE_LIBJCONV
-	{N_("/_Message/_Code set/Japanese (ISO-2022-JP-2)"),
-	 CODESET_ACTION(C_ISO_2022_JP_2)},
-#endif
-	{N_("/_Message/_Code set/Japanese (_EUC-JP)"),
-	 CODESET_ACTION(C_EUC_JP)},
-	{N_("/_Message/_Code set/Japanese (_Shift__JIS)"),
-	 CODESET_ACTION(C_SHIFT_JIS)},
-#if HAVE_LIBJCONV
-	CODESET_SEPARATOR,
-	{N_("/_Message/_Code set/Simplified Chinese (_GB2312)"),
-	 CODESET_ACTION(C_GB2312)},
-	{N_("/_Message/_Code set/Traditional Chinese (_Big5)"),
-	 CODESET_ACTION(C_BIG5)},
-	{N_("/_Message/_Code set/Traditional Chinese (EUC-_TW)"),
-	 CODESET_ACTION(C_EUC_TW)},
-	{N_("/_Message/_Code set/Chinese (ISO-2022-_CN)"),
-	 CODESET_ACTION(C_ISO_2022_CN)},
-	CODESET_SEPARATOR,
-	{N_("/_Message/_Code set/Korean (EUC-_KR)"),
-	 CODESET_ACTION(C_EUC_KR)},
-	{N_("/_Message/_Code set/Korean (ISO-2022-KR)"),
-	 CODESET_ACTION(C_ISO_2022_KR)},
-#endif
 
 	{N_("/_Summary"),			NULL, NULL, 0, "<Branch>"},
 	{N_("/_Summary/_Delete duplicated messages"),
@@ -437,7 +465,6 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Summary/_Filter messages"),	NULL, filter_cb, 0, NULL},
 	{N_("/_Summary/E_xecute"),		"<alt>X", execute_summary_cb, 0, NULL},
 	{N_("/_Summary/_Update"),		"<alt>U", update_summary_cb,  0, NULL},
-	{N_("/_Summary/Select a_ll"),		NULL, select_all_cb, 0, NULL},
 	{N_("/_Summary/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Summary/_Prev message"),		NULL, prev_cb, 0, NULL},
 	{N_("/_Summary/_Next message"),		NULL, next_cb, 0, NULL},
@@ -458,19 +485,6 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Summary/Unt_hread view"),	"<shift><control>T", thread_cb, 1, NULL},
 	{N_("/_Summary/Set display _item..."),	NULL, set_display_item_cb, 0, NULL},
 
-	{N_("/_View"),				NULL, NULL, 0, "<Branch>"},
-	{N_("/_View/_Folder tree"),		NULL, toggle_folder_cb, 0, "<ToggleItem>"},
-	{N_("/_View/_Message view"),		NULL, toggle_message_cb, 0, "<ToggleItem>"},
-	{N_("/_View/_Toolbar"),			NULL, NULL, 0, "<Branch>"},
-	{N_("/_View/_Toolbar/Icon _and text"),	NULL, toggle_toolbar_cb, TOOLBAR_BOTH, "<RadioItem>"},
-	{N_("/_View/_Toolbar/_Icon"),		NULL, toggle_toolbar_cb, TOOLBAR_ICON, "/View/Toolbar/Icon and text"},
-	{N_("/_View/_Toolbar/_Text"),		NULL, toggle_toolbar_cb, TOOLBAR_TEXT, "/View/Toolbar/Icon and text"},
-	{N_("/_View/_Toolbar/_Non-display"),	NULL, toggle_toolbar_cb, TOOLBAR_NONE, "/View/Toolbar/Icon and text"},
-	{N_("/_View/_Status bar"),		NULL, toggle_statusbar_cb, 0, "<ToggleItem>"},
-	{N_("/_View/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_View/Separate f_older tree"),	NULL, separate_widget_cb, SEPARATE_FOLDER, "<ToggleItem>"},
-	{N_("/_View/Separate m_essage view"),	NULL, separate_widget_cb, SEPARATE_MESSAGE, "<ToggleItem>"},
-
 	{N_("/_Tool"),				NULL, NULL, 0, "<Branch>"},
 	{N_("/_Tool/_Address book"),		"<alt>A", addressbook_open_cb, 0, NULL},
 	{N_("/_Tool/_Log window"),		"<alt>L", log_window_show_cb, 0, NULL},
@@ -650,7 +664,7 @@ MainWindow *main_window_create(SeparateType type)
 	/* set menu items */
 	ifactory = gtk_item_factory_from_widget(menubar);
 	menuitem = gtk_item_factory_get_item
-		(ifactory, "/Message/Code set/Auto detect");
+		(ifactory, "/View/Code set/Auto detect");
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), TRUE);
 
 	switch (prefs_common.toolbar_style) {
@@ -902,18 +916,18 @@ void main_window_get_position(MainWindow *mainwin)
 	debug_print(_("window position: x = %d, y = %d\n"), x, y);
 }
 
-void main_window_clean_trash(MainWindow *mainwin, gboolean confirm)
+void main_window_empty_trash(MainWindow *mainwin, gboolean confirm)
 {
 	GList *list;
 
 	if (confirm) {
-		if (alertpanel(_("Clean trash"),
-			       _("Clean all messages in trash?"),
+		if (alertpanel(_("Empty trash"),
+			       _("Empty all messages in trash?"),
 			       _("Yes"), _("No"), NULL) != G_ALERTDEFAULT)
 			return;
 	}
 
-	procmsg_clean_trash();
+	procmsg_empty_trash();
 
 	for (list = folder_get_list(); list != NULL; list = list->next) {
 		Folder *folder;
@@ -1021,6 +1035,7 @@ void main_window_set_menu_sensitive(MainWindow *mainwin, gint selection)
 
 	menu_set_sensitive(ifactory, "/File/Print..."  , sens);
 	menu_set_sensitive(ifactory, "/Message/Move...", sens && exec);
+	menu_set_sensitive(ifactory, "/Message/Copy...", sens && exec);
 	menu_set_sensitive(ifactory, "/Message/Delete" , sens && exec);
 	menu_set_sensitive(ifactory, "/Message/Mark"   , sens);
 
@@ -1032,7 +1047,6 @@ void main_window_set_menu_sensitive(MainWindow *mainwin, gint selection)
 	menu_set_sensitive(ifactory, "/Summary/Delete duplicated messages", sens && exec);
 	menu_set_sensitive(ifactory, "/Summary/Filter messages", sens && exec);
 	menu_set_sensitive(ifactory, "/Summary/Execute", sens);
-	menu_set_sensitive(ifactory, "/Summary/Select all", sens);
 	menu_set_sensitive(ifactory, "/Summary/Prev message", sens);
 	menu_set_sensitive(ifactory, "/Summary/Next message", sens);
 	menu_set_sensitive(ifactory, "/Summary/Next unread message", sens);
@@ -1569,16 +1583,34 @@ static void update_folderview_cb(MainWindow *mainwin, guint action,
 	folderview_update_all();
 }
 
+static void new_folder_cb(MainWindow *mainwin, guint action,
+			  GtkWidget *widget)
+{
+	folderview_new_folder(mainwin->folderview);
+}
+
+static void rename_folder_cb(MainWindow *mainwin, guint action,
+			     GtkWidget *widget)
+{
+	folderview_rename_folder(mainwin->folderview);
+}
+
+static void delete_folder_cb(MainWindow *mainwin, guint action,
+			     GtkWidget *widget)
+{
+	folderview_delete_folder(mainwin->folderview);
+}
+
 static void import_mbox_cb(MainWindow *mainwin, guint action,
 			   GtkWidget *widget)
 {
 	import_mbox();
 }
 
-static void clean_trash_cb(MainWindow *mainwin, guint action,
+static void empty_trash_cb(MainWindow *mainwin, guint action,
 			   GtkWidget *widget)
 {
-	main_window_clean_trash(mainwin, TRUE);
+	main_window_empty_trash(mainwin, TRUE);
 }
 
 static void save_as_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
@@ -1777,6 +1809,8 @@ static void reply_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 	default:
 		compose_reply(msginfo, prefs_common.reply_with_quote, FALSE);
 	}
+
+	summary_set_marks_selected(mainwin->summaryview);
 }
 
 static void move_to_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
@@ -1784,6 +1818,11 @@ static void move_to_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 	summary_move_to(mainwin->summaryview);
 }
 
+static void copy_to_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
+{
+	summary_copy_to(mainwin->summaryview);
+}
+
 static void delete_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 {
 	summary_delete(mainwin->summaryview);
@@ -1911,11 +1950,6 @@ static void update_summary_cb(MainWindow *mainwin, guint action,
 	summary_show(mainwin->summaryview, fitem, TRUE);
 }
 
-static void select_all_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
-{
-	summary_select_all(mainwin->summaryview);
-}
-
 static void prev_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 {
 	summary_step(mainwin->summaryview, GTK_SCROLL_STEP_BACKWARD);
@@ -1950,9 +1984,11 @@ static void copy_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 
 static void allsel_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 {
-	if (!mainwin->summaryview->msg_is_toggled_on) return;
-
-	messageview_select_all(mainwin->messageview);
+	if (GTK_WIDGET_HAS_FOCUS(mainwin->summaryview->ctree))
+		summary_select_all(mainwin->summaryview);
+	else if (mainwin->summaryview->msg_is_toggled_on &&
+		 GTK_WIDGET_HAS_FOCUS(mainwin->messageview->textview->text))
+		messageview_select_all(mainwin->messageview);
 }
 
 static void prefs_common_open_cb(MainWindow *mainwin, guint action,
