@@ -25,8 +25,6 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <fcntl.h>
 #include <sys/file.h>
 #include <ctype.h>
@@ -99,6 +97,7 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, GHashTable *folder_table)
 		gchar *startp, *endp, *rpath;
 		gint empty_line;
 		gint val;
+		gboolean is_next_msg = FALSE;
 
 		if ((tmp_fp = fopen(tmp_file, "w")) == NULL) {
 			FILE_OP_ERROR(tmp_file, "fopen");
@@ -106,7 +105,7 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, GHashTable *folder_table)
 			fclose(mbox_fp);
 			return -1;
 		}
-		if (chmod(tmp_file, S_IRUSR|S_IWUSR) < 0)
+		if (change_file_mode_rw(tmp_fp, tmp_file) < 0)
 			FILE_OP_ERROR(tmp_file, "chmod");
 
 		/* convert unix From into Return-Path */
@@ -135,15 +134,29 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, GHashTable *folder_table)
 			}
 
 			/* From separator */
-			if (!strncmp(buf, "From ", 5)) {
+			while (!strncmp(buf, "From ", 5)) {
 				strcpy(from_line, buf);
-				if (fgets(buf, sizeof(buf), mbox_fp) != NULL) {
-					if (is_header_line(buf)) break;
-					else
-						g_warning(_("unescaped From found:\n%s"), from_line);
-				} else
+				if (fgets(buf, sizeof(buf), mbox_fp) == NULL) {
 					buf[0] = '\0';
+					break;
+				}
+
+				if (is_header_line(buf)) {
+					is_next_msg = TRUE;
+					break;
+				} else if (!strncmp(buf, "From ", 5)) {
+					continue;
+				} else if (!strncmp(buf, ">From ", 6)) {
+					g_memmove(buf, buf + 1, strlen(buf));
+					is_next_msg = TRUE;
+					break;
+				} else {
+					g_warning(_("unescaped From found:\n%s"),
+						  from_line);
+					break;
+				}
 			}
+			if (is_next_msg) break;
 
 			if (empty_line > 0) {
 				while (empty_line--)
