@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include <glib.h>
 #include <dirent.h>
 #include <sys/stat.h>
+#include <errno.h>
 
 #undef MEASURE_TIME
 
@@ -102,7 +103,7 @@ GSList *mh_get_msg_list(Folder *folder, FolderItem *item, gboolean use_cache)
 
 	timersub(&tv_after, &tv_before, &tv_result);
 	g_print("mh_get_msg_list: %s: elapsed time: %ld.%06ld sec\n",
-		folder, tv_result.tv_sec, tv_result.tv_usec);
+		item->path, tv_result.tv_sec, tv_result.tv_usec);
 #endif
 
 	return mlist;
@@ -186,13 +187,32 @@ gint mh_move_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
 	destfile = g_strdup_printf("%s%c%d", destdir, G_DIR_SEPARATOR,
 				   dest->last_num + 1);
 	g_free(destdir);
-	if (rename(srcfile, destfile) < 0) {
-		FILE_OP_ERROR(srcfile, "rename");
+
+	if (is_file_exist(destfile)) {
+		g_warning(_("%s already exists."), destfile);
 		g_free(srcfile);
 		g_free(destfile);
 		if (fp) fclose(fp);
 		return -1;
 	}
+
+	if (rename(srcfile, destfile) < 0) {
+		if (EXDEV == errno) {
+			if (copy_file(srcfile, destfile) < 0) {
+				g_free(srcfile);
+				g_free(destfile);
+				return -1;
+			}
+			unlink(srcfile);
+		} else {
+			FILE_OP_ERROR(srcfile, "rename");
+			g_free(srcfile);
+			g_free(destfile);
+			if (fp) fclose(fp);
+			return -1;
+		}
+	}
+
 	g_free(srcfile);
 	g_free(destfile);
 	dest->last_num++;
@@ -201,13 +221,13 @@ gint mh_move_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
 		MsgInfo newmsginfo;
 
 		newmsginfo.msgnum = dest->last_num;
+		newmsginfo.flags = msginfo->flags;
 		if (dest->stype == F_OUTBOX ||
 		    dest->stype == F_QUEUE  ||
 		    dest->stype == F_DRAFT  ||
 		    dest->stype == F_TRASH)
-			newmsginfo.flags = 0;
-		else
-			newmsginfo.flags = msginfo->flags;
+			MSG_UNSET_FLAGS(newmsginfo.flags, MSG_NEW|MSG_UNREAD);
+
 		procmsg_write_flags(&newmsginfo, fp);
 		fclose(fp);
 	}
@@ -251,12 +271,29 @@ gint mh_move_msgs_with_dest(Folder *folder, FolderItem *dest, GSList *msglist)
 		destfile = g_strdup_printf("%s%c%d", destdir, G_DIR_SEPARATOR,
 					   dest->last_num + 1);
 
-		if (rename(srcfile, destfile) < 0) {
-			FILE_OP_ERROR(srcfile, "rename");
+		if (is_file_exist(destfile)) {
+			g_warning(_("%s already exists."), destfile);
 			g_free(srcfile);
 			g_free(destfile);
 			break;
 		}
+
+		if (rename(srcfile, destfile) < 0) {
+			if (EXDEV == errno) {
+				if (copy_file(srcfile, destfile) < 0) {
+					g_free(srcfile);
+					g_free(destfile);
+					break;
+				}
+				unlink(srcfile);
+			} else {
+				FILE_OP_ERROR(srcfile, "rename");
+				g_free(srcfile);
+				g_free(destfile);
+				break;
+			}
+		}
+
 		g_free(srcfile);
 		g_free(destfile);
 		dest->last_num++;
@@ -265,13 +302,158 @@ gint mh_move_msgs_with_dest(Folder *folder, FolderItem *dest, GSList *msglist)
 			MsgInfo newmsginfo;
 
 			newmsginfo.msgnum = dest->last_num;
+			newmsginfo.flags = msginfo->flags;
 			if (dest->stype == F_OUTBOX ||
 			    dest->stype == F_QUEUE  ||
 			    dest->stype == F_DRAFT  ||
 			    dest->stype == F_TRASH)
-				newmsginfo.flags = 0;
-			else
-				newmsginfo.flags = msginfo->flags;
+				MSG_UNSET_FLAGS(newmsginfo.flags,
+						MSG_NEW|MSG_UNREAD);
+
+			procmsg_write_flags(&newmsginfo, fp);
+		}
+	}
+
+	g_free(destdir);
+	if (fp) fclose(fp);
+
+	return dest->last_num;
+}
+
+gint mh_copy_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
+{
+	gchar *destdir;
+	gchar *srcfile;
+	gchar *destfile;
+	FILE *fp;
+
+	g_return_val_if_fail(dest != NULL, -1);
+	g_return_val_if_fail(msginfo != NULL, -1);
+
+	if (msginfo->folder == dest) {
+		g_warning(_("the src folder is identical to the dest.\n"));
+		return -1;
+	}
+
+	if (dest->last_num < 0) {
+		mh_scan_folder(folder, dest);
+		if (dest->last_num < 0) return -1;
+	}
+
+	destdir = folder_item_get_path(dest);
+	if ((fp = procmsg_open_mark_file(destdir, TRUE)) == NULL)
+		g_warning(_("Can't open mark file.\n"));
+
+	debug_print(_("Copying message %s%c%d to %s ...\n"),
+		    msginfo->folder->path, G_DIR_SEPARATOR,
+		    msginfo->msgnum, dest->path);
+	srcfile = procmsg_get_message_file_path(msginfo);
+	destfile = g_strdup_printf("%s%c%d", destdir, G_DIR_SEPARATOR,
+				   dest->last_num + 1);
+	g_free(destdir);
+
+	if (is_file_exist(destfile)) {
+		g_warning(_("%s already exists."), destfile);
+		g_free(srcfile);
+		g_free(destfile);
+		if (fp) fclose(fp);
+		return -1;
+	}
+
+	if (copy_file(srcfile, destfile) < 0) {
+		FILE_OP_ERROR(srcfile, "copy");
+		g_free(srcfile);
+		g_free(destfile);
+		if (fp) fclose(fp);
+		return -1;
+	}
+
+	g_free(srcfile);
+	g_free(destfile);
+	dest->last_num++;
+
+	if (fp) {
+		MsgInfo newmsginfo;
+
+		newmsginfo.msgnum = dest->last_num;
+		newmsginfo.flags = msginfo->flags;
+		if (dest->stype == F_OUTBOX ||
+		    dest->stype == F_QUEUE  ||
+		    dest->stype == F_DRAFT  ||
+		    dest->stype == F_TRASH)
+			MSG_UNSET_FLAGS(newmsginfo.flags, MSG_NEW|MSG_UNREAD);
+		procmsg_write_flags(&newmsginfo, fp);
+		fclose(fp);
+	}
+
+	return dest->last_num;
+}
+
+gint mh_copy_msgs_with_dest(Folder *folder, FolderItem *dest, GSList *msglist)
+{
+	gchar *destdir;
+	gchar *srcfile;
+	gchar *destfile;
+	FILE *fp;
+	GSList *cur;
+	MsgInfo *msginfo;
+
+	g_return_val_if_fail(dest != NULL, -1);
+	g_return_val_if_fail(msglist != NULL, -1);
+
+	if (dest->last_num < 0) {
+		mh_scan_folder(folder, dest);
+		if (dest->last_num < 0) return -1;
+	}
+
+	destdir = folder_item_get_path(dest);
+	if ((fp = procmsg_open_mark_file(destdir, TRUE)) == NULL)
+		g_warning(_("Can't open mark file.\n"));
+
+	for (cur = msglist; cur != NULL; cur = cur->next) {
+		msginfo = (MsgInfo *)cur->data;
+
+		if (msginfo->folder == dest) {
+			g_warning(_("the src folder is identical to the dest.\n"));
+			continue;
+		}
+		debug_print(_("Copying message %s%c%d to %s ...\n"),
+			    msginfo->folder->path, G_DIR_SEPARATOR,
+			    msginfo->msgnum, dest->path);
+
+		srcfile = procmsg_get_message_file_path(msginfo);
+		destfile = g_strdup_printf("%s%c%d", destdir, G_DIR_SEPARATOR,
+					   dest->last_num + 1);
+
+		if (is_file_exist(destfile)) {
+			g_warning(_("%s already exists."), destfile);
+			g_free(srcfile);
+			g_free(destfile);
+			break;
+		}
+
+		if (copy_file(srcfile, destfile) < 0) {
+			FILE_OP_ERROR(srcfile, "copy");
+			g_free(srcfile);
+			g_free(destfile);
+			break;
+		}
+
+		g_free(srcfile);
+		g_free(destfile);
+		dest->last_num++;
+
+		if (fp) {
+			MsgInfo newmsginfo;
+
+			newmsginfo.msgnum = dest->last_num;
+			newmsginfo.flags = msginfo->flags;
+			if (dest->stype == F_OUTBOX ||
+			    dest->stype == F_QUEUE  ||
+			    dest->stype == F_DRAFT  ||
+			    dest->stype == F_TRASH)
+				MSG_UNSET_FLAGS(newmsginfo.flags,
+						MSG_NEW|MSG_UNREAD);
 			procmsg_write_flags(&newmsginfo, fp);
 		}
 	}
