@@ -78,6 +78,10 @@ static void mimeview_selected		(GtkCTree	*ctree,
 					 GtkCTreeNode	*node,
 					 gint		 column,
 					 MimeView	*mimeview);
+static void mimeview_start_drag 	(GtkWidget	*widget,
+					 gint		 button,
+					 GdkEvent	*event,
+					 MimeView	*mimeview);
 static void mimeview_button_pressed	(GtkWidget	*widget,
 					 GdkEventButton	*event,
 					 MimeView	*mimeview);
@@ -97,12 +101,19 @@ static void mimeview_save_as		(MimeView	*mimeview);
 static void mimeview_launch		(MimeView	*mimeview);
 static void mimeview_view_file		(const gchar	*filename,
 					 MimeInfo	*partinfo);
+#if USE_GPGME
+static void mimeview_check_signature	(MimeView	*mimeview);
+#endif
 
 static GtkItemFactoryEntry mimeview_popup_entries[] =
 {
 	{N_("/_Open"),		  NULL, mimeview_launch,	  0, NULL},
 	{N_("/_Display as text"), NULL, mimeview_display_as_text, 0, NULL},
 	{N_("/_Save as..."),	  NULL, mimeview_save_as,	  0, NULL}
+#if USE_GPGME
+        ,
+        {N_("/_Check signature"), NULL, mimeview_check_signature, 0, NULL}
+#endif
 };
 
 static GtkTargetEntry mimeview_mime_types[] =
@@ -134,8 +145,8 @@ MimeView *mimeview_create(void)
 				       GTK_POLICY_ALWAYS);
 	gtk_widget_set_usize(scrolledwin, -1, 80);
 
-	ctree = gtk_ctree_new_with_titles(N_MIMEVIEW_COLS, 0, titles);
-	gtk_clist_set_selection_mode(GTK_CLIST(ctree), GTK_SELECTION_BROWSE);
+	ctree = gtk_sctree_new_with_titles(N_MIMEVIEW_COLS, 0, titles);
+	gtk_clist_set_selection_mode(GTK_CLIST(ctree), GTK_SELECTION_BROWSE);//SINGLE);
 	gtk_ctree_set_line_style(GTK_CTREE(ctree), GTK_CTREE_LINES_NONE);
 	gtk_clist_set_column_justification(GTK_CLIST(ctree), COL_SIZE,
 					   GTK_JUSTIFY_RIGHT);
@@ -152,6 +163,8 @@ MimeView *mimeview_create(void)
 			   GTK_SIGNAL_FUNC(mimeview_button_pressed), mimeview);
 	gtk_signal_connect(GTK_OBJECT(ctree), "key_press_event",
 			   GTK_SIGNAL_FUNC(mimeview_key_pressed), mimeview);
+	gtk_signal_connect(GTK_OBJECT (ctree),"start_drag",
+			   GTK_SIGNAL_FUNC (mimeview_start_drag), mimeview);
 	gtk_signal_connect(GTK_OBJECT(ctree), "drag_data_get",
 			   GTK_SIGNAL_FUNC(mimeview_drag_data_get), mimeview);
     
@@ -181,6 +194,56 @@ void mimeview_init(MimeView *mimeview)
 {
 }
 
+/* 
+ * Check whether the message is OpenPGP signed
+ */
+#if USE_GPGME
+static gboolean mimeview_is_signed(MimeView *mimeview)
+{
+	MimeInfo *partinfo;
+
+        debug_print("mimeview_is signed of %p\n", mimeview);
+
+        if (!mimeview) return FALSE;
+	if (!mimeview->opened) return FALSE;
+
+        debug_print("mimeview_is_signed: open\n" );
+
+	if (!mimeview->file) return FALSE;
+
+        debug_print("mimeview_is_signed: file\n" );
+
+	partinfo = gtk_ctree_node_get_row_data
+		(GTK_CTREE(mimeview->ctree), mimeview->opened);
+	g_return_val_if_fail(partinfo != NULL, FALSE);
+
+	/* walk the tree and see whether there is a signature somewhere */
+	do {
+		if (rfc2015_has_signature(partinfo))
+			return TRUE;
+        } while ((partinfo = partinfo->parent) != NULL);
+
+	debug_print("mimeview_is_signed: FALSE\n" );
+
+	return FALSE;
+}
+
+static void set_unchecked_signature(MimeInfo *mimeinfo)
+{
+	MimeInfo *sig_partinfo;
+
+	sig_partinfo = rfc2015_find_signature(mimeinfo);
+	if (sig_partinfo == NULL) return;
+
+	g_free(sig_partinfo->sigstatus);
+	sig_partinfo->sigstatus =
+		g_strdup(_("Select \"Check signature\" to check"));
+
+	g_free(sig_partinfo->sigstatus_full);
+	sig_partinfo->sigstatus_full = NULL;
+}
+#endif /*USE_GPGME*/
+
 void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 			   const gchar *file)
 {
@@ -212,7 +275,10 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 
 	procmime_scan_multipart_message(mimeinfo, fp);
 #if USE_GPGME
-	rfc2015_check_signature(mimeinfo, fp);
+	if (prefs_common.auto_check_signatures)
+		rfc2015_check_signature(mimeinfo, fp);
+	else
+		set_unchecked_signature(mimeinfo);
 #endif
 	fclose(fp);
 
@@ -260,6 +326,21 @@ static void mimeview_set_multipart_tree(MimeView *mimeview,
 	}
 }
 
+static gchar *get_part_name(MimeInfo *partinfo)
+{
+#if USE_GPGME
+	if (partinfo->sigstatus)
+		return partinfo->sigstatus;
+	else
+#endif
+	if (partinfo->name)
+		return partinfo->name;
+	else if (partinfo->filename)
+		return partinfo->filename;
+	else
+		return "";
+}
+
 static GtkCTreeNode *mimeview_append_part(MimeView *mimeview,
 					  MimeInfo *partinfo,
 					  GtkCTreeNode *parent)
@@ -271,17 +352,7 @@ static GtkCTreeNode *mimeview_append_part(MimeView *mimeview,
 	str[COL_MIMETYPE] =
 		partinfo->content_type ? partinfo->content_type : "";
 	str[COL_SIZE] = to_human_readable(partinfo->size);
-#if USE_GPGME
-	if (partinfo->sigstatus)
-		str[COL_NAME] = partinfo->sigstatus;
-	else
-#endif
-	if (partinfo->name)
-		str[COL_NAME] = partinfo->name;
-	else if (partinfo->filename)
-		str[COL_NAME] = partinfo->filename;
-	else
-		str[COL_NAME] = "";
+	str[COL_NAME] = get_part_name(partinfo);
 
 	node = gtk_ctree_insert_node(ctree, parent, NULL, str, 0,
 				     NULL, NULL, NULL, NULL,
@@ -397,6 +468,7 @@ static void mimeview_selected(GtkCTree *ctree, GtkCTreeNode *node, gint column,
 
 	if (mimeview->opened == node) return;
 	mimeview->opened = node;
+	gtk_ctree_node_moveto(ctree, node, -1, 0.5, 0);
 
 	partinfo = gtk_ctree_node_get_row_data(ctree, node);
 	if (!partinfo) return;
@@ -415,15 +487,35 @@ static void mimeview_selected(GtkCTree *ctree, GtkCTreeNode *node, gint column,
 #endif
 	default:
 		mimeview_change_view_type(mimeview, MIMEVIEW_TEXT);
-		textview_show_mime_part(mimeview->textview, partinfo);
+#if USE_GPGME
+		if (g_strcasecmp(partinfo->content_type,
+				 "application/pgp-signature") == 0)
+			textview_show_signature_part(mimeview->textview,
+						     partinfo);
+		else
+#endif
+			textview_show_mime_part(mimeview->textview, partinfo);
 		break;
 	}
+}
 
-	if (partinfo->filename || partinfo->name)
-		gtk_drag_source_set(GTK_WIDGET(ctree), GDK_BUTTON1_MASK,
-				    mimeview_mime_types, 1, GDK_ACTION_COPY);
-	else
-		gtk_drag_source_unset(GTK_WIDGET(ctree));
+static void mimeview_start_drag (GtkWidget *widget, gint button,
+				 GdkEvent *event, MimeView *mimeview)
+{
+	GtkTargetList *list;
+	GdkDragContext *context;
+	MimeInfo *partinfo;
+	GtkCTree *ctree;
+
+	g_return_if_fail(mimeview != NULL);
+	ctree = GTK_CTREE(mimeview->ctree);
+	partinfo = gtk_ctree_node_get_row_data(ctree, mimeview->opened);
+	if (partinfo->filename == NULL && partinfo->name == NULL) return;
+
+	list = gtk_target_list_new(mimeview_mime_types, 1);
+	context = gtk_drag_begin(widget, list,
+				 GDK_ACTION_COPY, button, event);
+	gtk_drag_set_icon_default(context);
 }
 
 static void mimeview_button_pressed(GtkWidget *widget, GdkEventButton *event,
@@ -461,8 +553,14 @@ static void mimeview_button_pressed(GtkWidget *widget, GdkEventButton *event,
 		else
 			menu_set_sensitive(mimeview->popupfactory,
 					   "/Display as text", TRUE);
+#if USE_GPGME
+		menu_set_sensitive(mimeview->popupfactory,
+				   "/Check signature",
+				   mimeview_is_signed(mimeview));
+#endif
 
-		gtk_menu_popup(GTK_MENU(mimeview->popupmenu), NULL, NULL, NULL, NULL,
+		gtk_menu_popup(GTK_MENU(mimeview->popupmenu),
+			       NULL, NULL, NULL, NULL,
 			       event->button, event->time);
 	}
 }
@@ -475,10 +573,10 @@ static void mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 {
 	SummaryView *summaryview;
 	GtkCTree *ctree = GTK_CTREE(mimeview->ctree);
-	GtkCList *clist = GTK_CLIST(mimeview->ctree);
 	GtkCTreeNode *node;
 
 	if (!event) return;
+	if (!mimeview->opened) return;
 
 	switch (event->keyval) {
 	case GDK_space:
@@ -486,10 +584,8 @@ static void mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 
 		node = GTK_CTREE_NODE_NEXT(mimeview->opened);
 		if (node) {
-			gtk_ctree_node_moveto(ctree, node, -1, 0.5, 0);
-			gtk_clist_unselect_all(clist);
-			gtk_ctree_select(ctree, node);
-			gtkut_ctree_set_focus_row(ctree, node);
+			gtk_sctree_unselect_all(GTK_SCTREE(ctree));
+			gtk_sctree_select(GTK_SCTREE(ctree), node);
 			return;
 		}
 		break;
@@ -508,7 +604,6 @@ static void mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 
 		gtk_signal_emit_by_name(GTK_OBJECT(ctree), "scroll_vertical",
 					GTK_SCROLL_STEP_FORWARD, 0.0);
-		gtk_ctree_node_moveto(ctree, mimeview->opened, -1, 0.5, 0);
 		return;
 	case GDK_p:
 	case GDK_P:
@@ -517,7 +612,6 @@ static void mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 
 		gtk_signal_emit_by_name(GTK_OBJECT(ctree), "scroll_vertical",
 					GTK_SCROLL_STEP_BACKWARD, 0.0);
-		gtk_ctree_node_moveto(ctree, mimeview->opened, -1, 0.5, 0);
 		return;
 	case GDK_y:
 		BREAK_ON_MODIFIER_KEY();
@@ -685,3 +779,80 @@ static void mimeview_view_file(const gchar *filename, MimeInfo *partinfo)
 
 	execute_command_line(buf);
 }
+
+#if USE_GPGME
+static void update_node_name(GtkCTree *ctree, GtkCTreeNode *node,
+			     gpointer data)
+{
+	MimeInfo *partinfo;
+	gchar *part_name;
+
+	partinfo = gtk_ctree_node_get_row_data(ctree, node);
+	g_return_if_fail(partinfo != NULL);
+
+	part_name = get_part_name(partinfo);
+	gtk_ctree_node_set_text(ctree, node, COL_NAME, part_name);
+}
+
+static void mimeview_update_names(MimeView *mimeview)
+{
+	GtkCTree *ctree = GTK_CTREE(mimeview->ctree);
+
+	gtk_ctree_pre_recursive(ctree, NULL, update_node_name, NULL);
+}
+
+static void mimeview_update_signature_info(MimeView *mimeview)
+{
+	MimeInfo *partinfo;
+
+	if (!mimeview) return;
+	if (!mimeview->opened) return;
+
+	partinfo = gtk_ctree_node_get_row_data
+		(GTK_CTREE(mimeview->ctree), mimeview->opened);
+	if (!partinfo) return;
+
+	if (g_strcasecmp(partinfo->content_type,
+			 "application/pgp-signature") == 0) {
+		mimeview_change_view_type(mimeview, MIMEVIEW_TEXT);
+		textview_show_signature_part(mimeview->textview, partinfo);
+	}
+}
+
+static void mimeview_check_signature(MimeView *mimeview)
+{
+	MimeInfo *mimeinfo;
+	gchar buf[BUFFSIZE];
+	FILE *fp;
+
+	g_return_if_fail (mimeview_is_signed(mimeview));
+
+	mimeinfo = gtk_ctree_node_get_row_data
+		(GTK_CTREE(mimeview->ctree), mimeview->opened);
+	g_return_if_fail(mimeinfo != NULL);
+	g_return_if_fail(mimeview->file != NULL);
+
+	while (mimeinfo->parent)
+		mimeinfo = mimeinfo->parent;
+
+	if ((fp = fopen(mimeview->file, "r")) == NULL) {
+		FILE_OP_ERROR(mimeview->file, "fopen");
+		return;
+	}
+
+	/* skip headers */
+	if (mimeinfo->mime_type == MIME_MULTIPART) {
+		if (fseek(fp, mimeinfo->fpos, SEEK_SET) < 0)
+		FILE_OP_ERROR(mimeview->file, "fseek");
+		while (fgets(buf, sizeof(buf), fp) != NULL)
+			if (buf[0] == '\r' || buf[0] == '\n') break;
+	}
+
+	procmime_scan_multipart_message(mimeinfo, fp);
+	rfc2015_check_signature(mimeinfo, fp);
+	fclose(fp);
+
+	mimeview_update_names(mimeview);
+	mimeview_update_signature_info(mimeview);
+}
+#endif /*USE_GPGME*/
