@@ -94,12 +94,16 @@ void prefs_read_config(PrefParam *param, const gchar *label,
 void prefs_config_parse_one_line(PrefParam *param, const gchar *buf)
 {
 	gint i;
+	gint name_len;
 	const gchar *value;
 
 	for (i = 0; param[i].name != NULL; i++) {
-		if (strncasecmp(buf, param[i].name, strlen(param[i].name)))
+		name_len = strlen(param[i].name);
+		if (strncasecmp(buf, param[i].name, name_len))
 			continue;
-		value = buf + strlen(param[i].name) + 1;
+		if (buf[name_len] != '=')
+			continue;
+		value = buf + name_len + 1;
 		debug_print("%s = %s\n", param[i].name, value);
 
 		switch (param[i].type) {
@@ -456,7 +460,7 @@ void prefs_dialog_create(PrefsDialog *dialog)
 	gtk_widget_show(notebook);
 	gtk_box_pack_start (GTK_BOX (vbox), notebook, TRUE, TRUE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (notebook), 2);
-	GTK_WIDGET_UNSET_FLAGS (notebook, GTK_CAN_FOCUS);
+	/* GTK_WIDGET_UNSET_FLAGS (notebook, GTK_CAN_FOCUS); */
 	gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
 
 	gtkut_button_set_create(&confirm_area,
