@@ -58,6 +58,7 @@ static struct Basic {
 	GtkWidget *addr_entry;
 	GtkWidget *org_entry;
 
+	GtkWidget *serv_frame;
 	GtkWidget *serv_table;
 	GtkWidget *protocol_optmenu;
 	GtkWidget *inbox_label;
@@ -173,7 +174,7 @@ static PrefParam param[] = {
 	 &receive.recvatgetall_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 
-	{"filter_on_receive", "FALSE", &tmp_ac_prefs.filter_on_recv, P_BOOL,
+	{"filter_on_receive", "TRUE", &tmp_ac_prefs.filter_on_recv, P_BOOL,
 	 &receive.filter_on_recv_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 
@@ -269,9 +270,10 @@ static void prefs_account_ok			(void);
 static gint prefs_account_apply			(void);
 static void prefs_account_cancel		(void);
 
-#define VSPACING	12
-#define VSPACING_NARROW	4
-#define BOX_BORDER	16
+#define VSPACING		12
+#define VSPACING_NARROW		4
+#define BOX_BORDER		16
+#define DEFAULT_ENTRY_WIDTH	80
 
 void prefs_account_read_config(PrefsAccount *ac_prefs, const gchar *label)
 {
@@ -428,7 +430,6 @@ PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 
 static void prefs_account_create(void)
 {
-	GtkStyle *style;
 	gint page = 0;
 
 	debug_print(_("Creating account preferences window...\n"));
@@ -461,19 +462,7 @@ static void prefs_account_create(void)
 	prefs_account_advanced_create();
 	SET_NOTEBOOK_LABEL(dialog.notebook, _("Advanced"), page++);
 
-	gtk_widget_show(dialog.window);
-
-	style = gtk_widget_get_style(dialog.window);
-	if (style && style->font) {
-		gint width, height;
-
-		width = gtkut_get_font_width(style->font);
-		height = gtkut_get_font_height(style->font);
-		gtk_widget_set_usize
-			(dialog.notebook,
-			 width * 58 + BOX_BORDER * 2 + 24,
-			 height * 10 + BOX_BORDER * 2 + VSPACING * 15 + 64);
-	}
+	gtk_widget_show_all(dialog.window);
 }
 
 #define SET_ACTIVATE(menuitem) \
@@ -496,7 +485,7 @@ static void prefs_account_basic_create(void)
 	GtkWidget *addr_entry;
 	GtkWidget *org_entry;
 
-	GtkWidget *frame2;
+	GtkWidget *serv_frame;
 	GtkWidget *vbox2;
 	GtkWidget *optmenu;
 	GtkWidget *optmenu_menu;
@@ -530,6 +519,7 @@ static void prefs_account_basic_create(void)
 
 	acname_entry = gtk_entry_new ();
 	gtk_widget_show (acname_entry);
+	gtk_widget_set_usize (acname_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), acname_entry, TRUE, TRUE, 0);
 
 	default_chkbtn = gtk_check_button_new_with_label (_("Usually used"));
@@ -581,11 +571,11 @@ static void prefs_account_basic_create(void)
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
-	PACK_FRAME (vbox1, frame2, _("Server information"));
+	PACK_FRAME (vbox1, serv_frame, _("Server information"));
 
 	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
 	gtk_widget_show (vbox2);
-	gtk_container_add (GTK_CONTAINER (frame2), vbox2);
+	gtk_container_add (GTK_CONTAINER (serv_frame), vbox2);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
 
 	hbox = gtk_hbox_new (FALSE, 8);
@@ -621,6 +611,7 @@ static void prefs_account_basic_create(void)
 
 	inbox_entry = gtk_entry_new ();
 	gtk_widget_show (inbox_entry);
+	gtk_widget_set_usize (inbox_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), inbox_entry, TRUE, TRUE, 0);
 
 	serv_table = gtk_table_new (5, 4, FALSE);
@@ -650,12 +641,14 @@ static void prefs_account_basic_create(void)
 
 	uid_entry = gtk_entry_new ();
 	gtk_widget_show (uid_entry);
+	gtk_widget_set_usize (uid_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_table_attach (GTK_TABLE (serv_table), uid_entry, 1, 2, 4, 5,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
 	pass_entry = gtk_entry_new ();
 	gtk_widget_show (pass_entry);
+	gtk_widget_set_usize (pass_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_table_attach (GTK_TABLE (serv_table), pass_entry, 3, 4, 4, 5,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
@@ -698,6 +691,7 @@ static void prefs_account_basic_create(void)
 	basic.addr_entry = addr_entry;
 	basic.org_entry  = org_entry;
 
+	basic.serv_frame       = serv_frame;
 	basic.serv_table       = serv_table;
 	basic.protocol_optmenu = optmenu;
 	basic.inbox_label      = inbox_label;
@@ -1165,4 +1159,6 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_set_sensitive(receive.pop3_frame, TRUE);
 		break;
 	}
+
+	gtk_widget_queue_resize(basic.serv_frame);
 }
