@@ -145,15 +145,21 @@ static struct Display {
 static struct Privacy {
 	GtkWidget *checkbtn_default_encrypt;
 	GtkWidget *checkbtn_default_sign;
+	GtkWidget *checkbtn_auto_check_signatures;
 	GtkWidget *checkbtn_passphrase_grab;
+	GtkWidget *optmenu_default_signkey;
 } privacy;
 #endif
 
 static struct Interface {
 	GtkWidget *checkbtn_emacs;
 	GtkWidget *checkbtn_openunread;
+	GtkWidget *checkbtn_openinbox;
 	GtkWidget *checkbtn_immedexec;
 	GtkWidget *checkbtn_confonexit;
+	GtkWidget *checkbtn_smoothscroll;
+	GtkWidget *spinbtn_scrollstep;
+	GtkObject *spinbtn_scrollstep_adj;
 	GtkWidget *checkbtn_cleanonexit;
 	GtkWidget *checkbtn_askonclean;
 } interface;
@@ -166,11 +172,25 @@ static struct Other {
 	GtkWidget *exteditor_entry;
 } other;
 
+static struct MessageColorButtons {
+	GtkWidget *quote_level1_btn;
+	GtkWidget *quote_level2_btn;
+	GtkWidget *quote_level3_btn;
+	GtkWidget *uri_btn;
+} color_buttons;
+
 static GtkWidget *quote_desc_win;
 static GtkWidget *font_sel_win;
+static GtkWidget *quote_color_win;
+static GtkWidget *color_dialog;
 
 static void prefs_common_charset_set_data_from_optmenu(PrefParam *pparam);
 static void prefs_common_charset_set_optmenu	      (PrefParam *pparam);
+#if USE_GPGME
+static void prefs_common_default_signkey_set_data_from_optmenu
+							(PrefParam *pparam);
+static void prefs_common_default_signkey_set_optmenu	(PrefParam *pparam);
+#endif
 
 /*
    parameter name, default value, pointer to the prefs variable, data type,
@@ -277,12 +297,18 @@ static PrefParam param[] = {
 	 &display.chkbtn_enablecol,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 
-	{"quote_color", "0", &prefs_common.quote_col, P_USHORT,
+	{"quote_level1_color", "179", &prefs_common.quote_level1_col, P_INT,
 	 NULL, NULL, NULL},
-	{"uri_color", "0", &prefs_common.uri_col, P_USHORT,
+	{"quote_level2_color", "179", &prefs_common.quote_level2_col, P_INT,
+	 NULL, NULL, NULL},
+	{"quote_level3_color", "179", &prefs_common.quote_level3_col, P_INT,
+	 NULL, NULL, NULL},
+	{"uri_color", "32512", &prefs_common.uri_col, P_INT,
 	 NULL, NULL, NULL},
 	{"signature_color", "0", &prefs_common.sig_col, P_USHORT,
 	 NULL, NULL, NULL},
+	{"recycle_quote_colors", "FALSE", &prefs_common.recycle_quote_colors,
+	 P_BOOL, NULL, NULL, NULL},
 	{"toolbar_style", "3", &prefs_common.toolbar_style, P_ENUM,
 	 NULL, NULL, NULL},
 	{"show_statusbar", "TRUE", &prefs_common.show_statusbar, P_BOOL,
@@ -354,7 +380,7 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"summary_col_mark", "10", &prefs_common.summary_col_mark, P_INT,
 	 NULL, NULL, NULL},
-	{"summary_col_unread", "10", &prefs_common.summary_col_unread, P_INT,
+	{"summary_col_unread", "13", &prefs_common.summary_col_unread, P_INT,
 	 NULL, NULL, NULL},
 	{"summary_col_mime", "10", &prefs_common.summary_col_mime, P_INT,
 	 NULL, NULL, NULL},
@@ -401,9 +427,17 @@ static PrefParam param[] = {
 	{"default_sign", "FALSE", &prefs_common.default_sign, P_BOOL,
 	 &privacy.checkbtn_default_sign,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
+	{"auto_check_signatures", "TRUE",
+	 &prefs_common.auto_check_signatures, P_BOOL,
+	 &privacy.checkbtn_auto_check_signatures,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"passphrase_grab", "FALSE", &prefs_common.passphrase_grab, P_BOOL,
 	 &privacy.checkbtn_passphrase_grab,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
+	{"default_signkey", CS_AUTO, &prefs_common.default_signkey, P_STRING,
+	 &privacy.optmenu_default_signkey,
+	 prefs_common_default_signkey_set_data_from_optmenu,
+	 prefs_common_default_signkey_set_optmenu},
 #endif /* USE_GPGME */
 
 	/* Interface */
@@ -418,9 +452,19 @@ static PrefParam param[] = {
 	{"open_unread_on_enter", "FALSE", &prefs_common.open_unread_on_enter,
 	 P_BOOL, &interface.checkbtn_openunread,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
+	{"open_inbox_on_inc", "TRUE", &prefs_common.open_inbox_on_inc,
+	 P_BOOL, &interface.checkbtn_openinbox,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"immediate_execution", "TRUE", &prefs_common.immediate_exec, P_BOOL,
 	 &interface.checkbtn_immedexec,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
+	{"enable_smooth_scroll", "FALSE",
+	 &prefs_common.enable_smooth_scroll, P_BOOL,
+	 &interface.checkbtn_smoothscroll,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
+	{"scroll_step", "1", &prefs_common.scroll_step, P_INT,
+	 &interface.spinbtn_scrollstep,
+	 prefs_set_data_from_spinbtn, prefs_set_spinbtn},
 
 	{"confirm_on_exit", "TRUE", &prefs_common.confirm_on_exit, P_BOOL,
 	 &interface.checkbtn_confonexit,
@@ -445,10 +489,11 @@ static PrefParam param[] = {
 	{NULL, NULL, NULL, P_OTHER, NULL, NULL, NULL}
 };
 
-#define VSPACING	12
-#define VSPACING_NARROW	4
-#define VBOX_BORDER	16
-#define PREFSBUFSIZE	1024
+#define VSPACING		12
+#define VSPACING_NARROW		4
+#define VBOX_BORDER		16
+#define DEFAULT_ENTRY_WIDTH	80
+#define PREFSBUFSIZE		1024
 
 /* widget creating functions */
 static void prefs_common_create		(void);
@@ -458,13 +503,25 @@ static void prefs_compose_create	(void);
 static void prefs_filter_create		(void);
 static void prefs_display_create	(void);
 #if USE_GPGME
-  static void prefs_privacy_create	(void);
+static void prefs_privacy_create	(void);
 #endif
 static void prefs_interface_create	(void);
 static void prefs_other_create		(void);
 
-static void prefs_quote_description	   (void);
-static void prefs_quote_description_create (void);
+static void prefs_quote_description		(void);
+static void prefs_quote_description_create	(void);
+static void prefs_quote_colors_dialog		(void);
+static void prefs_quote_colors_dialog_create	(void);
+static void quote_color_set_dialog		(GtkWidget	*widget,
+						 gpointer	 data);
+static void quote_colors_set_dialog_ok		(GtkWidget	*widget,
+						 gpointer	 data);
+static void quote_colors_set_dialog_cancel	(GtkWidget	*widget,
+						 gpointer	 data);
+static void set_button_bg_color			(GtkWidget	*widget,
+						 gint		 color);
+static void prefs_enable_message_color_toggled	(void);
+static void prefs_recycle_colors_toggled	(GtkWidget	*widget);
 
 static void prefs_filter_read_config	(void);
 static void prefs_filter_write_config	(void);
@@ -543,7 +600,6 @@ void prefs_common_open(void)
 
 static void prefs_common_create(void)
 {
-	GtkStyle *style;
 	gint page = 0;
 
 	debug_print(_("Creating common preferences window...\n"));
@@ -588,18 +644,6 @@ static void prefs_common_create(void)
 	SET_NOTEBOOK_LABEL(dialog.notebook, _("Other"),     page++);
 
 	gtk_widget_show_all(dialog.window);
-
-	style = gtk_widget_get_style(dialog.window);
-	if (style && style->font) {
-		gint width, height;
-
-		width = gtkut_get_font_width(style->font);
-		height = gtkut_get_font_height(style->font);
-		gtk_widget_set_usize
-			(dialog.notebook,
-			 width * 58 + VBOX_BORDER * 2 + 24,
-			 height * 10 + VBOX_BORDER * 2 + VSPACING * 15 + 64);
-	}
 }
 
 static void prefs_receive_create(void)
@@ -870,17 +914,21 @@ static void prefs_send_create(void)
 	SET_MENUITEM(_("Cyrillic (KOI8-R)"),		 CS_KOI8_R);
 	SET_MENUITEM(_("Cyrillic (Windows-1251)"),	 CS_CP1251);
 	SET_MENUITEM(_("Cyrillic (KOI8-U)"),		 CS_KOI8_U);
-#endif
+#endif /* HAVE_LIBJCONV */
 	SET_MENUITEM(_("Japanese (ISO-2022-JP)"),	 CS_ISO_2022_JP);
+#if 0
 	SET_MENUITEM(_("Japanese (EUC-JP)"),		 CS_EUC_JP);
 	SET_MENUITEM(_("Japanese (Shift_JIS)"),		 CS_SHIFT_JIS);
+#endif /* 0 */
 #if HAVE_LIBJCONV
 	SET_MENUITEM(_("Simplified Chinese (GB2312)"),	 CS_GB2312);
 	SET_MENUITEM(_("Traditional Chinese (Big5)"),	 CS_BIG5);
-	//SET_MENUITEM(_("Traditional Chinese (EUC-TW)"),CS_EUC_TW);
-	//SET_MENUITEM(_("Chinese (ISO-2022-CN)"),	 CS_ISO_2022_CN);
+#if 0
+	SET_MENUITEM(_("Traditional Chinese (EUC-TW)"),  CS_EUC_TW);
+	SET_MENUITEM(_("Chinese (ISO-2022-CN)"),	 CS_ISO_2022_CN);
+#endif /* 0 */
 	SET_MENUITEM(_("Korean (EUC-KR)"),		 CS_EUC_KR);
-#endif
+#endif /* HAVE_LIBJCONV */
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (optmenu), optmenu_menu);
 
@@ -1108,8 +1156,7 @@ static void prefs_filter_create(void)
 	op_label = gtk_label_new (_("Operator"));
 	gtk_widget_show (op_label);
 	gtk_table_attach (GTK_TABLE (table1), op_label, 0, 1, 0, 1,
-			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
-			  0, 0, 0);
+			  GTK_FILL, 0, 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (op_label), 0, 0.5);
 
 	op_combo = gtk_combo_new ();
@@ -1125,8 +1172,7 @@ static void prefs_filter_create(void)
 	hdr_label = gtk_label_new (_("Header"));
 	gtk_widget_show (hdr_label);
 	gtk_table_attach (GTK_TABLE (table1), hdr_label, 1, 2, 0, 1,
-			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
-			  0, 0, 0);
+			  GTK_FILL, 0, 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (hdr_label), 0, 0.5);
 
 	hdr_combo1 = gtk_combo_new ();
@@ -1160,12 +1206,14 @@ static void prefs_filter_create(void)
 
 	key_entry1 = gtk_entry_new ();
 	gtk_widget_show (key_entry1);
+	gtk_widget_set_usize (key_entry1, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_table_attach (GTK_TABLE (table1), key_entry1, 2, 3, 1, 2,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  0, 0, 0);
 
 	key_entry2 = gtk_entry_new ();
 	gtk_widget_show (key_entry2);
+	gtk_widget_set_usize (key_entry2, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_table_attach (GTK_TABLE (table1), key_entry2, 2, 3, 2, 3,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  0, 0, 0);
@@ -1173,8 +1221,7 @@ static void prefs_filter_create(void)
 	pred_label = gtk_label_new (_("Predicate"));
 	gtk_widget_show (pred_label);
 	gtk_table_attach (GTK_TABLE (table1), pred_label, 3, 4, 0, 1,
-			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
-			  0, 0, 0);
+			  GTK_FILL, 0, 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (pred_label), 0, 0.5);
 
 	pred_combo1 = gtk_combo_new ();
@@ -1223,6 +1270,7 @@ static void prefs_filter_create(void)
 
 	dest_entry = gtk_entry_new ();
 	gtk_widget_show (dest_entry);
+	gtk_widget_set_usize (dest_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_box_pack_start (GTK_BOX (dest_hbox), dest_entry, TRUE, TRUE, 0);
 	gtk_entry_set_editable (GTK_ENTRY (dest_entry), FALSE);
 
@@ -1307,7 +1355,6 @@ static void prefs_filter_create(void)
 	btn_vbox = gtk_vbox_new (FALSE, 8);
 	gtk_widget_show (btn_vbox);
 	gtk_box_pack_start (GTK_BOX (cond_hbox), btn_vbox, FALSE, FALSE, 0);
-	gtk_widget_set_usize (btn_vbox, 38, -1);
 
 	up_btn = gtk_button_new_with_label (_("Up"));
 	gtk_widget_show (up_btn);
@@ -1419,13 +1466,17 @@ static void prefs_display_create(void)
 
 	PACK_CHECK_BUTTON (hbox1, chkbtn_enablecol,
 			   _("Enable coloration of message"));
+	gtk_signal_connect (GTK_OBJECT (chkbtn_enablecol), "toggled",
+						GTK_SIGNAL_FUNC (prefs_enable_message_color_toggled),
+						NULL);
 
 	button_edit_col = gtk_button_new_with_label (_(" Edit... "));
 	gtk_widget_show (button_edit_col);
 	gtk_box_pack_end (GTK_BOX (hbox1), button_edit_col, FALSE, TRUE, 0);
+	gtk_signal_connect (GTK_OBJECT (button_edit_col), "clicked",
+			    GTK_SIGNAL_FUNC (prefs_quote_colors_dialog), NULL);
 
-	//SET_TOGGLE_SENSITIVITY(chkbtn_enablecol, button_edit_col);
-	gtk_widget_set_sensitive(button_edit_col, FALSE);
+	SET_TOGGLE_SENSITIVITY(chkbtn_enablecol, button_edit_col);
 
 	PACK_CHECK_BUTTON
 		(vbox2, chkbtn_mbalnum,
@@ -1497,13 +1548,13 @@ static void prefs_display_create(void)
 	display.chkbtn_enablecol = chkbtn_enablecol;
 	display.button_edit_col  = button_edit_col;
 
-	display.chkbtn_transhdr   = chkbtn_transhdr;
-	display.chkbtn_mbalnum    = chkbtn_mbalnum;
-	display.chkbtn_swapfrom   = chkbtn_swapfrom;
-	display.chkbtn_hscrollbar = chkbtn_hscrollbar;
-	display.spinbtn_linespc   = spinbtn_linespc;
-	display.chkbtn_headspc    = chkbtn_headspc;
-	display.chkbtn_disphdr    = chkbtn_disphdr;
+	display.chkbtn_transhdr     = chkbtn_transhdr;
+	display.chkbtn_mbalnum      = chkbtn_mbalnum;
+	display.chkbtn_swapfrom     = chkbtn_swapfrom;
+	display.chkbtn_hscrollbar   = chkbtn_hscrollbar;
+	display.spinbtn_linespc     = spinbtn_linespc;
+	display.chkbtn_headspc      = chkbtn_headspc;
+	display.chkbtn_disphdr      = chkbtn_disphdr;
 }
 
 #if USE_GPGME
@@ -1511,9 +1562,15 @@ static void prefs_privacy_create(void)
 {
 	GtkWidget *vbox1;
 	GtkWidget *vbox2;
+	GtkWidget *hbox1;
 	GtkWidget *checkbtn_default_encrypt;
 	GtkWidget *checkbtn_default_sign;
+	GtkWidget *checkbtn_auto_check_signatures;
 	GtkWidget *checkbtn_passphrase_grab;
+	GtkWidget *label;
+	GtkWidget *menuitem;
+	GtkWidget *optmenu;
+	GtkWidget *optmenu_menu;
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
@@ -1530,12 +1587,83 @@ static void prefs_privacy_create(void)
 	PACK_CHECK_BUTTON (vbox2, checkbtn_default_sign,
 			   _("Sign message by default"));
 
+	PACK_CHECK_BUTTON (vbox2, checkbtn_auto_check_signatures,
+			   _("Automatically check signatures"));
+
 	PACK_CHECK_BUTTON (vbox2, checkbtn_passphrase_grab,
 			   _("Grab input while entering a passphrase"));
 
+	hbox1 = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (hbox1);
+	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
+
+	/* create default signkey box */
+	label = gtk_label_new (_("Default Sign Key"));
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
+	optmenu = gtk_option_menu_new ();
+	gtk_widget_show (optmenu);
+	gtk_box_pack_start(GTK_BOX (hbox1), optmenu, FALSE, FALSE, 0);
+	optmenu_menu = gtk_menu_new ();
+
+	MENUITEM_ADD(optmenu_menu, menuitem, "Default Key", "def_key");
+	MENUITEM_ADD(optmenu_menu, menuitem, "Second Key", "2nd_key");
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (optmenu), optmenu_menu);
+	/* FIXME: disabled because not implemented */
+	gtk_widget_set_sensitive(optmenu, FALSE);
+
 	privacy.checkbtn_default_encrypt = checkbtn_default_encrypt;
 	privacy.checkbtn_default_sign    = checkbtn_default_sign;
+	privacy.checkbtn_auto_check_signatures
+					 = checkbtn_auto_check_signatures;
 	privacy.checkbtn_passphrase_grab = checkbtn_passphrase_grab;
+	privacy.optmenu_default_signkey  = optmenu;
+}
+
+static void
+prefs_common_default_signkey_set_data_from_optmenu(PrefParam *pparam)
+{
+#if 0
+	GtkWidget *menu;
+	GtkWidget *menuitem;
+	gchar *charset;
+
+	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(*pparam->widget));
+	menuitem = gtk_menu_get_active(GTK_MENU(menu));
+	charset = gtk_object_get_user_data(GTK_OBJECT(menuitem));
+	g_free(*((gchar **)pparam->data));
+	*((gchar **)pparam->data) = g_strdup(charset);
+#endif
+}
+
+static void prefs_common_default_signkey_set_optmenu(PrefParam *pparam)
+{
+#if 0
+	GList *cur;
+	GtkOptionMenu *optmenu = GTK_OPTION_MENU(*pparam->widget);
+	GtkWidget *menu;
+	GtkWidget *menuitem;
+	gchar *charset;
+	gint n = 0;
+
+	g_return_if_fail(optmenu != NULL);
+	g_return_if_fail(*((gchar **)pparam->data) != NULL);
+
+	menu = gtk_option_menu_get_menu(optmenu);
+	for (cur = GTK_MENU_SHELL(menu)->children;
+	     cur != NULL; cur = cur->next) {
+		menuitem = GTK_WIDGET(cur->data);
+		charset = gtk_object_get_user_data(GTK_OBJECT(menuitem));
+		if (!strcmp(charset, *((gchar **)pparam->data))) {
+			gtk_option_menu_set_history(optmenu, n);
+			return;
+		}
+		n++;
+	}
+
+	gtk_option_menu_set_history(optmenu, 0);
+	prefs_common_charset_set_data_from_optmenu(pparam);
+#endif
 }
 #endif /* USE_GPGME */
 
@@ -1546,7 +1674,13 @@ static void prefs_interface_create(void)
 	GtkWidget *vbox3;
 	GtkWidget *checkbtn_emacs;
 	GtkWidget *checkbtn_openunread;
+	GtkWidget *checkbtn_openinbox;
 	GtkWidget *checkbtn_immedexec;
+	GtkWidget *checkbtn_smoothscroll;
+	GtkWidget *hbox_scr;
+	GtkWidget *label_scr;
+	GtkObject *spinbtn_scrollstep_adj;
+	GtkWidget *spinbtn_scrollstep;
 	GtkWidget *label;
 
 	GtkWidget *frame_exit;
@@ -1575,6 +1709,10 @@ static void prefs_interface_create(void)
 		(vbox2, checkbtn_openunread,
 		 _("Open first unread message when entering a folder"));
 
+	PACK_CHECK_BUTTON
+		(vbox2, checkbtn_openinbox,
+		 _("Go to inbox after receiving new mail"));
+
 	vbox3 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox3);
 	gtk_box_pack_start (GTK_BOX (vbox2), vbox3, FALSE, FALSE, 0);
@@ -1593,6 +1731,29 @@ static void prefs_interface_create(void)
 	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 8);
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
 
+	PACK_CHECK_BUTTON
+		(vbox2, checkbtn_smoothscroll,
+		 _("Enable smooth scrolling on the message view"));
+
+	hbox_scr = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (hbox_scr);
+	gtk_box_pack_start (GTK_BOX (vbox2), hbox_scr, FALSE, FALSE, 0);
+
+	label_scr = gtk_label_new (_("Scroll step"));
+	gtk_widget_show (label_scr);
+	gtk_box_pack_start (GTK_BOX (hbox_scr), label_scr, FALSE, FALSE, 0);
+
+	spinbtn_scrollstep_adj = gtk_adjustment_new (1, 1, 100, 1, 10, 10);
+	spinbtn_scrollstep = gtk_spin_button_new
+		(GTK_ADJUSTMENT (spinbtn_scrollstep_adj), 1, 0);
+	gtk_widget_show (spinbtn_scrollstep);
+	gtk_box_pack_start (GTK_BOX (hbox_scr), spinbtn_scrollstep,
+			    FALSE, FALSE, 0);
+	gtk_widget_set_usize (spinbtn_scrollstep, 64, -1);
+	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbtn_scrollstep),
+				     TRUE);
+	SET_TOGGLE_SENSITIVITY (checkbtn_smoothscroll, hbox_scr)
+
 	PACK_FRAME (vbox1, frame_exit, _("On exit"));
 
 	vbox_exit = gtk_vbox_new (FALSE, VSPACING_NARROW);
@@ -1608,17 +1769,21 @@ static void prefs_interface_create(void)
 	gtk_box_pack_start (GTK_BOX (vbox_exit), hbox1, FALSE, FALSE, 0);
 
 	PACK_CHECK_BUTTON (hbox1, checkbtn_cleanonexit,
-			   _("Clean trash on exit"));
+			   _("Empty trash on exit"));
 	PACK_CHECK_BUTTON (hbox1, checkbtn_askonclean,
-			   _("Ask before cleaning"));
+			   _("Ask before emptying"));
 	SET_TOGGLE_SENSITIVITY (checkbtn_cleanonexit, checkbtn_askonclean);
 
-	interface.checkbtn_emacs	= checkbtn_emacs;
-	interface.checkbtn_openunread	= checkbtn_openunread;
-	interface.checkbtn_immedexec	= checkbtn_immedexec;
-	interface.checkbtn_confonexit	= checkbtn_confonexit;
-	interface.checkbtn_cleanonexit	= checkbtn_cleanonexit;
-	interface.checkbtn_askonclean	= checkbtn_askonclean;
+	interface.checkbtn_emacs         = checkbtn_emacs;
+	interface.checkbtn_openunread    = checkbtn_openunread;
+	interface.checkbtn_openinbox     = checkbtn_openinbox;
+	interface.checkbtn_immedexec     = checkbtn_immedexec;
+	interface.checkbtn_confonexit    = checkbtn_confonexit;
+	interface.checkbtn_smoothscroll  = checkbtn_smoothscroll;
+	interface.spinbtn_scrollstep     = spinbtn_scrollstep;
+	interface.spinbtn_scrollstep_adj = spinbtn_scrollstep_adj;
+	interface.checkbtn_cleanonexit   = checkbtn_cleanonexit;
+	interface.checkbtn_askonclean    = checkbtn_askonclean;
 }
 
 static void prefs_other_create(void)
@@ -1718,37 +1883,297 @@ static void prefs_other_create(void)
 	other.exteditor_entry = exteditor_entry;
 }
 
+void prefs_quote_colors_dialog(void)
+{
+	if (!quote_color_win)
+		prefs_quote_colors_dialog_create();
+	gtk_widget_show(quote_color_win);
+
+	gtk_main();
+	gtk_widget_hide(quote_color_win);
+
+	textview_update_message_colors();
+}
+
+static void prefs_quote_colors_dialog_create(void)
+{
+	GtkWidget *window;
+	GtkWidget *vbox;
+	GtkWidget *table;
+	GtkWidget *quotelevel1_label;
+	GtkWidget *quotelevel2_label;
+	GtkWidget *quotelevel3_label;
+	GtkWidget *uri_label;
+	GtkWidget *hbbox;
+	GtkWidget *ok_btn;
+	//GtkWidget *cancel_btn;
+	GtkWidget *recycle_colors_btn;
+	GtkWidget *frame_colors;
+
+	window = gtk_window_new(GTK_WINDOW_DIALOG);
+	gtk_container_set_border_width(GTK_CONTAINER(window), 2);
+	gtk_window_set_title(GTK_WINDOW(window), _("Set message colors"));
+	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
+	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
+	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, FALSE);
+
+	vbox = gtk_vbox_new (FALSE, VSPACING);
+	gtk_container_add (GTK_CONTAINER (window), vbox);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 8);
+	PACK_FRAME(vbox, frame_colors, _("Colors"));
+
+	table = gtk_table_new (4, 2, FALSE);
+	gtk_container_add (GTK_CONTAINER (frame_colors), table);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 8);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 2);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 4);
+
+/*	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0); */
+
+	color_buttons.quote_level1_btn = gtk_button_new();
+	gtk_table_attach(GTK_TABLE (table), color_buttons.quote_level1_btn, 0, 1, 0, 1,
+					(GtkAttachOptions) (0),
+					(GtkAttachOptions) (0), 0, 0);
+	gtk_widget_set_usize (color_buttons.quote_level1_btn, 40, 30);
+	gtk_container_set_border_width (GTK_CONTAINER (color_buttons.quote_level1_btn), 5);
+
+	color_buttons.quote_level2_btn = gtk_button_new();
+	gtk_table_attach(GTK_TABLE (table), color_buttons.quote_level2_btn, 0, 1, 1, 2,
+					(GtkAttachOptions) (0),
+					(GtkAttachOptions) (0), 0, 0);
+	gtk_widget_set_usize (color_buttons.quote_level2_btn, 40, 30);
+	gtk_container_set_border_width (GTK_CONTAINER (color_buttons.quote_level2_btn), 5);
+
+	color_buttons.quote_level3_btn = gtk_button_new_with_label ("");
+	gtk_table_attach(GTK_TABLE (table), color_buttons.quote_level3_btn, 0, 1, 2, 3,
+					(GtkAttachOptions) (0),
+					(GtkAttachOptions) (0), 0, 0);
+	gtk_widget_set_usize (color_buttons.quote_level3_btn, 40, 30);
+	gtk_container_set_border_width (GTK_CONTAINER (color_buttons.quote_level3_btn), 5);
+
+	color_buttons.uri_btn = gtk_button_new_with_label ("");
+	gtk_table_attach(GTK_TABLE (table), color_buttons.uri_btn, 0, 1, 3, 4,
+					(GtkAttachOptions) (0),
+					(GtkAttachOptions) (0), 0, 0);
+	gtk_widget_set_usize (color_buttons.uri_btn, 40, 30);
+	gtk_container_set_border_width (GTK_CONTAINER (color_buttons.uri_btn), 5);
+
+	quotelevel1_label = gtk_label_new (_("Quoted Text - First Level"));
+	gtk_table_attach(GTK_TABLE (table), quotelevel1_label, 1, 2, 0, 1,
+					(GTK_EXPAND | GTK_FILL), 0, 0, 0);
+	gtk_label_set_justify (GTK_LABEL (quotelevel1_label), GTK_JUSTIFY_LEFT);
+	gtk_misc_set_alignment (GTK_MISC (quotelevel1_label), 0, 0.5);
+
+	quotelevel2_label = gtk_label_new (_("Quoted Text - Second Level"));
+	gtk_table_attach(GTK_TABLE (table), quotelevel2_label, 1, 2, 1, 2,
+					(GTK_EXPAND | GTK_FILL), 0, 0, 0);
+	gtk_label_set_justify (GTK_LABEL (quotelevel2_label), GTK_JUSTIFY_LEFT);
+	gtk_misc_set_alignment (GTK_MISC (quotelevel2_label), 0, 0.5);
+
+	quotelevel3_label = gtk_label_new (_("Quoted Text - Third Level"));
+	gtk_table_attach(GTK_TABLE (table), quotelevel3_label, 1, 2, 2, 3,
+					(GTK_EXPAND | GTK_FILL), 0, 0, 0);
+	gtk_label_set_justify (GTK_LABEL (quotelevel3_label), GTK_JUSTIFY_LEFT);
+	gtk_misc_set_alignment (GTK_MISC (quotelevel3_label), 0, 0.5);
+
+	uri_label = gtk_label_new (_("URI link"));
+	gtk_table_attach(GTK_TABLE (table), uri_label, 1, 2, 3, 4,
+					(GTK_EXPAND | GTK_FILL), 0, 0, 0);
+	gtk_label_set_justify (GTK_LABEL (uri_label), GTK_JUSTIFY_LEFT);
+	gtk_misc_set_alignment (GTK_MISC (uri_label), 0, 0.5);
+
+	PACK_CHECK_BUTTON (vbox, recycle_colors_btn,
+			   _("Recycle quote colors"));
+
+	gtkut_button_set_create(&hbbox, &ok_btn, _("OK"),
+				NULL, NULL, NULL, NULL);
+	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
+
+	gtk_widget_grab_default(ok_btn);
+	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
+					GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
+					GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+
+	gtk_signal_connect(GTK_OBJECT(color_buttons.quote_level1_btn), "clicked",
+			   GTK_SIGNAL_FUNC(quote_color_set_dialog), "LEVEL1");
+	gtk_signal_connect(GTK_OBJECT(color_buttons.quote_level2_btn), "clicked",
+			   GTK_SIGNAL_FUNC(quote_color_set_dialog), "LEVEL2");
+	gtk_signal_connect(GTK_OBJECT(color_buttons.quote_level3_btn), "clicked",
+			   GTK_SIGNAL_FUNC(quote_color_set_dialog), "LEVEL3");
+	gtk_signal_connect(GTK_OBJECT(color_buttons.uri_btn), "clicked",
+			   GTK_SIGNAL_FUNC(quote_color_set_dialog), "URI");
+	gtk_signal_connect(GTK_OBJECT(recycle_colors_btn), "toggled",
+			   GTK_SIGNAL_FUNC(prefs_recycle_colors_toggled), NULL);
+
+	/* show message button colors and recycle options */
+	set_button_bg_color(color_buttons.quote_level1_btn,
+			    prefs_common.quote_level1_col);
+	set_button_bg_color(color_buttons.quote_level2_btn,
+			    prefs_common.quote_level2_col);
+	set_button_bg_color(color_buttons.quote_level3_btn,
+			    prefs_common.quote_level3_col);
+	set_button_bg_color(color_buttons.uri_btn,
+			    prefs_common.uri_col);
+	gtk_toggle_button_set_active((GtkToggleButton *)recycle_colors_btn,
+				     prefs_common.recycle_quote_colors);
+
+	gtk_widget_show_all(vbox);
+	quote_color_win = window;
+}
+
+static void quote_color_set_dialog(GtkWidget *widget, gpointer data)
+{
+	gchar *type = (gchar *)data;
+	gchar *title = NULL;
+	gdouble color[4] = {0.0, 0.0, 0.0, 0.0};
+	gint rgbvalue = 0;
+	GtkColorSelectionDialog *dialog;
+
+	if(g_strcasecmp(type, "LEVEL1") == 0) {
+		title = _("Pick color for quotation level 1");
+		rgbvalue = prefs_common.quote_level1_col;
+	} else if(g_strcasecmp(type, "LEVEL2") == 0) {
+		title = _("Pick color for quotation level 2");
+		rgbvalue = prefs_common.quote_level2_col;
+	} else if(g_strcasecmp(type, "LEVEL3") == 0) {
+		title = _("Pick color for quotation level 3");
+		rgbvalue = prefs_common.quote_level3_col;
+	} else if(g_strcasecmp(type, "URI") == 0) {
+		title = _("Pick color for URI");
+		rgbvalue = prefs_common.uri_col;
+	} else {   /* Should never be called */
+		fprintf(stderr, "Unrecognized datatype '%s' in quote_color_set_dialog\n", type);
+		return;
+	}
+
+	color_dialog = gtk_color_selection_dialog_new(title);
+	gtk_window_set_position(GTK_WINDOW(color_dialog), GTK_WIN_POS_CENTER);
+	gtk_window_set_modal(GTK_WINDOW(color_dialog), TRUE);
+	gtk_window_set_policy(GTK_WINDOW(color_dialog), FALSE, FALSE, FALSE);
+
+	gtk_signal_connect(GTK_OBJECT(((GtkColorSelectionDialog *)color_dialog)->ok_button),
+			   "clicked", GTK_SIGNAL_FUNC(quote_colors_set_dialog_ok), data);
+	gtk_signal_connect(GTK_OBJECT(((GtkColorSelectionDialog *)color_dialog)->cancel_button),
+			   "clicked", GTK_SIGNAL_FUNC(quote_colors_set_dialog_cancel), data);
+
+	/* preselect the previous color in the color selection dialog */
+	color[0] = (gdouble) ((rgbvalue & 0xff0000) >> 16) / 255.0;
+	color[1] = (gdouble) ((rgbvalue & 0x00ff00) >>  8) / 255.0;
+	color[2] = (gdouble)  (rgbvalue & 0x0000ff)        / 255.0;
+	dialog = (GtkColorSelectionDialog *)color_dialog;
+	gtk_color_selection_set_color
+		((GtkColorSelection *)dialog->colorsel, color);
+
+	gtk_widget_show(color_dialog);
+}
+
+static void quote_colors_set_dialog_ok(GtkWidget *widget, gpointer data)
+{
+	GtkColorSelection *colorsel = (GtkColorSelection *)
+						((GtkColorSelectionDialog *)color_dialog)->colorsel;
+	gdouble color[4];
+	gint red, green, blue, rgbvalue;
+	gchar *type = (gchar *)data;
+
+	gtk_color_selection_get_color(colorsel, color);
+
+	red      = (gint) (color[0] * 255.0);
+	green    = (gint) (color[1] * 255.0);
+	blue     = (gint) (color[2] * 255.0);
+	rgbvalue = (gint) ((red * 0x10000) | (green * 0x100) | blue);
+
+#if 0
+	fprintf(stderr, "redc = %f, greenc = %f, bluec = %f\n", color[0], color[1], color[2]);
+	fprintf(stderr, "red = %d, green = %d, blue = %d\n", red, green, blue);
+	fprintf(stderr, "Color is %x\n", rgbvalue);
+#endif
+
+	if (g_strcasecmp(type, "LEVEL1") == 0) {
+		prefs_common.quote_level1_col = rgbvalue;
+		set_button_bg_color(color_buttons.quote_level1_btn, rgbvalue);
+	} else if (g_strcasecmp(type, "LEVEL2") == 0) {
+		prefs_common.quote_level2_col = rgbvalue;
+		set_button_bg_color(color_buttons.quote_level2_btn, rgbvalue);
+	} else if (g_strcasecmp(type, "LEVEL3") == 0) {
+		prefs_common.quote_level3_col = rgbvalue;
+		set_button_bg_color(color_buttons.quote_level3_btn, rgbvalue);
+	} else if (g_strcasecmp(type, "URI") == 0) {
+		prefs_common.uri_col = rgbvalue;
+		set_button_bg_color(color_buttons.uri_btn, rgbvalue);
+	} else
+		fprintf( stderr, "Unrecognized datatype '%s' in quote_color_set_dialog_ok\n", type );
+
+	gtk_widget_hide(color_dialog);
+}
+
+static void quote_colors_set_dialog_cancel(GtkWidget *widget, gpointer data)
+{
+	gtk_widget_hide(color_dialog);
+}
+
+static void set_button_bg_color(GtkWidget *widget, gint rgbvalue)
+{
+	GtkStyle *newstyle;
+	GdkColor color;
+
+	gtkut_convert_int_to_gdk_color(rgbvalue, &color);
+	newstyle = gtk_style_copy(gtk_widget_get_default_style());
+	newstyle->bg[GTK_STATE_NORMAL]   = color;
+	newstyle->bg[GTK_STATE_PRELIGHT] = color;
+	newstyle->bg[GTK_STATE_ACTIVE]   = color;
+
+	gtk_widget_set_style(GTK_WIDGET(widget), newstyle);
+}
+
+static void prefs_enable_message_color_toggled(void)
+{
+	gboolean is_active;
+
+	is_active = gtk_toggle_button_get_active((GtkToggleButton *)
+				display.chkbtn_enablecol);
+	gtk_widget_set_sensitive(display.button_edit_col, is_active);
+	prefs_common.enable_color = is_active;
+}
+
+static void prefs_recycle_colors_toggled(GtkWidget *widget)
+{
+	gboolean is_active;
+
+	is_active = gtk_toggle_button_get_active((GtkToggleButton *) widget);
+	prefs_common.recycle_quote_colors = is_active;
+}
+
 static void prefs_quote_description(void)
 {
 	if (!quote_desc_win)
 		prefs_quote_description_create();
 
-	if (GTK_WIDGET_VISIBLE(quote_desc_win))
-		gtk_widget_hide(quote_desc_win);
-	else
-		gtk_widget_show(quote_desc_win);
+	gtk_widget_show(quote_desc_win);
+	gtk_main();
+	gtk_widget_hide(quote_desc_win);
 }
 
 static void prefs_quote_description_create(void)
 {
 	GtkWidget *vbox;
 	GtkWidget *hbox;
+	GtkWidget *hbbox;
 	GtkWidget *label;
-	/* GtkWidget *ok_btn; */
+	GtkWidget *ok_btn;
 
 	quote_desc_win = gtk_window_new(GTK_WINDOW_DIALOG);
 	gtk_window_set_title(GTK_WINDOW(quote_desc_win),
 			     _("Description of symbols"));
 	gtk_container_set_border_width(GTK_CONTAINER(quote_desc_win), 8);
-	gtk_signal_connect(GTK_OBJECT(quote_desc_win), "delete_event",
-			   GTK_SIGNAL_FUNC(gtk_widget_hide_on_delete), NULL);
+	gtk_window_set_position(GTK_WINDOW(quote_desc_win), GTK_WIN_POS_CENTER);
+	gtk_window_set_modal(GTK_WINDOW(quote_desc_win), TRUE);
+	gtk_window_set_policy(GTK_WINDOW(quote_desc_win), FALSE, TRUE, FALSE);
 
 	vbox = gtk_vbox_new(FALSE, 8);
-	gtk_widget_show(vbox);
 	gtk_container_add(GTK_CONTAINER(quote_desc_win), vbox);
 
 	hbox = gtk_hbox_new(FALSE, 4);
-	gtk_widget_show(hbox);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
 	label = gtk_label_new
@@ -1761,7 +2186,7 @@ static void prefs_quote_description_create(void)
 		 "%t:\n"
 		 "%i:\n"
 		 "%%:");
-	gtk_widget_show(label);
+
 	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
 	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
 
@@ -1775,18 +2200,22 @@ static void prefs_quote_description_create(void)
 		   "To\n"
 		   "Message-ID\n"
 		   "%"));
-	gtk_widget_show(label);
+
 	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
 	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
 
-#if 0
-	ok_btn = gtk_button_new_with_label(_("OK"));
-	gtk_widget_show(ok_btn);
-	gtk_box_pack_end(GTK_BOX(vbox), ok_btn, FALSE, FALSE, 0);
-	gtk_signal_connect_object(GTK_OBJECT(ok_btn), "clicked",
-				  GTK_SIGNAL_FUNC(gtk_widget_hide_on_delete),
-				  GTK_OBJECT(quote_desc_win));
-#endif
+	gtkut_button_set_create(&hbbox, &ok_btn, _("OK"),
+				NULL, NULL, NULL, NULL);
+	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
+
+	gtk_widget_grab_default(ok_btn);
+	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
+				  GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+
+	gtk_signal_connect(GTK_OBJECT(quote_desc_win), "delete_event",
+					  GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+
+	gtk_widget_show_all(vbox);
 }
 
 /* filtering functions */
