@@ -510,9 +510,10 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 	t.tm_year = year - 1900;
 	t.tm_wday = 0;
 	t.tm_yday = 0;
-	t.tm_isdst = 0;
+	t.tm_isdst = -1;
 
 	timer = mktime(&t);
+	timer += tzoffset_sec(&timer) - remote_tzoffset_sec(zone);
 
 	if (dest)
 		procheader_date_get_localtime(dest, len, timer);
