@@ -25,10 +25,13 @@
 
 #include "procmime.h"
 
-void rfc2015_secure_remove ( const char *fname );
-void rfc2015_check_signature (MimeInfo *mimeinfo, FILE *fp );
-int rfc2015_is_encrypted (MimeInfo *mimeinfo );
-void rfc2015_decrypt_message (MsgInfo *msginfo, MimeInfo *mimeinfo, FILE *fp );
-int rfc2015_encrypt ( const char *file, GSList *recp_list);
+void rfc2015_secure_remove (const char *fname);
+MimeInfo * rfc2015_find_signature (MimeInfo *mimeinfo);
+gboolean rfc2015_has_signature (MimeInfo *mimeinfo);
+void rfc2015_check_signature (MimeInfo *mimeinfo, FILE *fp);
+int rfc2015_is_encrypted (MimeInfo *mimeinfo);
+void rfc2015_decrypt_message (MsgInfo *msginfo, MimeInfo *mimeinfo, FILE *fp);
+int rfc2015_encrypt (const char *file, GSList *recp_list);
+int rfc2015_sign (const char *file);
 
 #endif /* __RFC2015_H__ */
