@@ -22,6 +22,7 @@
 #include <string.h>
 
 #include "smtp.h"
+#include "esmtp.h"
 #include "socket.h"
 #include "utils.h"
 
@@ -30,18 +31,43 @@
 static gint verbose = 1;
 static gchar smtp_response[MSGBUFSIZE];
 
-gint smtp_helo(gint sock, const char *hostname)
+gint smtp_helo(gint sock, const char *hostname, gboolean use_smtp_auth)
 {
-	sock_printf(sock, "HELO %s\r\n", hostname);
-	if (verbose)
-		log_print("SMTP> HELO %s\n", hostname);
+	if (use_smtp_auth) {
+		sock_printf(sock, "EHLO %s\r\n", hostname);
+		if (verbose)
+			log_print("ESMTP> EHLO %s\n", hostname);
 
-	return smtp_ok(sock);
+		return esmtp_ok(sock);
+	} else {
+		sock_printf(sock, "HELO %s\r\n", hostname);
+		if (verbose)
+			log_print("SMTP> HELO %s\n", hostname);
+
+		return smtp_ok(sock);
+	}
 }
 
-gint smtp_from(gint sock, const gchar *from)
+gint smtp_from(gint sock, const gchar *from,
+	       const gchar *userid, const gchar *passwd,
+	       gboolean use_smtp_auth)
 {
 	gchar buf[MSGBUFSIZE];
+	SMTPAuthType authtype;
+
+	if (use_smtp_auth) {
+		/* exist AUTH-Type CRAM_MD5 */
+		if (esmtp_auth_cram_md5(sock) == SM_ERROR) {
+			/* exist AUTH-Type LOGIN */
+			if (esmtp_auth_login(sock) == SM_ERROR)
+				return SM_ERROR;
+			else
+				authtype = SMTPAUTH_LOGIN;
+		} else
+			authtype = SMTPAUTH_CRAM_MD5;
+
+		esmtp_auth(sock, authtype, userid, passwd, use_smtp_auth);
+	}
 
 	if (strchr(from, '<'))
 		g_snprintf(buf, sizeof(buf), "MAIL FROM: %s", from);
@@ -123,6 +149,9 @@ gint smtp_ok(gint sock)
 			return SM_OK;
 		else if (smtp_response[3] != '-')
 			return SM_ERROR;
+			else if (smtp_response[0] == '5' && smtp_response[1] == '0' &&
+				 (smtp_response[3] == '4' || smtp_response[3] == '3' || smtp_response[3] == '1'))
+				return SM_ERROR;
 	}
 
 	return SM_UNRECOVERABLE;
