@@ -196,41 +196,42 @@ static gint sock_connect_by_hostname(gint sock, const gchar *hostname,
 #else /* INET6 */
 static gint sock_connect_by_getaddrinfo(const gchar *hostname, gushort	port)
 {
-	gint sock;
-	struct addrinfo hints, *res, *res0;
-	gchar strport[6];
+	gint sock = -1, gai_error;
+	struct addrinfo hints, *res, *ai;
+	gchar port_str[6];
 
 	memset(&hints, 0, sizeof(hints));
-
-	hints.ai_flags = AI_CANONNAME;
+	/* hints.ai_flags = AI_CANONNAME; */
 	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_STREAM;
+	hints.ai_protocol = IPPROTO_TCP;
 
 	/* convert port from integer to string. */
-	g_snprintf(strport, sizeof(strport), "%d", port);
+	g_snprintf(port_str, sizeof(port_str), "%d", port);
 
-	if (getaddrinfo(hostname, strport, &hints, &res0)) {
-		perror("getaddrinfo");
+	if ((gai_error = getaddrinfo(hostname, port_str, &hints, &res)) != 0) {
+		fprintf(stderr, "getaddrinfo for %s:%s failed: %s\n",
+			hostname, port_str, gai_strerror(gai_error));
 		return -1;
 	}
 
-	res = res0;
-	do {
-		sock = socket(res->ai_family, res->ai_socktype,
-			      res->ai_protocol);
+	for (ai = res; ai != NULL; ai = ai->ai_next) {
+		sock = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
 		if (sock < 0)
 			continue;
-		if (connect(sock, res->ai_addr, res->ai_addrlen) == 0)
+
+		if (connect(sock, ai->ai_addr, ai->ai_addrlen) == 0)
 			break;
-	} while ((res = res->ai_next) != NULL);
 
-	freeaddrinfo(res0);
-
-	if (res == NULL) {
-		perror("connect");
-		return -1;
+		close(sock);
 	}
 
+	if (res != NULL)
+		freeaddrinfo(res);
+
+	if (ai == NULL)
+		return -1;
+
 	return sock;
 }
 #endif /* !INET6 */
