@@ -60,6 +60,10 @@ gboolean sock_is_nonblocking_mode	(gint sock);
 SockInfo *sock_connect_nb		(const gchar *hostname, gushort port);
 SockInfo *sock_connect			(const gchar *hostname, gushort port);
 
+gint sock_connect_unix	(const gchar *path);
+gint sock_open_unix	(const gchar *path);
+gint sock_accept	(gint sock);
+
 #if USE_THREADS
 SockInfo *sock_connect_with_thread	(const gchar *hostname, gushort port);
 #endif
