@@ -21,7 +21,6 @@
 
 #include <glib.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtkbindings.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkpixmap.h>
@@ -71,9 +70,12 @@
 #include "folder.h"
 
 #include "pixmaps/dir-open.xpm"
-//#include "regular.xpm"
+#include "pixmaps/mark.xpm"
+#include "pixmaps/deleted.xpm"
 #include "pixmaps/new.xpm"
 #include "pixmaps/unread.xpm"
+#include "pixmaps/replied.xpm"
+#include "pixmaps/forwarded.xpm"
 #include "pixmaps/clip.xpm"
 
 #define STATUSBAR_PUSH(mainwin, str) \
@@ -89,16 +91,29 @@
 			  mainwin->summaryview_cid); \
 }
 
+#define SUMMARY_COL_MARK_WIDTH		10
+#define SUMMARY_COL_UNREAD_WIDTH	13
+#define SUMMARY_COL_MIME_WIDTH		10
+
 static GdkFont *smallfont;
 
 static GdkPixmap *folderxpm;
 static GdkBitmap *folderxpmmask;
-//static GdkPixmap *regularxpm;
-//static GdkBitmap *regularxpmmask;
+
+static GdkPixmap *markxpm;
+static GdkBitmap *markxpmmask;
+static GdkPixmap *deletedxpm;
+static GdkBitmap *deletedxpmmask;
+
 static GdkPixmap *newxpm;
 static GdkBitmap *newxpmmask;
 static GdkPixmap *unreadxpm;
 static GdkBitmap *unreadxpmmask;
+static GdkPixmap *repliedxpm;
+static GdkBitmap *repliedxpmmask;
+static GdkPixmap *forwardedxpm;
+static GdkBitmap *forwardedxpmmask;
+
 static GdkPixmap *clipxpm;
 static GdkBitmap *clipxpmmask;
 
@@ -117,6 +132,9 @@ static void summary_set_menu_sensitive	(SummaryView		*summaryview);
 static GtkCTreeNode *summary_find_next_unread_msg
 					(SummaryView		*summaryview,
 					 GtkCTreeNode		*current_node);
+static GtkCTreeNode *summary_find_msg_by_msgnum
+					(SummaryView		*summaryview,
+					 guint			 msgnum);
 #if 0
 static GtkCTreeNode *summary_find_prev_unread_msg
 					(SummaryView		*summaryview,
@@ -135,10 +153,16 @@ static void summary_display_msg		(SummaryView		*summaryview,
 					 GtkCTreeNode		*row,
 					 gboolean		 new_window);
 static void summary_toggle_view		(SummaryView		*summaryview);
+static void summary_set_row_marks	(SummaryView		*summaryview,
+					 GtkCTreeNode		*row);
 
 /* message handling */
 static void summary_mark_row		(SummaryView		*summaryview,
 					 GtkCTreeNode		*row);
+static void summary_mark_row_as_read	(SummaryView		*summaryview,
+					 GtkCTreeNode		*row);
+static void summary_mark_row_as_unread	(SummaryView		*summaryview,
+					 GtkCTreeNode		*row);
 static void summary_delete_row		(SummaryView		*summaryview,
 					 GtkCTreeNode		*row);
 static void summary_unmark_row		(SummaryView		*summaryview,
@@ -146,6 +170,9 @@ static void summary_unmark_row		(SummaryView		*summaryview,
 static void summary_move_row_to		(SummaryView		*summaryview,
 					 GtkCTreeNode		*row,
 					 FolderItem		*to_folder);
+static void summary_copy_row_to		(SummaryView		*summaryview,
+					 GtkCTreeNode		*row,
+					 FolderItem		*to_folder);
 
 static void summary_delete_duplicated_func
 					(GtkCTree		*ctree,
@@ -156,6 +183,10 @@ static void summary_execute_move	(SummaryView		*summaryview);
 static void summary_execute_move_func	(GtkCTree		*ctree,
 					 GtkCTreeNode		*node,
 					 gpointer		 data);
+static void summary_execute_copy	(SummaryView		*summaryview);
+static void summary_execute_copy_func	(GtkCTree		*ctree,
+					 GtkCTreeNode		*node,
+					 gpointer		 data);
 static void summary_execute_delete	(SummaryView		*summaryview);
 static void summary_execute_delete_func	(GtkCTree		*ctree,
 					 GtkCTreeNode		*node,
@@ -255,9 +286,16 @@ GtkTargetEntry summary_drag_types[1] =
 static GtkItemFactoryEntry summary_popup_entries[] =
 {
 	{N_("/M_ove..."),		NULL, summary_move_to,	0, NULL},
-	{N_("/_Copy..."),		NULL, NULL,		0, NULL},
+	{N_("/_Copy..."),		NULL, summary_copy_to,	0, NULL},
 	{N_("/_Delete"),		NULL, summary_delete,	0, NULL},
 	{N_("/E_xecute"),		NULL, summary_execute,	0, NULL},
+	{N_("/_Mark"),			NULL, NULL,		0, "<Branch>"},
+	{N_("/_Mark/_Mark"),		NULL, summary_mark,	0, NULL},
+	{N_("/_Mark/_Unmark"),		NULL, summary_unmark,	0, NULL},
+	{N_("/_Mark/---"),		NULL, NULL,		0, "<Separator>"},
+	{N_("/_Mark/Mark as unr_ead"),	NULL, summary_mark_as_unread, 0, NULL},
+	{N_("/_Mark/Make it as _being read"),
+					NULL, summary_mark_as_read, 0, NULL},
 	{N_("/---"),			NULL, NULL,		0, "<Separator>"},
 	{N_("/_Reply"),			NULL, summary_reply_cb,	COMPOSE_REPLY, NULL},
 	{N_("/Reply to a_ll"),		NULL, summary_reply_cb,	COMPOSE_REPLY_TO_ALL, NULL},
@@ -273,15 +311,6 @@ static GtkItemFactoryEntry summary_popup_entries[] =
 	{N_("/_Save as..."),		NULL, summary_save_as,	0, NULL},
 	{N_("/_Print..."),		NULL, summary_print,	0, NULL},
 	{N_("/---"),			NULL, NULL,		0, "<Separator>"},
-	{N_("/_Mark"),			NULL, NULL,		0, "<Branch>"},
-	{N_("/_Mark/_Mark"),		NULL, summary_mark,	0, NULL},
-	{N_("/_Mark/_Unmark"),		NULL, summary_unmark,	0, NULL},
-	{N_("/_Mark/---"),		NULL, NULL,		0, "<Separator>"},
-	{N_("/_Mark/Mark as unr_ead"),	NULL, summary_mark_as_unread, 0, NULL},
-	{N_("/_Mark/Mark as _important"),
-					NULL, NULL,		0, NULL},
-	{N_("/_Mark/Make it as _being read"),
-					NULL, summary_mark_as_read, 0, NULL},
 	{N_("/Select _all"),		NULL, summary_select_all, 0, NULL}
 };
 
@@ -300,7 +329,6 @@ SummaryView *summary_create(void)
 	GtkWidget *toggle_arrow;
 	GtkWidget *popupmenu;
 	GtkItemFactory *popupfactory;
-	GtkBindingSet *binding_set;
 	gint n_entries;
 	gint i;
 
@@ -337,7 +365,7 @@ SummaryView *summary_create(void)
 	gtk_scrolled_window_set_vadjustment(GTK_SCROLLED_WINDOW(scrolledwin),
 					    GTK_CLIST(ctree)->vadjustment);
 	gtk_container_add(GTK_CONTAINER(scrolledwin), ctree);
-	//gtk_clist_set_selection_mode(GTK_CLIST(ctree), GTK_SELECTION_EXTENDED);
+	gtk_clist_set_selection_mode(GTK_CLIST(ctree), GTK_SELECTION_EXTENDED);
 	gtk_clist_set_column_justification(GTK_CLIST(ctree), S_COL_MARK,
 					   GTK_JUSTIFY_CENTER);
 	gtk_clist_set_column_justification(GTK_CLIST(ctree), S_COL_UNREAD,
@@ -349,11 +377,11 @@ SummaryView *summary_create(void)
 	gtk_clist_set_column_justification(GTK_CLIST(ctree), S_COL_SIZE,
 					   GTK_JUSTIFY_RIGHT);
 	gtk_clist_set_column_width(GTK_CLIST(ctree), S_COL_MARK,
-				   prefs_common.summary_col_mark);
+				   SUMMARY_COL_MARK_WIDTH);
 	gtk_clist_set_column_width(GTK_CLIST(ctree), S_COL_UNREAD,
-				   prefs_common.summary_col_unread);
+				   SUMMARY_COL_UNREAD_WIDTH);
 	gtk_clist_set_column_width(GTK_CLIST(ctree), S_COL_MIME,
-				   prefs_common.summary_col_mime);
+				   SUMMARY_COL_MIME_WIDTH);
 	gtk_clist_set_column_width(GTK_CLIST(ctree), S_COL_NUMBER,
 				   prefs_common.summary_col_number);
 	gtk_clist_set_column_width(GTK_CLIST(ctree), S_COL_SIZE,
@@ -373,12 +401,8 @@ SummaryView *summary_create(void)
 				     GTK_CTREE_EXPANDER_TRIANGLE);
 #endif
 	gtk_ctree_set_indent(GTK_CTREE(ctree), 18);
-	//gtk_clist_set_reorderable(GTK_CLIST(ctree), TRUE);
 	gtk_object_set_user_data(GTK_OBJECT(ctree), summaryview);
 
-	/* don't expand tree with double-click */
-	//GTK_CLIST(ctree)->button_actions[0] &= ~GTK_BUTTON_EXPANDS;
-
 	/* don't let title buttons take key focus */
 	for (i = 0; i < N_SUMMARY_COLS; i++)
 		GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(ctree)->column[i].button,
@@ -436,20 +460,6 @@ SummaryView *summary_create(void)
 				      "<SummaryView>", &popupfactory,
 				      summaryview);
 
-	/* bind keys */
-	binding_set = gtk_binding_set_by_class
-		(GTK_CLIST_CLASS(GTK_OBJECT(ctree)->klass));
-
-	gtk_binding_entry_add_signal(binding_set, GDK_n, GDK_CONTROL_MASK,
-				     "scroll_vertical", 2,
-				     GTK_TYPE_ENUM, GTK_SCROLL_STEP_FORWARD,
-				     GTK_TYPE_FLOAT, 0.0);
-	gtk_binding_entry_add_signal(binding_set, GDK_p, GDK_CONTROL_MASK,
-				     "scroll_vertical", 2,
-				     GTK_TYPE_ENUM, GTK_SCROLL_STEP_BACKWARD,
-				     GTK_TYPE_FLOAT, 0.0);
-	gtk_binding_entry_clear(binding_set, GDK_space, 0);
-
 	/* connect signals */
 	gtk_signal_connect(GTK_OBJECT(ctree), "tree_select_row",
 			   GTK_SIGNAL_FUNC(summary_selected), summaryview);
@@ -496,10 +506,15 @@ void summary_init(SummaryView *summaryview)
 	GtkStyle *style;
 	GtkWidget *pixmap;
 
-	//PIXMAP_CREATE(summaryview->ctree, regularxpm, regularxpmmask,
-	//		regular_xpm);
+	PIXMAP_CREATE(summaryview->ctree, markxpm, markxpmmask, mark_xpm);
+	PIXMAP_CREATE(summaryview->ctree, deletedxpm, deletedxpmmask,
+		      deleted_xpm);
 	PIXMAP_CREATE(summaryview->ctree, newxpm, newxpmmask, new_xpm);
 	PIXMAP_CREATE(summaryview->ctree, unreadxpm, unreadxpmmask, unread_xpm);
+	PIXMAP_CREATE(summaryview->ctree, repliedxpm, repliedxpmmask,
+		      replied_xpm);
+	PIXMAP_CREATE(summaryview->ctree, forwardedxpm, forwardedxpmmask,
+		      forwarded_xpm);
 	PIXMAP_CREATE(summaryview->ctree, clipxpm, clipxpmmask, clip_xpm);
 	PIXMAP_CREATE(summaryview->hbox, folderxpm, folderxpmmask,
 		      DIRECTORY_OPEN_XPM);
@@ -535,9 +550,15 @@ gboolean summary_show(SummaryView *summaryview, FolderItem *item,
 	GtkCTreeNode *node;
 	GSList *mlist = NULL;
 	gchar *buf;
+	gboolean is_refresh;
+	guint prev_msgnum = 0;
 
 	STATUSBAR_POP(summaryview->mainwin);
 
+	is_refresh = (item == summaryview->folder_item) ? TRUE : FALSE;
+	if (is_refresh)
+		prev_msgnum = summary_get_current_msgnum(summaryview);
+
 #if 0
 	/* process the marks if any */
 	if (summaryview->deleted > 0 || summaryview->moved > 0) {
@@ -574,6 +595,7 @@ gboolean summary_show(SummaryView *summaryview, FolderItem *item,
 		gtk_clist_thaw(GTK_CLIST(ctree));
 		return TRUE;
 	}
+	g_free(buf);
 
 	summaryview->folder_item = item;
 
@@ -623,17 +645,21 @@ gboolean summary_show(SummaryView *summaryview, FolderItem *item,
 	
 	gtk_clist_thaw(GTK_CLIST(ctree));
 
-	/* select first unread message */
-	node = summary_find_next_unread_msg(summaryview, NULL);
-	if (node == NULL && GTK_CLIST(ctree)->row_list != NULL)
-		node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list_end);
-	if (node) {
-		GTK_EVENTS_FLUSH();
-		gtk_ctree_node_moveto(ctree, node, -1, 0.5, 0);
-		gtk_widget_grab_focus(GTK_WIDGET(ctree));
-		if (prefs_common.open_unread_on_enter)
-			summaryview->display_msg = TRUE;
-		gtk_sctree_select(GTK_SCTREE(ctree), node);
+	if (is_refresh) {
+		summary_select_by_msgnum(summaryview, prev_msgnum);
+	} else {
+		/* select first unread message */
+		node = summary_find_next_unread_msg(summaryview, NULL);
+		if (node == NULL && GTK_CLIST(ctree)->row_list != NULL)
+			node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list_end);
+		if (node) {
+			GTK_EVENTS_FLUSH();
+			gtk_ctree_node_moveto(ctree, node, -1, 0.5, 0);
+			gtk_widget_grab_focus(GTK_WIDGET(ctree));
+			if (prefs_common.open_unread_on_enter)
+				summaryview->display_msg = TRUE;
+			gtk_sctree_select(GTK_SCTREE(ctree), node);
+		}
 	}
 
 	summary_status_show(summaryview);
@@ -670,6 +696,7 @@ void summary_clear_list(SummaryView *summaryview)
 	summaryview->newmsgs   = summaryview->unread     = 0;
 	summaryview->messages  = summaryview->total_size = 0;
 	summaryview->deleted   = summaryview->moved      = 0;
+	summaryview->copied    = 0;
 	if (summaryview->msgid_table) {
 		g_hash_table_destroy(summaryview->msgid_table);
 		summaryview->msgid_table = NULL;
@@ -730,6 +757,7 @@ static void summary_set_menu_sensitive(SummaryView *summaryview)
 		if (summaryview->folder_item->stype != F_TRASH)
 			menu_set_sensitive(ifactory, "/Delete", TRUE);
 		menu_set_sensitive(ifactory, "/Move...", TRUE);
+		menu_set_sensitive(ifactory, "/Copy...", TRUE);
 	}
 	menu_set_sensitive(ifactory, "/Execute", TRUE);
 
@@ -790,6 +818,33 @@ void summary_select_next_unread(SummaryView *summaryview)
 	}
 }
 
+void summary_select_by_msgnum(SummaryView *summaryview, guint msgnum)
+{
+	GtkCTreeNode *node;
+	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
+
+	node = summary_find_msg_by_msgnum(summaryview, msgnum);
+
+	if (node) {
+		gtk_sctree_unselect_all(GTK_SCTREE(ctree));
+		gtk_sctree_select(GTK_SCTREE(ctree), node);
+		if (summaryview->displayed == node)
+			summaryview->displayed = NULL;
+		summary_display_msg(summaryview, node, FALSE);
+	}
+}
+
+guint summary_get_current_msgnum(SummaryView *summaryview)
+{
+	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
+	MsgInfo *msginfo;
+
+	if (!summaryview->selected)
+		return 0;
+	msginfo = gtk_ctree_node_get_row_data(ctree, summaryview->selected);
+	return msginfo->msgnum;
+}
+
 static GtkCTreeNode *summary_find_next_unread_msg(SummaryView *summaryview,
 						  GtkCTreeNode *current_node)
 {
@@ -811,6 +866,23 @@ static GtkCTreeNode *summary_find_next_unread_msg(SummaryView *summaryview,
 	return node;
 }
 
+static GtkCTreeNode *summary_find_msg_by_msgnum(SummaryView *summaryview,
+						guint msgnum)
+{
+	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
+	GtkCTreeNode *node;
+	MsgInfo *msginfo;
+
+	node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+
+	for (; node != NULL; node = GTK_CTREE_NODE_NEXT(node)) {
+		msginfo = gtk_ctree_node_get_row_data(ctree, node);
+		if (msginfo->msgnum == msgnum) break;
+	}
+
+	return node;
+}
+
 #if 0
 static GtkCTreeNode *summary_find_prev_unread_msg(SummaryView *summaryview,
 						  GtkCTreeNode *current_node)
@@ -834,6 +906,31 @@ static GtkCTreeNode *summary_find_prev_unread_msg(SummaryView *summaryview,
 }
 #endif
 
+static guint attract_hash_func(gconstpointer key)
+{
+	gchar *str;
+	gchar *p;
+	guint h;
+
+	Xstrdup_a(str, (const gchar *)key, return 0);
+	trim_subject(str);
+
+	p = str;
+	h = *p;
+
+	if (h) {
+		for (p += 1; *p != '\0'; p++)
+			h = (h << 5) - h + *p;
+	}
+
+	return h;
+}
+
+static gint attract_compare_func(gconstpointer a, gconstpointer b)
+{
+	return subject_compare((const gchar *)a, (const gchar *)b) == 0;
+}
+
 void summary_attract_by_subject(SummaryView *summaryview)
 {
 	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
@@ -842,6 +939,7 @@ void summary_attract_by_subject(SummaryView *summaryview)
 	GtkCTreeNode *dst_node, *sibling;
 	GtkCTreeNode *tmp;
 	MsgInfo *src_msginfo, *dst_msginfo;
+	GHashTable *subject_table;
 
 	debug_print(_("Attracting messages by subject..."));
 	STATUSBAR_PUSH(summaryview->mainwin,
@@ -850,6 +948,9 @@ void summary_attract_by_subject(SummaryView *summaryview)
 	main_window_cursor_wait(summaryview->mainwin);
 	gtk_clist_freeze(clist);
 
+	subject_table = g_hash_table_new(attract_hash_func,
+					 attract_compare_func);
+
 	for (src_node = GTK_CTREE_NODE(clist->row_list);
 	     src_node != NULL;
 	     src_node = tmp) {
@@ -859,32 +960,29 @@ void summary_attract_by_subject(SummaryView *summaryview)
 		if (!src_msginfo->subject) continue;
 
 		/* find attracting node */
-		for (dst_node = GTK_CTREE_NODE_PREV(src_node);
-		     dst_node != NULL;
-		     dst_node = GTK_CTREE_NODE_PREV(dst_node)) {
-			/* ignore child node */
-			if (GTK_CTREE_ROW(dst_node)->level > 1) continue;
+		dst_node = g_hash_table_lookup(subject_table,
+					       src_msginfo->subject);
 
+		if (dst_node) {
 			dst_msginfo = GTKUT_CTREE_NODE_GET_ROW_DATA(dst_node);
-			if (!dst_msginfo->subject) continue;
 
-			/* if the time difference is more than 30 days,
-			   skip the comparison */
+			/* if the time difference is more than 20 days,
+			   don't attract */
 			if (ABS(src_msginfo->date_t - dst_msginfo->date_t)
-			    > 2592000)
+			    > 60 * 60 * 24 * 20)
 				continue;
 
-			if (!subject_compare(src_msginfo->subject,
-					     dst_msginfo->subject)) {
-				sibling = GTK_CTREE_ROW(dst_node)->sibling;
-				if (src_node != sibling)
-					gtk_ctree_move(ctree, src_node,
-						       NULL, sibling);
-				break;
-			}
+			sibling = GTK_CTREE_ROW(dst_node)->sibling;
+			if (src_node != sibling)
+				gtk_ctree_move(ctree, src_node, NULL, sibling);
 		}
+
+		g_hash_table_insert(subject_table,
+				    src_msginfo->subject, src_node);
 	}
 
+	g_hash_table_destroy(subject_table);
+
 	gtk_ctree_node_moveto(ctree, summaryview->selected, -1, 0.5, 0);
 
 	gtk_clist_thaw(clist);
@@ -912,24 +1010,17 @@ static void summary_set_marks_func(GtkCTree *ctree, GtkCTreeNode *node,
 
 	msginfo = gtk_ctree_node_get_row_data(ctree, node);
 
-	if (MSG_IS_NEW(msginfo->flags)) {
-		gtk_ctree_node_set_pixmap(ctree, node, S_COL_UNREAD,
-					  newxpm, newxpmmask);
+	if (MSG_IS_NEW(msginfo->flags))
 		summaryview->newmsgs++;
-		if (MSG_IS_UNREAD(msginfo->flags))
-			summaryview->unread++;
-	} else if (MSG_IS_UNREAD(msginfo->flags)) {
-		gtk_ctree_node_set_pixmap(ctree, node, S_COL_UNREAD,
-					  unreadxpm, unreadxpmmask);
+	if (MSG_IS_UNREAD(msginfo->flags))
 		summaryview->unread++;
-	}
-
-	if (MSG_IS_MIME(msginfo->flags))
-		gtk_ctree_node_set_pixmap(ctree, node, S_COL_MIME,
-					  clipxpm, clipxpmmask);
+	if (MSG_IS_DELETED(msginfo->flags))
+		summaryview->deleted++;
 
 	summaryview->messages++;
 	summaryview->total_size += msginfo->size;
+
+	summary_set_row_marks(summaryview, node);
 }
 
 static void summary_update_status(SummaryView *summaryview)
@@ -940,7 +1031,7 @@ static void summary_update_status(SummaryView *summaryview)
 
 	summaryview->newmsgs = summaryview->unread =
 	summaryview->messages = summaryview->total_size =
-	summaryview->deleted = summaryview->moved = 0;
+	summaryview->deleted = summaryview->moved = summaryview->copied = 0;
 
 	for (node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
 	     node != NULL; node = GTK_CTREE_NODE_NEXT(node)) {
@@ -954,6 +1045,8 @@ static void summary_update_status(SummaryView *summaryview)
 			summaryview->deleted++;
 		if (MSG_IS_MOVE(msginfo->flags))
 			summaryview->moved++;
+		if (MSG_IS_COPY(msginfo->flags))
+			summaryview->copied++;
 		summaryview->messages++;
 		summaryview->total_size += msginfo->size;
 	}
@@ -961,7 +1054,10 @@ static void summary_update_status(SummaryView *summaryview)
 
 static void summary_status_show(SummaryView *summaryview)
 {
-	gchar *buf1, *buf2, *buf3;
+	gchar *str;
+	gchar *del, *mv, *cp;
+	gchar *sel;
+	gchar *spc;
 	GList *rowlist, *cur;
 	guint n_selected = 0;
 	off_t sel_size = 0;
@@ -990,50 +1086,57 @@ static void summary_status_show(SummaryView *summaryview)
 		      : summaryview->folder_item->path);
 
 	if (summaryview->deleted)
-		buf1 = g_strdup_printf(_("%d deleted"), summaryview->deleted);
+		del = g_strdup_printf(_("%d deleted"), summaryview->deleted);
 	else
-		buf1 = g_strdup("");
+		del = g_strdup("");
 	if (summaryview->moved)
-		buf2 = g_strdup_printf(_("%d moved"), summaryview->moved);
+		mv = g_strdup_printf(_("%s%d moved"),
+				     summaryview->deleted ? _(", ") : "",
+				     summaryview->moved);
 	else
-		buf2 = g_strdup("");
+		mv = g_strdup("");
+	if (summaryview->copied)
+		cp = g_strdup_printf(_("%s%d copied"),
+				     summaryview->deleted ||
+				     summaryview->moved ? _(", ") : "",
+				     summaryview->copied);
+	else
+		cp = g_strdup("");
 
-	buf3 = g_strdup_printf("%s%s%s%s",
-			       summaryview->deleted || summaryview->moved
-						? "    " : "", buf1,
-			       summaryview->deleted && summaryview->moved
-						? _(", ") : "", buf2);
+	if (summaryview->deleted || summaryview->moved || summaryview->copied)
+		spc = "    ";
+	else
+		spc = "";
 
-	g_free(buf1);
-	g_free(buf2);
 	if (n_selected)
-		buf2 = g_strdup_printf(" (%s)", to_human_readable(sel_size));
+		sel = g_strdup_printf(" (%s)", to_human_readable(sel_size));
 	else
-		buf2 = g_strdup("");
-	buf1 = g_strdup_printf("%s%s%s%s",
-			      n_selected ? itos(n_selected) : "",
-			      n_selected ? _(" item(s) selected") : "",
-			      buf2, buf3);
-	gtk_label_set(GTK_LABEL(summaryview->statlabel_select), buf1);
-	g_free(buf1);
-	g_free(buf2);
-	g_free(buf3);
+		sel = g_strdup("");
+	str = g_strconcat(n_selected ? itos(n_selected) : "",
+			  n_selected ? _(" item(s) selected") : "",
+			  sel, spc, del, mv, cp, NULL);
+	gtk_label_set(GTK_LABEL(summaryview->statlabel_select), str);
+	g_free(str);
+	g_free(sel);
+	g_free(del);
+	g_free(mv);
+	g_free(cp);
 
 	if (summaryview->folder_item &&
 	    summaryview->folder_item->folder->type == F_MH) {
-		buf1 = g_strdup_printf(_("%d new, %d unread, %d total (%s)"),
-				       summaryview->newmsgs,
-				       summaryview->unread,
-				       summaryview->messages,
-				       to_human_readable(summaryview->total_size));
+		str = g_strdup_printf(_("%d new, %d unread, %d total (%s)"),
+				      summaryview->newmsgs,
+				      summaryview->unread,
+				      summaryview->messages,
+				      to_human_readable(summaryview->total_size));
 	} else {
-		buf1 = g_strdup_printf(_("%d new, %d unread, %d total"),
-				       summaryview->newmsgs,
-				       summaryview->unread,
-				       summaryview->messages);
+		str = g_strdup_printf(_("%d new, %d unread, %d total"),
+				      summaryview->newmsgs,
+				      summaryview->unread,
+				      summaryview->messages);
 	}
-	gtk_label_set(GTK_LABEL(summaryview->statlabel_msgs), buf1);
-	g_free(buf1);
+	gtk_label_set(GTK_LABEL(summaryview->statlabel_msgs), str);
+	g_free(str);
 
 	folderview_update_msg_num(summaryview->folderview,
 				  summaryview->folderview->opened,
@@ -1323,7 +1426,7 @@ static void summary_display_msg(SummaryView *summaryview, GtkCTreeNode *row,
 		summaryview->unread--;
 	if (MSG_IS_NEW(msginfo->flags) || MSG_IS_UNREAD(msginfo->flags)) {
 		MSG_UNSET_FLAGS(msginfo->flags, MSG_NEW | MSG_UNREAD);
-		gtk_ctree_node_set_text(ctree, row, S_COL_UNREAD, NULL);
+		summary_set_row_marks(summaryview, row);
 		gtk_clist_thaw(GTK_CLIST(ctree));
 		summary_status_show(summaryview);
 	}
@@ -1334,13 +1437,19 @@ static void summary_display_msg(SummaryView *summaryview, GtkCTreeNode *row,
 		msgview = messageview_create_with_new_window();
 		messageview_show(msgview, msginfo);
 	} else {
+		MessageView *msgview;
+
+		msgview = summaryview->messageview;
+
 		summaryview->displayed = row;
 		if (!summaryview->msg_is_toggled_on)
 			summary_toggle_view(summaryview);
-		messageview_show(summaryview->messageview, msginfo);
-		GTK_EVENTS_FLUSH();
-		if (summaryview->messageview->type == MVIEW_TEXT)
+		messageview_show(msgview, msginfo);
+		if (msgview->type == MVIEW_TEXT ||
+		    (msgview->type == MVIEW_MIME &&
+		     GTK_CLIST(msgview->mimeview->ctree)->row_list == NULL))
 			gtk_widget_grab_focus(summaryview->ctree);
+		GTK_EVENTS_FLUSH();
 		gtkut_ctree_node_move_if_on_the_edge(ctree, row);
 	}
 
@@ -1449,6 +1558,70 @@ static void summary_toggle_view(SummaryView *summaryview)
 	gtk_widget_grab_focus(summaryview->ctree);
 }
 
+static void summary_set_row_marks(SummaryView *summaryview, GtkCTreeNode *row)
+{
+	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
+	MsgInfo *msginfo;
+	MsgFlags flags;
+
+	msginfo = gtk_ctree_node_get_row_data(ctree, row);
+	if (!msginfo) return;
+
+	flags = msginfo->flags;
+
+	gtk_ctree_node_set_foreground(ctree, row, &summaryview->color_normal);
+
+	/* set new/unread column */
+	if (MSG_IS_NEW(flags)) {
+		gtk_ctree_node_set_pixmap(ctree, row, S_COL_UNREAD,
+					  newxpm, newxpmmask);
+	} else if (MSG_IS_UNREAD(flags)) {
+		gtk_ctree_node_set_pixmap(ctree, row, S_COL_UNREAD,
+					  unreadxpm, unreadxpmmask);
+	} else if (MSG_IS_REPLIED(flags)) {
+		gtk_ctree_node_set_pixmap(ctree, row, S_COL_UNREAD,
+					  repliedxpm, repliedxpmmask);
+	} else if (MSG_IS_FORWARDED(flags)) {
+		gtk_ctree_node_set_pixmap(ctree, row, S_COL_UNREAD,
+					  forwardedxpm, forwardedxpmmask);
+	} else {
+		gtk_ctree_node_set_text(ctree, row, S_COL_UNREAD, NULL);
+	}
+
+	/* set mark column */
+	if (MSG_IS_DELETED(flags)) {
+		gtk_ctree_node_set_pixmap(ctree, row, S_COL_MARK,
+					  deletedxpm, deletedxpmmask);
+		gtk_ctree_node_set_foreground(ctree, row,
+					      &summaryview->color_dim);
+	} else if (MSG_IS_MARKED(flags)) {
+		gtk_ctree_node_set_pixmap(ctree, row, S_COL_MARK,
+					  markxpm, markxpmmask);
+	} else if (MSG_IS_MOVE(flags)) {
+		gtk_ctree_node_set_text(ctree, row, S_COL_MARK, "o");
+		gtk_ctree_node_set_foreground(ctree, row,
+					      &summaryview->color_marked);
+	} else if (MSG_IS_COPY(flags)) {
+		gtk_ctree_node_set_text(ctree, row, S_COL_MARK, "O");
+		gtk_ctree_node_set_foreground(ctree, row,
+					      &summaryview->color_marked);
+	} else {
+		gtk_ctree_node_set_text(ctree, row, S_COL_MARK, NULL);
+	}
+
+	if (MSG_IS_MIME(flags)) {
+		gtk_ctree_node_set_pixmap(ctree, row, S_COL_MIME,
+					  clipxpm, clipxpmmask);
+	} else {
+		gtk_ctree_node_set_text(ctree, row, S_COL_MIME, NULL);
+	}
+}
+
+void summary_set_marks_selected(SummaryView *summaryview)
+{
+	summary_set_row_marks(summaryview, summaryview->selected);
+}
+
 static void summary_mark_row(SummaryView *summaryview, GtkCTreeNode *row)
 {
 	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
@@ -1460,11 +1633,11 @@ static void summary_mark_row(SummaryView *summaryview, GtkCTreeNode *row)
 		summaryview->deleted--;
 	if (MSG_IS_MOVE(msginfo->flags))
 		summaryview->moved--;
+	if (MSG_IS_COPY(msginfo->flags))
+		summaryview->copied--;
 	MSG_UNSET_FLAGS(msginfo->flags, MSG_DELETED | MSG_MOVE | MSG_COPY);
 	MSG_SET_FLAGS(msginfo->flags, MSG_MARKED);
-	gtk_ctree_node_set_text(ctree, row, S_COL_MARK, "*");
-	gtk_ctree_node_set_foreground
-		(ctree, row, &summaryview->color_marked);
+	summary_set_row_marks(summaryview, row);
 	debug_print(_("Message %d is marked\n"), msginfo->msgnum);
 }
 
@@ -1474,65 +1647,75 @@ void summary_mark(SummaryView *summaryview)
 	GList *cur;
 
 	for (cur = GTK_CLIST(ctree)->selection; cur != NULL; cur = cur->next)
-		summary_mark_row(summaryview, (GtkCTreeNode *)cur->data);
+		summary_mark_row(summaryview, GTK_CTREE_NODE(cur->data));
 
-	summary_step(summaryview, GTK_SCROLL_STEP_FORWARD);
+	/* summary_step(summaryview, GTK_SCROLL_STEP_FORWARD); */
+}
+
+static void summary_mark_row_as_read(SummaryView *summaryview,
+				     GtkCTreeNode *row)
+{
+	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
+	MsgInfo *msginfo;
+
+	msginfo = gtk_ctree_node_get_row_data(ctree, row);
+	if (MSG_IS_NEW(msginfo->flags))
+		summaryview->newmsgs--;
+	if (MSG_IS_UNREAD(msginfo->flags))
+		summaryview->unread--;
+	if (MSG_IS_NEW(msginfo->flags) ||
+	    MSG_IS_UNREAD(msginfo->flags)) {
+		MSG_UNSET_FLAGS(msginfo->flags, MSG_NEW | MSG_UNREAD);
+		summary_set_row_marks(summaryview, row);
+		debug_print(_("Message %d is marked as being read\n"),
+			    msginfo->msgnum);
+	}
 }
 
 void summary_mark_as_read(SummaryView *summaryview)
 {
 	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
 	GList *cur;
-	GtkCTreeNode *row;
-	MsgInfo *msginfo;
 
-	for (cur = GTK_CLIST(ctree)->selection; cur != NULL;
-	     cur = cur->next) {
-		row = cur->data;
-		msginfo = gtk_ctree_node_get_row_data(ctree, row);
-		if (MSG_IS_NEW(msginfo->flags))
-			summaryview->newmsgs--;
-		if (MSG_IS_UNREAD(msginfo->flags))
-			summaryview->unread--;
-		if (MSG_IS_NEW(msginfo->flags) ||
-		    MSG_IS_UNREAD(msginfo->flags)) {
-			MSG_UNSET_FLAGS(msginfo->flags, MSG_NEW | MSG_UNREAD);
-			gtk_ctree_node_set_text(ctree, row, S_COL_UNREAD, NULL);
-			debug_print(_("Message %d is marked as being read\n"),
-				    msginfo->msgnum);
-		}
-	}
+	for (cur = GTK_CLIST(ctree)->selection; cur != NULL; cur = cur->next)
+		summary_mark_row_as_read(summaryview,
+					 GTK_CTREE_NODE(cur->data));
 
 	summary_status_show(summaryview);
 }
 
+static void summary_mark_row_as_unread(SummaryView *summaryview,
+				       GtkCTreeNode *row)
+{
+	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
+	MsgInfo *msginfo;
+
+	msginfo = gtk_ctree_node_get_row_data(ctree, row);
+	if (MSG_IS_DELETED(msginfo->flags)) {
+		msginfo->to_folder = NULL;
+		MSG_UNSET_FLAGS(msginfo->flags, MSG_DELETED);
+		summaryview->deleted--;
+	}
+	MSG_UNSET_FLAGS(msginfo->flags, MSG_REPLIED | MSG_FORWARDED);
+	if (!MSG_IS_UNREAD(msginfo->flags)) {
+		MSG_SET_FLAGS(msginfo->flags, MSG_UNREAD);
+		gtk_ctree_node_set_pixmap(ctree, row, S_COL_UNREAD,
+					  unreadxpm, unreadxpmmask);
+		summaryview->unread++;
+		debug_print(_("Message %d is marked as unread\n"),
+			    msginfo->msgnum);
+	}
+	summary_set_row_marks(summaryview, row);
+}
+
 void summary_mark_as_unread(SummaryView *summaryview)
 {
 	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
 	GList *cur;
-	GtkCTreeNode *row;
-	MsgInfo *msginfo;
 
-	for (cur = GTK_CLIST(ctree)->selection; cur != NULL;
-	     cur = cur->next) {
-		row = cur->data;
-		msginfo = gtk_ctree_node_get_row_data(ctree, row);
-		if (MSG_IS_DELETED(msginfo->flags)) {
-			msginfo->to_folder = NULL;
-			MSG_UNSET_FLAGS(msginfo->flags, MSG_DELETED);
-			summaryview->deleted--;
-		}
-		if (!MSG_IS_UNREAD(msginfo->flags)) {
-			MSG_SET_FLAGS(msginfo->flags, MSG_UNREAD);
-			gtk_ctree_node_set_pixmap(ctree, row, S_COL_UNREAD,
-						  unreadxpm, unreadxpmmask);
-			summaryview->unread++;
-			gtk_ctree_node_set_foreground
-				(ctree, row, &summaryview->color_normal);
-			debug_print(_("Message %d is marked as unread\n"),
-				    msginfo->msgnum);
-		}
-	}
+	for (cur = GTK_CLIST(ctree)->selection; cur != NULL; cur = cur->next)
+		summary_mark_row_as_unread(summaryview,
+					   GTK_CTREE_NODE(cur->data));
 
 	summary_status_show(summaryview);
 }
@@ -1549,6 +1732,8 @@ static void summary_delete_row(SummaryView *summaryview, GtkCTreeNode *row)
 	msginfo->to_folder = NULL;
 	if (MSG_IS_MOVE(msginfo->flags))
 		summaryview->moved--;
+	if (MSG_IS_COPY(msginfo->flags))
+		summaryview->copied--;
 	MSG_UNSET_FLAGS(msginfo->flags,
 			MSG_MARKED |
 			MSG_MOVE |
@@ -1556,11 +1741,9 @@ static void summary_delete_row(SummaryView *summaryview, GtkCTreeNode *row)
 	MSG_SET_FLAGS(msginfo->flags, MSG_DELETED);
 	summaryview->deleted++;
 
-	if (!prefs_common.immediate_exec) {
-		gtk_ctree_node_set_text(ctree, row, S_COL_MARK, _("D"));
-		gtk_ctree_node_set_foreground(ctree, row,
-					      &summaryview->color_dim);
-	}
+	if (!prefs_common.immediate_exec)
+		summary_set_row_marks(summaryview, row);
+
 	debug_print(_("Message %s/%d is set to delete\n"),
 		    msginfo->folder->path, msginfo->msgnum);
 }
@@ -1640,15 +1823,15 @@ static void summary_unmark_row(SummaryView *summaryview, GtkCTreeNode *row)
 		summaryview->deleted--;
 	if (MSG_IS_MOVE(msginfo->flags))
 		summaryview->moved--;
+	if (MSG_IS_COPY(msginfo->flags))
+		summaryview->copied--;
 	MSG_UNSET_FLAGS(msginfo->flags,
 			MSG_MARKED |
 			MSG_DELETED |
-			MSG_IMPORTANT |
 			MSG_MOVE |
 			MSG_COPY);
-	gtk_ctree_node_set_text(ctree, row, S_COL_MARK, NULL);
-	gtk_ctree_node_set_foreground
-		(ctree, row, &summaryview->color_normal);
+	summary_set_row_marks(summaryview, row);
+
 	debug_print(_("Message %s/%d is unmarked\n"),
 		    msginfo->folder->path, msginfo->msgnum);
 }
@@ -1683,11 +1866,9 @@ static void summary_move_row_to(SummaryView *summaryview, GtkCTreeNode *row,
 		MSG_SET_FLAGS(msginfo->flags, MSG_MOVE);
 		summaryview->moved++;
 	}
-	if (!prefs_common.immediate_exec) {
-		gtk_ctree_node_set_text(ctree, row, S_COL_MARK, _("o"));
-		gtk_ctree_node_set_foreground
-			(ctree, row, &summaryview->color_marked);
-	}
+	if (!prefs_common.immediate_exec)
+		summary_set_row_marks(summaryview, row);
+
 	debug_print(_("Message %d is set to move to %s\n"),
 		    msginfo->msgnum, to_folder->path);
 }
@@ -1704,7 +1885,8 @@ void summary_move_selected_to(SummaryView *summaryview, FolderItem *to_folder)
 		return;
 	}
 
-	for (cur = GTK_CLIST(summaryview->ctree)->selection; cur != NULL; cur = cur->next)
+	for (cur = GTK_CLIST(summaryview->ctree)->selection;
+	     cur != NULL; cur = cur->next)
 		summary_move_row_to
 			(summaryview, GTK_CTREE_NODE(cur->data), to_folder);
 
@@ -1727,6 +1909,68 @@ void summary_move_to(SummaryView *summaryview)
 	summary_move_selected_to(summaryview, to_folder);
 }
 
+static void summary_copy_row_to(SummaryView *summaryview, GtkCTreeNode *row,
+				FolderItem *to_folder)
+{
+	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
+	MsgInfo *msginfo;
+
+	g_return_if_fail(to_folder != NULL);
+
+	msginfo = gtk_ctree_node_get_row_data(ctree, row);
+	msginfo->to_folder = to_folder;
+	if (MSG_IS_DELETED(msginfo->flags))
+		summaryview->deleted--;
+	MSG_UNSET_FLAGS(msginfo->flags,
+			MSG_MARKED | MSG_DELETED | MSG_MOVE);
+	if (!MSG_IS_COPY(msginfo->flags)) {
+		MSG_SET_FLAGS(msginfo->flags, MSG_COPY);
+		summaryview->copied++;
+	}
+	if (!prefs_common.immediate_exec)
+		summary_set_row_marks(summaryview, row);
+
+	debug_print(_("Message %d is set to copy to %s\n"),
+		    msginfo->msgnum, to_folder->path);
+}
+
+void summary_copy_selected_to(SummaryView *summaryview, FolderItem *to_folder)
+{
+	GList *cur;
+
+	if (!to_folder) return;
+	if (!summaryview->folder_item ||
+	    summaryview->folder_item->folder->type == F_NEWS) return;
+	if (summaryview->folder_item == to_folder) {
+		alertpanel_notice
+			(_("Destination to copy is same as current folder."));
+		return;
+	}
+
+	for (cur = GTK_CLIST(summaryview->ctree)->selection;
+	     cur != NULL; cur = cur->next)
+		summary_copy_row_to
+			(summaryview, GTK_CTREE_NODE(cur->data), to_folder);
+
+	summary_step(summaryview, GTK_SCROLL_STEP_FORWARD);
+
+	if (prefs_common.immediate_exec)
+		summary_execute(summaryview);
+	else
+		summary_status_show(summaryview);
+}
+
+void summary_copy_to(SummaryView *summaryview)
+{
+	FolderItem *to_folder;
+
+	if (!summaryview->folder_item ||
+	    summaryview->folder_item->folder->type == F_NEWS) return;
+
+	to_folder = foldersel_folder_sel(NULL);
+	summary_copy_selected_to(summaryview, to_folder);
+}
+
 void summary_select_all(SummaryView *summaryview)
 {
 	if (summaryview->messages >= 500) {
@@ -1819,6 +2063,7 @@ void summary_execute(SummaryView *summaryview)
 		summary_unthread_for_exec(summaryview);
 
 	summary_execute_move(summaryview);
+	summary_execute_copy(summaryview);
 	summary_execute_delete(summaryview);
 
 	node = GTK_CTREE_NODE(clist->row_list);
@@ -1911,6 +2156,50 @@ static void summary_execute_move_func(GtkCTree *ctree, GtkCTreeNode *node,
 	}
 }
 
+static void summary_execute_copy(SummaryView *summaryview)
+{
+	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
+
+	summaryview->folder_table = g_hash_table_new(NULL, NULL);
+
+	/* search copying messages and execute */
+	gtk_ctree_pre_recursive(ctree, NULL, summary_execute_copy_func,
+				summaryview);
+
+	if (summaryview->mlist) {
+		procmsg_copy_messages(summaryview->mlist);
+
+		folder_item_scan_foreach(summaryview->folder_table);
+		folderview_update_item_foreach(summaryview->folder_table);
+
+		g_slist_free(summaryview->mlist);
+		summaryview->mlist = NULL;
+	}
+
+	g_hash_table_destroy(summaryview->folder_table);
+	summaryview->folder_table = NULL;
+}
+
+static void summary_execute_copy_func(GtkCTree *ctree, GtkCTreeNode *node,
+				      gpointer data)
+{
+	SummaryView *summaryview = data;
+	MsgInfo *msginfo;
+
+	msginfo = GTKUT_CTREE_NODE_GET_ROW_DATA(node);
+
+	if (msginfo && MSG_IS_COPY(msginfo->flags) && msginfo->to_folder) {
+		g_hash_table_insert(summaryview->folder_table,
+				    msginfo->to_folder, GINT_TO_POINTER(1));
+
+		summaryview->mlist =
+			g_slist_append(summaryview->mlist, msginfo);
+
+		MSG_UNSET_FLAGS(msginfo->flags, MSG_COPY);
+		summary_set_row_marks(summaryview, node);
+	}
+}
+
 static void summary_execute_delete(SummaryView *summaryview)
 {
 	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
@@ -2072,7 +2361,8 @@ static void summary_unthread_for_exec_func(GtkCTree *ctree, GtkCTreeNode *node,
 	msginfo = GTKUT_CTREE_NODE_GET_ROW_DATA(node);
 
 	if (!msginfo ||
-	    (!MSG_IS_MOVE(msginfo->flags) && !MSG_IS_DELETED(msginfo->flags)))
+	    (!MSG_IS_MOVE(msginfo->flags) &&
+	     !MSG_IS_DELETED(msginfo->flags)))
 		return;
 	child = GTK_CTREE_ROW(node)->children;
 	if (!child) return;
@@ -2108,7 +2398,10 @@ void summary_filter(SummaryView *summaryview)
 
 	gtk_clist_thaw(GTK_CLIST(summaryview->ctree));
 
-	summary_status_show(summaryview);
+	if (prefs_common.immediate_exec)
+		summary_execute(summaryview);
+	else
+		summary_status_show(summaryview);
 
 	debug_print(_("done.\n"));
 	STATUSBAR_POP(summaryview->mainwin);
@@ -2209,13 +2502,13 @@ static void summary_key_pressed(GtkWidget *widget, GdkEventKey *event,
 		else
 			compose_new(NULL);
 		return;
-	case GDK_D:		/* Clean trash */
+	case GDK_D:		/* Empty trash */
 		BREAK_ON_MODIFIER_KEY();
 		if (gtk_signal_n_emissions_by_name
 			(GTK_OBJECT(ctree), "key_press_event") > 0)
 			gtk_signal_emit_stop_by_name(GTK_OBJECT(ctree),
 						     "key_press_event");
-		main_window_clean_trash(summaryview->mainwin, TRUE);
+		main_window_empty_trash(summaryview->mainwin, TRUE);
 		return;
 	case GDK_Q:		/* Quit */
 		BREAK_ON_MODIFIER_KEY();
@@ -2303,10 +2596,13 @@ static void summary_key_pressed(GtkWidget *widget, GdkEventKey *event,
 		summary_unmark(summaryview);
 		break;
 	case GDK_o:		/* Move */
-	case GDK_O:
 		BREAK_ON_MODIFIER_KEY();
 		summary_move_to(summaryview);
 		break;
+	case GDK_O:		/* Copy */
+		BREAK_ON_MODIFIER_KEY();
+		summary_copy_to(summaryview);
+		break;
 	case GDK_x:		/* Execute */
 	case GDK_X:
 		BREAK_ON_MODIFIER_KEY();
@@ -2351,6 +2647,8 @@ static void summary_open_row(GtkSCTree *sctree, SummaryView *summaryview)
 static void summary_selected(GtkCTree *ctree, GtkCTreeNode *row,
 			     gint column, SummaryView *summaryview)
 {
+	MsgInfo *msginfo;
+
 	summary_status_show(summaryview);
 	summary_set_menu_sensitive(summaryview);
 
@@ -2362,6 +2660,29 @@ static void summary_selected(GtkCTree *ctree, GtkCTreeNode *row,
 
 	summaryview->selected = row;
 
+	msginfo = gtk_ctree_node_get_row_data(ctree, row);
+
+	switch (column) {
+	case S_COL_MARK:
+		if (MSG_IS_MARKED(msginfo->flags)) {
+			MSG_UNSET_FLAGS(msginfo->flags, MSG_MARKED);
+			summary_set_row_marks(summaryview, row);
+		} else
+			summary_mark_row(summaryview, row);
+		break;
+	case S_COL_UNREAD:
+		if (MSG_IS_UNREAD(msginfo->flags)) {
+			summary_mark_row_as_read(summaryview, row);
+			summary_status_show(summaryview);
+		} else if (!MSG_IS_REPLIED(msginfo->flags) &&
+			 !MSG_IS_FORWARDED(msginfo->flags)) {
+			summary_mark_row_as_unread(summaryview, row);
+			summary_status_show(summaryview);
+		}
+		break;
+	default:
+	}
+
 	if (summaryview->display_msg)
 		summary_display_msg(summaryview, row, FALSE);
 
@@ -2437,6 +2758,8 @@ static void summary_reply_cb(SummaryView *summaryview, guint action,
 	default:
 		compose_reply(msginfo, prefs_common.reply_with_quote, FALSE);
 	}
+
+	summary_set_marks_selected(summaryview);
 }
 
 static void summary_show_all_header_cb(SummaryView *summaryview,
