@@ -66,17 +66,16 @@ struct _RemoteURI
 	guint end;
 };
 
-static GdkColor quote_color = {
-	(gulong)0,
-	(gushort)0,
-	(gushort)0,
-	(gushort)0xbfff
+static GdkColor quote_colors[3] = {
+	{(gulong)0, (gushort)0, (gushort)0, (gushort)0},
+	{(gulong)0, (gushort)0, (gushort)0, (gushort)0},
+	{(gulong)0, (gushort)0, (gushort)0, (gushort)0}
 };
 
 static GdkColor uri_color = {
 	(gulong)0,
 	(gushort)0,
-	(gushort)0x8fff,
+	(gushort)0,
 	(gushort)0
 };
 
@@ -116,6 +115,16 @@ static void textview_button_pressed	(GtkWidget	*widget,
 
 static void textview_uri_list_remove_all(GSList		*uri_list);
 
+static void textview_smooth_scroll_do		(TextView	*textview,
+						 gfloat		 old_value,
+						 gfloat		 last_value,
+						 gint		 step);
+static void textview_smooth_scroll_one_line	(TextView	*textview,
+						 gboolean	 up);
+static gboolean textview_smooth_scroll_page	(TextView	*textview,
+						 gboolean	 up);
+
+
 TextView *textview_create(void)
 {
 	TextView *textview;
@@ -131,9 +140,9 @@ TextView *textview_create(void)
 	scrolledwin_sb = gtk_scrolled_window_new(NULL, NULL);
 	scrolledwin_mb = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin_sb),
-				       GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin_mb),
-				       GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	gtk_widget_set_usize(scrolledwin_sb, prefs_common.mainview_width, -1);
 	gtk_widget_set_usize(scrolledwin_mb, prefs_common.mainview_width, -1);
 
@@ -201,30 +210,32 @@ TextView *textview_create(void)
 
 void textview_init(TextView *textview)
 {
-	GdkColormap *colormap;
-	GdkColor color[3];
-	gboolean success[3];
-
 	gtkut_widget_disable_theme_engine(textview->text_sb);
 	gtkut_widget_disable_theme_engine(textview->text_mb);
-
-	color[0] = quote_color;
-	color[1] = uri_color;
-	color[2] = emphasis_color;
-	colormap = gdk_window_get_colormap(textview->text->window);
-	gdk_colormap_alloc_colors(colormap, color, 3, FALSE, TRUE, success);
-	if (success[0] == FALSE || success[1] == FALSE ||
-	    success[2] == FALSE) {
-		GtkStyle *style;
-
-		g_warning(_("TextView: color allocation failed\n"));
-		style = gtk_widget_get_style(textview->text);
-		quote_color = uri_color = emphasis_color = style->black;
-	}
-
+	textview_update_message_colors();
 	textview_set_font(textview, NULL);
 }
 
+void textview_update_message_colors(void)
+{
+	GdkColor black = {0, 0, 0, 0};
+
+	if (prefs_common.enable_color) {
+		/* grab the quote colors, converting from an int to a GdkColor */
+		gtkut_convert_int_to_gdk_color(prefs_common.quote_level1_col,
+					       &quote_colors[0]);
+		gtkut_convert_int_to_gdk_color(prefs_common.quote_level2_col,
+					       &quote_colors[1]);
+		gtkut_convert_int_to_gdk_color(prefs_common.quote_level3_col,
+					       &quote_colors[2]);
+		gtkut_convert_int_to_gdk_color(prefs_common.uri_col,
+					       &uri_color);
+	} else {
+		quote_colors[0] = quote_colors[1] = quote_colors[2] = 
+			uri_color = emphasis_color = black;
+	}
+}
+
 void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 			   const gchar *file)
 {
@@ -380,6 +391,38 @@ void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 	gtk_text_thaw(text);
 }
 
+#if USE_GPGME
+void textview_show_signature_part(TextView *textview, MimeInfo *partinfo)
+{
+	GtkText *text;
+
+	if (!partinfo) return;
+
+	textview_set_font(textview, NULL);
+	text = GTK_TEXT(textview->text);
+	textview_clear(textview);
+
+	gtk_text_freeze(text);
+
+	if (partinfo->sigstatus_full == NULL) {
+		gtk_text_insert
+			(text, textview->msgfont, NULL, NULL,
+			 _("This signature has not been checked yet.\n"), -1);
+		gtk_text_insert
+			(text, textview->msgfont, NULL, NULL,
+			 _("To check it, pop up the context menu with\n"), -1);
+		gtk_text_insert
+			(text, textview->msgfont, NULL, NULL,
+			 _("right click and select `Check signature'.\n"), -1);
+	} else {
+		gtk_text_insert(text, textview->msgfont, NULL, NULL,
+				partinfo->sigstatus_full, -1);
+	}
+
+	gtk_text_thaw(text);
+}
+#endif /* USE_GPGME */
+
 static void textview_show_html(TextView *textview, FILE *fp,
 			       CodeConverter *conv)
 {
@@ -631,8 +674,8 @@ static void textview_write_line(TextView *textview, const gchar *str,
 	GtkText *text = GTK_TEXT(textview->text);
 	gchar buf[BUFFSIZE];
 	size_t len;
-	gchar *p;
 	GdkColor *fg_color;
+	gint quotelevel = -1;
 
 	if (!conv)
 		strncpy2(buf, str, sizeof(buf));
@@ -653,26 +696,28 @@ static void textview_write_line(TextView *textview, const gchar *str,
 	if (prefs_common.conv_mb_alnum) conv_mb_alnum(buf);
 	fg_color = NULL;
 
-	/* change color of quotation */
-	/* >, foo>, _> ... ok, <foo>, foo bar>, foo-> ... ng */
-	if (prefs_common.enable_color &&
-	    (p = strchr(buf, '>')) != NULL) {
-		if (p == buf || (p - buf < 25 &&
-		    memchr(buf, ' ' , p - buf) == NULL &&
-		    memchr(buf, '\t', p - buf) == NULL &&
-		    memchr(buf, '<' , p - buf) == NULL &&
-		    *(p - 1) != '-'))
-			fg_color = &quote_color;
-		else {
-			gchar *bp = buf;
+	/* change color of quotation
+	   >, foo>, _> ... ok, <foo>, foo bar>, foo-> ... ng
+	   Up to 3 levels of quotations are detected, and each
+	   level is colored using a different color. */
+	if (prefs_common.enable_color && strchr(buf, '>')) {
+		quotelevel = get_quote_level(buf);
 
-			for (; bp != p; bp++)
-				if (!isspace(*bp)) break;
-			if (bp == p)
-				fg_color = &quote_color;
+		/* set up the correct foreground color */
+		if (quotelevel > 2) {
+			/* recycle colors */
+			if (prefs_common.recycle_quote_colors)
+				quotelevel %= 3;
+			else
+				quotelevel = 2;
 		}
 	}
 
+	if (quotelevel == -1)
+		fg_color = NULL;
+	else
+		fg_color = &quote_colors[quotelevel];
+
 	if (prefs_common.head_space && spacingfont && buf[0] != '\n')
 		gtk_text_insert(text, spacingfont, NULL, NULL, " ", 1);
 
@@ -717,10 +762,6 @@ void textview_destroy(TextView *textview)
 	g_free(textview);
 }
 
-void textview_quote_color_set(void)
-{
-}
-
 void textview_set_font(TextView *textview, const gchar *codeset)
 {
 	gboolean use_fontset = TRUE;
@@ -906,6 +947,11 @@ void textview_scroll_one_line(TextView *textview, gboolean up)
 	GtkText *text = GTK_TEXT(textview->text);
 	gfloat upper;
 
+	if (prefs_common.enable_smooth_scroll) {
+		textview_smooth_scroll_one_line(textview, up);
+		return;
+	}
+
 	if (!up) {
 		upper = text->vadj->upper - text->vadj->page_size;
 		if (text->vadj->value < upper) {
@@ -933,6 +979,9 @@ gboolean textview_scroll_page(TextView *textview, gboolean up)
 	GtkText *text = GTK_TEXT(textview->text);
 	gfloat upper;
 
+	if (prefs_common.enable_smooth_scroll)
+		return textview_smooth_scroll_page(textview, up);
+
 	if (!up) {
 		upper = text->vadj->upper - text->vadj->page_size;
 		if (text->vadj->value < upper) {
@@ -955,6 +1004,107 @@ gboolean textview_scroll_page(TextView *textview, gboolean up)
 	return TRUE;
 }
 
+static void textview_smooth_scroll_do(TextView *textview,
+				      gfloat old_value, gfloat last_value,
+				      gint step)
+{
+	GtkText *text = GTK_TEXT(textview->text);
+	gint change_value;
+	gboolean up;
+	gint i;
+
+	if (old_value < last_value) {
+		change_value = last_value - old_value;
+		up = FALSE;
+	} else {
+		change_value = old_value - last_value;
+		up = TRUE;
+	}
+
+	gdk_key_repeat_disable();
+
+	for (i = step; i <= change_value; i += step) {
+		text->vadj->value = old_value + (up ? -i : i);
+		gtk_signal_emit_by_name(GTK_OBJECT(text->vadj),
+					"value_changed");
+	}
+
+	text->vadj->value = last_value;
+	gtk_signal_emit_by_name(GTK_OBJECT(text->vadj), "value_changed");
+
+	gdk_key_repeat_restore();
+}
+
+static void textview_smooth_scroll_one_line(TextView *textview, gboolean up)
+{
+	GtkText *text = GTK_TEXT(textview->text);
+	gfloat upper;
+	gfloat old_value;
+	gfloat last_value;
+
+	if (!up) {
+		upper = text->vadj->upper - text->vadj->page_size;
+		if (text->vadj->value < upper) {
+			old_value = text->vadj->value;
+			last_value = text->vadj->value +
+				text->vadj->step_increment * 4;
+			last_value = MIN(last_value, upper);
+
+			textview_smooth_scroll_do(textview, old_value,
+						  last_value,
+						  prefs_common.scroll_step);
+		}
+	} else {
+		if (text->vadj->value > 0.0) {
+			old_value = text->vadj->value;
+			last_value = text->vadj->value -
+				text->vadj->step_increment * 4;
+			last_value = MAX(last_value, 0.0);
+
+			textview_smooth_scroll_do(textview, old_value,
+						  last_value,
+						  prefs_common.scroll_step);
+		}
+	}
+}
+
+static gboolean textview_smooth_scroll_page(TextView *textview, gboolean up)
+{
+	GtkText *text = GTK_TEXT(textview->text);
+	gfloat upper;
+	gfloat old_value;
+	gfloat last_value;
+
+	if (!up) {
+		upper = text->vadj->upper - text->vadj->page_size;
+		if (text->vadj->value < upper) {
+			old_value = text->vadj->value;
+			last_value = text->vadj->value +
+				text->vadj->page_increment / 2;
+			last_value = MIN(last_value, upper);
+
+			textview_smooth_scroll_do(textview, old_value,
+						  last_value,
+						  prefs_common.scroll_step);
+		} else
+			return FALSE;
+	} else {
+		if (text->vadj->value > 0.0) {
+			old_value = text->vadj->value;
+			last_value = text->vadj->value -
+				text->vadj->page_increment / 2;
+			last_value = MAX(last_value, 0.0);
+
+			textview_smooth_scroll_do(textview, old_value,
+						  last_value,
+						  prefs_common.scroll_step);
+		} else
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
 static void textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 				 TextView *textview)
 {
