@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -408,6 +408,22 @@ wchar_t *wcscasestr(const wchar_t *haystack, const wchar_t *needle)
 	return NULL;
 }
 
+/* Examine if next block is non-ASCII string */
+gboolean is_next_nonascii(const wchar_t *s)
+{
+	const wchar_t *wp;
+
+	/* skip head space */
+	for (wp = s; *wp != (wchar_t)0 && iswspace(*wp); wp++)
+		;
+	for (; *wp != (wchar_t)0 && !iswspace(*wp); wp++) {
+		if (*wp > 127)
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
 /* Examine if next block is multi-byte string */
 gboolean is_next_mbs(const wchar_t *s)
 {
@@ -712,6 +728,36 @@ GSList *address_list_append(GSList *addr_list, const gchar *str)
 	return addr_list;
 }
 
+GSList *references_list_append(GSList *msgid_list, const gchar *str)
+{
+	const gchar *strp;
+
+	if (!str) return msgid_list;
+	strp = str;
+
+	while (strp && *strp) {
+		const gchar *start, *end;
+		gchar *msgid;
+
+		if ((start = strchr(strp, '<')) != NULL) {
+			end = strchr(start + 1, '>');
+			if (!end) break;
+		} else
+			break;
+
+		msgid = g_strndup(start + 1, end - start - 1);
+		g_strstrip(msgid);
+		if (*msgid)
+			msgid_list = g_slist_append(msgid_list, msgid);
+		else
+			g_free(msgid);
+
+		strp = end + 1;
+	}
+
+	return msgid_list;
+}
+
 GSList *newsgroup_list_append(GSList *group_list, const gchar *str)
 {
 	gchar *work;
@@ -808,6 +854,59 @@ gboolean is_ascii_str(const guchar *str)
 	return TRUE;
 }
 
+gint get_quote_level(const gchar *str)
+{
+	size_t firstquotepos;
+	size_t lastquotepos = -1;
+	const gchar *p = str;
+	const gchar *pos;
+	gint quotelevel = -1;
+	gint i = 0;
+
+	/* speed up line processing by only searching to the last '>' */
+	if ((pos = strchr(str, '>')) != NULL) {
+		firstquotepos = pos - str;
+		lastquotepos = strrchr(str, '>') - str + 1;
+
+		/* skip a line if it contains a '<' before the initial '>' */
+		if (memchr(str, '<', pos - str) != NULL)
+			return -1;
+	} else
+		return -1;
+
+	while (i < lastquotepos) {
+		while (i < lastquotepos) {
+			if (isspace(*p) || (*p == '\t')) {
+				p++;
+				i++;
+			} else
+				break;
+		}
+		if (i >= lastquotepos)
+			break;
+
+		if (*p == '>')
+			quotelevel++;
+		else if ((*p != '-') && !isspace(*p) && (i < lastquotepos)) {
+			/* any characters are allowed except '-' and space */
+			while ((*p != '-') && (*p != '>') && !isspace(*p) &&
+			       (i < lastquotepos)) {
+				p++;
+				i++;
+			}
+			if (*p == '>')
+				quotelevel++;
+			else if ((i >= lastquotepos) || isspace(*p))
+				break;
+		}
+
+		p++;
+		i++;
+	}
+
+	return quotelevel;
+}
+
 GList *uri_list_extract_filenames(const gchar *uri_list)
 {
 	GList *result = NULL;
@@ -1231,11 +1330,91 @@ gint remove_dir_recursive(const gchar *dir)
 	return 0;
 }
 
+#if 0
+/* this seems to be slower than the stdio version... */
+gint copy_file(const gchar *src, const gchar *dest)
+{
+	gint src_fd, dest_fd;
+	gint n_read;
+	gint n_write;
+	gchar buf[BUFSIZ];
+	gchar *dest_bak = NULL;
+
+	if ((src_fd = open(src, O_RDONLY)) < 0) {
+		FILE_OP_ERROR(src, "open");
+		return -1;
+	}
+
+	if (is_file_exist(dest)) {
+		dest_bak = g_strconcat(dest, ".bak", NULL);
+		if (rename(dest, dest_bak) < 0) {
+			FILE_OP_ERROR(dest, "rename");
+			close(src_fd);
+			g_free(dest_bak);
+			return -1;
+		}
+	}
+
+	if ((dest_fd = open(dest, O_RDWR|O_CREAT, S_IRUSR|S_IWUSR)) < 0) {
+		FILE_OP_ERROR(dest, "open");
+		close(src_fd);
+		if (dest_bak) {
+			if (rename(dest_bak, dest) < 0)
+				FILE_OP_ERROR(dest_bak, "rename");
+			g_free(dest_bak);
+		}
+		return -1;
+	}
+
+	while ((n_read = read(src_fd, buf, sizeof(buf))) > 0) {
+		gint len = n_read;
+		gchar *bufp = buf;
+
+		while (len > 0) {
+			n_write = write(dest_fd, bufp, len);
+			if (n_write <= 0) {
+				g_warning(_("writing to %s failed.\n"), dest);
+				close(dest_fd);
+				close(src_fd);
+				unlink(dest);
+				if (dest_bak) {
+					if (rename(dest_bak, dest) < 0)
+						FILE_OP_ERROR(dest_bak, "rename");
+					g_free(dest_bak);
+				}
+				return -1;
+			}
+			len -= n_write;
+			bufp += n_write;
+		}
+	}
+
+	close(src_fd);
+	close(dest_fd);
+
+	if (n_read < 0 || get_file_size(src) != get_file_size(dest)) {
+		g_warning(_("File copy from %s to %s failed.\n"), src, dest);
+		unlink(dest);
+		if (dest_bak) {
+			if (rename(dest_bak, dest) < 0)
+				FILE_OP_ERROR(dest_bak, "rename");
+			g_free(dest_bak);
+		}
+		return -1;
+	}
+	g_free(dest_bak);
+
+	return 0;
+}
+#endif
+
 gint copy_file(const gchar *src, const gchar *dest)
 {
 	FILE *src_fp, *dest_fp;
+	gint n_read;
 	gchar buf[BUFSIZ];
 	gchar *dest_bak = NULL;
+	gboolean err = FALSE;
 
 	if ((src_fp = fopen(src, "r")) == NULL) {
 		FILE_OP_ERROR(src, "fopen");
@@ -1250,6 +1429,7 @@ gint copy_file(const gchar *src, const gchar *dest)
 			return -1;
 		}
 	}
+
 	if ((dest_fp = fopen(dest, "w")) == NULL) {
 		FILE_OP_ERROR(dest, "fopen");
 		fclose(src_fp);
@@ -1266,8 +1446,10 @@ gint copy_file(const gchar *src, const gchar *dest)
 		g_warning(_("can't change file mode\n"));
 	}
 
-	while (fgets(buf, sizeof(buf), src_fp) != NULL) {
-		if (fputs(buf, dest_fp) == EOF) {
+	while ((n_read = fread(buf, sizeof(gchar), sizeof(buf), src_fp)) > 0) {
+		if (n_read < sizeof(buf) && ferror(src_fp))
+			break;
+		if (fwrite(buf, n_read, 1, dest_fp) < 1) {
 			g_warning(_("writing to %s failed.\n"), dest);
 			fclose(dest_fp);
 			fclose(src_fp);
@@ -1281,9 +1463,17 @@ gint copy_file(const gchar *src, const gchar *dest)
 		}
 	}
 
+	if (ferror(src_fp)) {
+		FILE_OP_ERROR(src, "fread");
+		err = TRUE;
+	}
 	fclose(src_fp);
 	if (fclose(dest_fp) == EOF) {
 		FILE_OP_ERROR(dest, "fclose");
+		err = TRUE;
+	}
+
+	if (err) {
 		unlink(dest);
 		if (dest_bak) {
 			if (rename(dest_bak, dest) < 0)
@@ -1292,6 +1482,7 @@ gint copy_file(const gchar *src, const gchar *dest)
 		}
 		return -1;
 	}
+
 	g_free(dest_bak);
 
 	return 0;
@@ -1430,6 +1621,99 @@ gint open_uri(const gchar *uri, const gchar *cmdline)
 	return 0;
 }
 
+time_t remote_tzoffset_sec(const gchar *zone)
+{
+	static gchar ustzstr[] = "PSTPDTMSTMDTCSTCDTESTEDT";
+	gchar zone3[4];
+	gchar *p;
+	gchar c;
+	gint iustz;
+	gint h, m;
+	time_t remoteoffset;
+
+	strncpy(zone3, zone, 3);
+	zone3[3] = '\0';
+	remoteoffset = 0;
+
+	if (sscanf(zone, "%c%2d%2d", &c, &h, &m) == 3 &&
+	    (c == '+' || c == '-')) {
+		remoteoffset = ((h * 60) + m) * 60;
+		if (c == '-')
+			remoteoffset = -remoteoffset;
+	} else if (!strncmp(zone, "UT" , 2) ||
+		   !strncmp(zone, "GMT", 2)) {
+		remoteoffset = 0;
+	} else if (strlen(zone3) == 3 &&
+		   (p = strstr(ustzstr, zone3)) != NULL &&
+		   (p - ustzstr) % 3 == 0) {
+		iustz = ((gint)(p - ustzstr) / 3 + 1) / 2 - 8;
+		remoteoffset = iustz * 3600;
+	} else if (strlen(zone3) == 1) {
+		switch (zone[0]) {
+		case 'Z': remoteoffset =   0; break;
+		case 'A': remoteoffset =  -1; break;
+		case 'B': remoteoffset =  -2; break;
+		case 'C': remoteoffset =  -3; break;
+		case 'D': remoteoffset =  -4; break;
+		case 'E': remoteoffset =  -5; break;
+		case 'F': remoteoffset =  -6; break;
+		case 'G': remoteoffset =  -7; break;
+		case 'H': remoteoffset =  -8; break;
+		case 'I': remoteoffset =  -9; break;
+		case 'K': remoteoffset = -10; break; /* J is not used */
+		case 'L': remoteoffset = -11; break;
+		case 'M': remoteoffset = -12; break;
+		case 'N': remoteoffset =   1; break;
+		case 'O': remoteoffset =   2; break;
+		case 'P': remoteoffset =   3; break;
+		case 'Q': remoteoffset =   4; break;
+		case 'R': remoteoffset =   5; break;
+		case 'S': remoteoffset =   6; break;
+		case 'T': remoteoffset =   7; break;
+		case 'U': remoteoffset =   8; break;
+		case 'V': remoteoffset =   9; break;
+		case 'W': remoteoffset =  10; break;
+		case 'X': remoteoffset =  11; break;
+		case 'Y': remoteoffset =  12; break;
+		default:  remoteoffset =   0; break;
+		}
+		remoteoffset = remoteoffset * 3600;
+	}
+
+	return remoteoffset;
+}
+
+time_t tzoffset_sec(time_t *now)
+{
+	struct tm gmt, *lt;
+	gint off;
+
+	gmt = *gmtime(now);
+	lt = localtime(now);
+
+	off = (lt->tm_hour - gmt.tm_hour) * 60 + lt->tm_min - gmt.tm_min;
+
+	if (lt->tm_year < gmt.tm_year)
+		off -= 24 * 60;
+	else if (lt->tm_year > gmt.tm_year)
+		off += 24 * 60;
+	else if (lt->tm_yday < gmt.tm_yday)
+		off -= 24 * 60;
+	else if (lt->tm_yday > gmt.tm_yday)
+		off += 24 * 60;
+
+	if (off >= 24 * 60)		/* should be impossible */
+		off = 23 * 60 + 59;	/* if not, insert silly value */
+	if (off <= -24 * 60)
+		off = -(23 * 60 + 59);
+	if (off > 12 * 60)
+		off -= 24 * 60;
+	if (off < -12 * 60)
+		off += 24 * 60;
+
+	return off * 60;
+}
+
 /* calculate timezone offset */
 gchar *tzoffset(time_t *now)
 {
@@ -1440,7 +1724,9 @@ gchar *tzoffset(time_t *now)
 
 	gmt = *gmtime(now);
 	lt = localtime(now);
+
 	off = (lt->tm_hour - gmt.tm_hour) * 60 + lt->tm_min - gmt.tm_min;
+
 	if (lt->tm_year < gmt.tm_year)
 		off -= 24 * 60;
 	else if (lt->tm_year > gmt.tm_year)
@@ -1449,15 +1735,18 @@ gchar *tzoffset(time_t *now)
 		off -= 24 * 60;
 	else if (lt->tm_yday > gmt.tm_yday)
 		off += 24 * 60;
+
 	if (off < 0) {
 		sign = '-';
 		off = -off;
 	}
+
 	if (off >= 24 * 60)		/* should be impossible */
 		off = 23 * 60 + 59;	/* if not, insert silly value */
+
 	sprintf(offset_string, "%c%02d%02d", sign, off / 60, off % 60);
 
-	return (offset_string);
+	return offset_string;
 }
 
 void get_rfc822_date(gchar *buf, gint len)
