@@ -142,18 +142,20 @@ wchar_t *wcsncpy	(wchar_t       *dest,
 			 size_t		n);
 #endif
 
-wchar_t *wcsdup		(const wchar_t *s);
-wchar_t *wcsndup	(const wchar_t *s,
-			 size_t		n);
-wchar_t *strdup_mbstowcs(const gchar   *s);
-gchar *strdup_wcstombs	(const wchar_t *s);
-gint wcsncasecmp	(const wchar_t *s1,
-			 const wchar_t *s2,
-			 size_t		n);
-wchar_t *wcscasestr	(const wchar_t *haystack,
-			 const wchar_t *needle);
-gboolean is_next_mbs	(const wchar_t *s);
-wchar_t *find_wspace	(const wchar_t *s);
+wchar_t *wcsdup			(const wchar_t *s);
+wchar_t *wcsndup		(const wchar_t *s,
+				 size_t		n);
+wchar_t *strdup_mbstowcs	(const gchar   *s);
+gchar *strdup_wcstombs		(const wchar_t *s);
+gint wcsncasecmp		(const wchar_t *s1,
+				 const wchar_t *s2,
+				 size_t		n);
+wchar_t *wcscasestr		(const wchar_t *haystack,
+				 const wchar_t *needle);
+
+gboolean is_next_nonascii	(const wchar_t *s);
+gboolean is_next_mbs		(const wchar_t *s);
+wchar_t *find_wspace		(const wchar_t *s);
 
 /* functions for string parsing */
 gint subject_compare			(const gchar	*s1,
@@ -183,6 +185,8 @@ gchar *strrchr_with_skip_quote		(const gchar	*str,
 void extract_address			(gchar		*str);
 GSList *address_list_append		(GSList		*addr_list,
 					 const gchar	*str);
+GSList *references_list_append		(GSList		*msgid_list,
+					 const gchar	*str);
 GSList *newsgroup_list_append		(GSList		*group_list,
 					 const gchar	*str);
 void remove_return			(gchar		*str);
@@ -192,6 +196,7 @@ void subst_char				(gchar		*str,
 					 gchar		 subst);
 gboolean is_header_line			(const gchar	*str);
 gboolean is_ascii_str			(const guchar	*str);
+gint get_quote_level			(const gchar	*str);
 GList *uri_list_extract_filenames	(const gchar	*uri_list);
 gchar *strstr_with_skip_quote		(const gchar	*haystack,
 					 const gchar	*needle);
@@ -233,9 +238,11 @@ gint execute_command_line	(const gchar	*cmdline);
 gint open_uri(const gchar *uri, const gchar *cmdline);
 
 /* time functions */
-gchar *tzoffset		(time_t	*now);
-void get_rfc822_date	(gchar	*buf,
-			 gint	 len);
+time_t remote_tzoffset_sec	(const gchar	*zone);
+time_t tzoffset_sec		(time_t		*now);
+gchar *tzoffset			(time_t		*now);
+void get_rfc822_date		(gchar		*buf,
+				 gint		 len);
 
 /* logging */
 void debug_print(const gchar *format, ...) G_GNUC_PRINTF(1, 2);
